/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import org.eclipse.core.runtime.IAdapterManager;

import com.ibm.team.repository.common.internal.util.InternalTeamPlatform;

/**
 * This class contains helper methods aimed at providing access to the adapter
 * functionality. This class can be used regardless if OSGi plugin is available
 * and will work in plain java clients.
 * <p>
 * The class is not intended to be subclassed or instantiated by clients.
 * </p>
 * 
 * @since 0.5
 */
public final class AdapterManagerFactory {

    /**
     * Prevent this class from being instantiated.
     */
    private AdapterManagerFactory() {
        // do nothing
    }

    /**
     * Returns the adapter manager used for extending <code>IAdaptable</code>
     * objects.
     * 
     * @return the adapter manager for the team platform
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public static IAdapterManager getAdapterManager() {
        return InternalTeamPlatform.getDefault().getAdapterManager();
    }
}
