/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * Adding this annotation to a Jazz service marks the service as allowing unauthenticated access to 
 * the service methods.
 * @since 0.7
 */
@Retention(RetentionPolicy.RUNTIME)
public @interface AllowAnonymousAccess {

    /**
     * The attemptOptionalAuthentication attribute is a hint to the servlet
     * whether authentication headers should be processed when handling services
     * with the AllowAnonymousAccess annotation.  The value defaults to false
     * to preserve backwards compatibility with code written before this attribute
     * was introduced.
     * <p>
     * If the value is false and the incoming request includes OAuth headers, 
     * then the headers will not be processed.  If the value is true and the 
     * incoming request includes OAuth headers, then the servlet will try to 
     * process the OAuth headers to discern an authenticated user.
     * </p>
     * @since 3.0.1.1
     */
    boolean attemptOptionalAuthentication() default false;
    
    /**
     * The fetch contributor attribute lets the servlet know if the contributor record needs to 
     * be fetched.  If true, the GUEST user is fetched from the contributor service and the servlet 
     * context is setup accordingly.  If false, no contributor is fetched and the team service context
     * will not include any information about the user.  This allows a request to not depend on the contributor 
     * service
     * 
     * @since 4.0
     * @return
     */
    boolean fetchContributor() default true;
}
