/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import com.ibm.team.repository.common.internal.nls.Messages;

/**
 * Exception indicating that a team repository operation failed because the
 * caller was using stale data.
 * 
 * @since 4.0.5
 */
public class ContextConflictException extends TeamRepositoryException {

    private static final long serialVersionUID = 1L;

    private static final String REPOSITORY_CONTEXT_CONFLICT = Messages.getServerString("ContextConflictException.ERROR_CONTEXT_CONFLICT"); //$NON-NLS-1$
        
    /**
     * Creates a new exception.
     */
    public ContextConflictException() {
        this(REPOSITORY_CONTEXT_CONFLICT);
    }

    /**
     * @param cause
     */
    public ContextConflictException(Throwable cause) {
        this(REPOSITORY_CONTEXT_CONFLICT, cause);
    }

    /**
     * Creates a new exception.
     */
    public ContextConflictException(String message) {
        super(message);
    }

    /**
     * @param origin
     * @param message
     * @param cause
     */
    public ContextConflictException(Object origin, String message, Throwable cause) {
        super(origin, message, cause);
    }

    /**
     * @param origin
     * @param message
     */
    public ContextConflictException(Object origin, String message) {
        super(origin, message);
    }

    /**
     * @param message
     * @param cause
     */
    public ContextConflictException(String message, Throwable cause) {
        super(message, cause);
    }

    /**
     * Creates a new exception.
     * @param message exception message
     * @param current Handle of the current state found in the database
     */
    public ContextConflictException(String message, IItemHandle data) {
        super(message);
        this.setData(data);
    }
}
