/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import java.sql.SQLException;

/**
 * When it is determined that the reason for a SQLException is that the 
 * DBMS detected a deadlock, the SQLException is wrapped in a
 * DeadlockVictimException, which is thrown after the transaction has
 * been rolled back (i.e. ended/aborted).
 * <p>
 * This allows the client to re-execute the server call that failed,
 * and/or allows the top-level server caller of runInTransaction to
 * call runInTransaction again with the same (or perhaps suitably
 * altered) arguments.
 * <p>
 * While it is likely that the DBMS aborted the transaction before the
 * SQLException was thrown, we don't assume that this is the case
 * and do explicitly rollback the transaction.  Server side code
 * should not interfere with the effort to propagate this exception
 * up to the top-level (outer-most) server caller of runInTransaction.
 *
 * @since 0.5
 * @deprecated RetryableDatabaseException should be used instead
 */
public class DeadlockVictimException extends RetryableDatabaseException {
    private static final long serialVersionUID = 1L;

    /* Note: while we want only the (String, SQLException) version
     * of the constructor, we need others to support the marshalling
     * layer.
     */

    public DeadlockVictimException() {
        super();
    }

    /**
     * @param message
     */
    public DeadlockVictimException(String message) {
        super(message);
    }

    /**
     * @param message
     * @param cause
     */
    public DeadlockVictimException(String message, SQLException cause) {
        super(message, cause);
    }

    /**
     * @param cause
     */
    public DeadlockVictimException(SQLException cause) {
        super(cause);
    }
}
