/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

/**
 * Represents an auditable item. Auditable items exist in multiple states in a
 * repository. There is a global notion of the current state of an auditable
 * item; states other than the current are used for an audit trail.
 * <p>
 * Item handles and items can only be created or retrieved from a repository
 * while the client is logged in. When the client logs out, all objects that
 * they obtained while logged in must be considered invalid. It is unspecified
 * whether these objects come back to life, so to speak, if the client logs back
 * in.
 * </p>
 * <p>
 * [TODO Issue: Locking for auditable items. Currently the client API does not
 * allow a client to lock or reserve an auditable item that they are about to
 * update. In situations where there may be a number of users vying to update
 * the same item simultaneously, it may become difficult for users to get work
 * done. The problem is magnified if there are large auditable item states where
 * each user retrieves and updates just a small subpart, because the platform
 * will have to treat these as conflicts even if there is an easy and automatic
 * way to handle most of them. The schemas for the standard auditable items are
 * all dead easy; it's likely that work items will push on this matter (cf.
 * Bugzilla "Mid-air collision"). We should be on the lookout for signs that we
 * will need to add locking sooner rather than later.]
 * </p>
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IAuditableHandle extends IManagedItemHandle {
}
