/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

/**
 * Expresses the difference in the state of an item between two auditable
 * baselines, referred to as the "prior" baseline and the "after" baseline. The
 * two auditable state handles returned by the methods below are guaranteed to
 * refer to different states of the same item. See
 * {@link IBaselineComparison} for more information about comparing two
 * auditable baselines.
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IBaselineMemberComparison {

    /**
     * Get the item being compared.
     * 
     * @return the handle, without state id, of the item; never
     *         <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public IAuditableHandle getItem();

    /**
     * Get the state of the item present in the prior baseline.
     * 
     * @return the state handle from the prior baseline; <code>null</code> if
     *         no state of the given item was present in the prior baseline
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public IAuditableHandle getPriorState();

    /**
     * Get the state of the item present in the prior baseline.
     * 
     * @return the state handle from the after baseline; <code>null</code> if
     *         no state of the given item was present in the after baseline
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public IAuditableHandle getAfterState();

}
