/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

/**
 * There are 4 access policies supported - Public, Protected, Application 
 * managed, and Internal. </p>
 * <p> Public access policy must be used for types that will be 
 * visible to everyone. For example, Contributor items must be accessible to
 * everyone. So, contributor type must use Public access policy. Items with 
 * public access policy would have its context set to {@link IContext#PUBLIC}.</p>
 * <p> Protected access policy must be used when the items must be visible only to 
 * the members of the context. The context of the item must be set to the project 
 * area id. All the members of the context (project area) can read the items. <p>
 * <p> Application managed context is used for items whose context is managed 
 * by the application. </p>
 * <p> Internal access policy is used for types that will be visible to noone.
 * These are typically types that have sensitive values that are used for server
 * side processing but should never leave the server. </p>
 * <p> You select the read access policy for a type by specifying the read
 * access policy attribute in the ecore model.  If no policy is specified, the
 * type will be Protected by default.</p>
 * @since 0.7
 * @see com.ibm.team.repository.service.permission
 */
public interface IContext {

    /**
     * Constant UUID used by the items that are accessible by everyone. All
     * the items with a public read access policy should set their context to
     * PUBLIC */
    public static final UUID PUBLIC = UUID.valueOf("_8lNyYNwSEd2pIJ5QVwgQGg"); //$NON-NLS-1$ 

    /**
     * Constant UUID used by items that are managed by the application.
     * Items with a Application Managed read access policy can set their context
     * to APPLICATION_MANAGED or to another value that has meaning to the
     * IComponentPermissionParticipant. */
    public static final UUID APPLICATION_MANAGED = UUID.valueOf("_Dp6kMdwTEd2jUupDpQV1Rw"); //$NON-NLS-1$
    
    public static final String PUBLIC_CONTEXT_URL = "_PUBLIC_CONTEXT"; //$NON-NLS-1$
}