/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import com.ibm.team.repository.common.model.RepositoryPackage;

/**
 * Represents a contributor extension known to the repository.
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IContributorDetails extends IExtensibleItem, IAuditable, IContributorDetailsHandle {

    /**
     * Contributor details item type.
     */
    public static final IItemType ITEM_TYPE = IItemType.IRegistry.INSTANCE
            .getItemType(RepositoryPackage.eINSTANCE.getContributorDetails()
                    .getName(), RepositoryPackage.eNS_URI);

    /**
     * Returns the content description for the photo.
     * 
     * @return the content description for the photo; or <code>null</code>
     * @LocalOp This is a local operation; the server/repository itself is not
     *          accessed.
     */
    public IContent getPhoto();

    /**
     * Sets the content description for the photo.
     * 
     * @param photo
     *            the content description for the photo; or <code>null</code>
     * @LocalOp This is a local operation; the server/repository itself is not
     *          accessed.
     */
    public void setPhoto(IContent photo);

}
