/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.repository.common;

import java.util.List;

import com.ibm.team.repository.common.model.RepositoryPackage;

public interface IContributorGroup extends IContributorGroupHandle, IAuditable {

    /**
     * Contributor item type.
     */
    public static final IItemType ITEM_TYPE = IItemType.IRegistry.INSTANCE
            .getItemType(
                    RepositoryPackage.eINSTANCE.getContributorGroup().getName(),
                    RepositoryPackage.eNS_URI);

    /**
     * Returns the name.
     * 
     * @return the name; never <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public String getName();

    /**
     * Sets the name.
     * 
     * @param name
     *            the name; must not be <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public void setName(String name);

    /**
     * Returns the summary for this contributor group.
     * 
     * @return the summary; never <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public String getSummary();

    /**
     * Sets the summary.
     * 
     * @param userid
     *            the summary; must not be <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public void setSummary(String summary);

    /**
     * Returns the description.
     * 
     * @return the description; never <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public String getDescription();

    /**
     * Sets the description.
     * 
     * @param description
     *            the description; must not be <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public void setDescription(String description);
    
    /**
     * Returns the list of members.
     * 
     * @return the list of members; never <code>null</code>, possibly empty
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public List getContributors();
}

