/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import com.ibm.team.repository.common.model.ContributorIdentity;
import com.ibm.team.repository.common.model.RepositoryFactory;
import com.ibm.team.repository.common.model.RepositoryPackage;

/**
 * Represents a contributor identity.
 * 
 * @since 0.7
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IContributorIdentity {
    public static final class ContributorIdentityFactory {
        ContributorIdentityFactory() {
            //disallow instantiation
        }
        
        /**
         * create a new IContributorIdentity object with the specified identity
         */
        public IContributorIdentity create(UUID identity) {
            ContributorIdentity ci = RepositoryFactory.eINSTANCE.createContributorIdentity();
            ci.setId(identity);
            return ci;
        }
    }
    
    /**
     * Factory to create IContributorIdentity objects
     */
    public static final ContributorIdentityFactory FACTORY = new ContributorIdentityFactory();

    /*
     * Property names supported by the contributor identity helper.
     */

    /**
     * The id property.
     * 
     * @see #getId()
     */
    public static final String ID_PROPERTY = RepositoryPackage.eINSTANCE
            .getContributorIdentity_Id().getName();

    /**
     * Returns the id.
     * 
     * @return the id; never <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public UUID getId();
}