/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import com.ibm.team.repository.common.model.ContributorUserId;
import com.ibm.team.repository.common.model.RepositoryFactory;
import com.ibm.team.repository.common.model.RepositoryPackage;

/**
 * Represents a contributor user id.
 * 
 * @since 0.7
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IContributorUserId {
    public static final class ContributorUserIdFactory {
        ContributorUserIdFactory() {
            //disallow instantiation
        }
        
        /**
         * create a new IContributorUserId object with the specified userId
         */
        public IContributorUserId create(String userId) {
            ContributorUserId cui = RepositoryFactory.eINSTANCE.createContributorUserId();
            cui.setUserId(userId);
            return cui;
        }
    }
    
    /**
     * Factory to create IContributorUserId objects
     */
    public static final ContributorUserIdFactory FACTORY = new ContributorUserIdFactory();
    
    /*
     * Property names supported by the contributor user id helper.
     */

    /**
     * The userId property.
     * 
     * @see #getUserId()
     */
    public static final String USERID_PROPERTY = RepositoryPackage.eINSTANCE
            .getContributorUserId_UserId().getName();

    /**
     * Returns the user id.
     * 
     * @return the user id; never <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public String getUserId();
}