/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;

/**
 * Transient object used for querying user info from an external registry (e.g., LDAP) 
 * and for importing that into a Jazz User.
 * 
 * @NoImplement
 * @since 0.6
 */
public interface IExternalUser {

    /**
     * Gets a full name for the user.  If the user has multiple full names in
     * the registry, it returns an arbitrary one.
     * @return a full name, or null if the user does not have any full names
     */
    String getFullName();
    
    /**
     * Gets an email address for the user.  If the user has multiple email
     * addresses in the registry, it returns an arbitrary one.
     * @return an email address, or null if the user does not have any
     */
    String getEmailAddress();
    
    /**
     * Get the user id of the user
     */
    String getUserId();

    /**
     * Get all of the full names for the user
     * @return a (possibly empty) List of the names.  Never null.
     */
    List<String> getFullNames();
    
    /**
     * Get all of the email addresses for the user
     * @return a (possibly empty) List of the email addresses.  Never null.
     */
    List<String> getEmailAddresses();
    
    /**
     * Returns a list of other fields, as customized for the 
     * environment (e.g., photo URL, manager, etc).
     * @return a key to value mapping of other fields
     */
    Map<String, String> getDetailFields();
    
    /**
     * Is user valid
     * @return
     */
    IStatus isValidUser();
    
    /**
     * @param propertiesToTest
     * @return
     */
    IStatus isValidUser(List<String> propertiesToTest);
} 