/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common;

import com.ibm.team.repository.common.internal.IFeedService;
import com.ibm.team.repository.common.model.Item;

/**
 * String constants used by the {@link IFeedService FeedService}, available
 * publically to clients.
 */
public interface IFeedServiceConstants {

    /**
     * The name of this service, to be used when referring to this service
     * as implemented on the server.
     * 
     * @see Location#getServiceName()
     */
    final public static String SERVICE_NAME = IFeedService.class.getName();
    
    /**
     * The base NS_PREFIX used by this service when creating custom
     * text or person elements.
     */
    static public final String NS_PREFIX = "syndication"; //$NON-NLS-1$
    
    /**
     * The base NS_URI used by this service to introduce NS_PREFIX in feed output
     * for custom text or person elements.
     */
    static public final String NS_URI = "http://www.ibm.com/jazz/0.1.0/syndication"; //$NON-NLS-1$
    
    /**
     * The base Activity Stream namespace
     */
    static public final String ACTIVITY_STREAM_NS_PREFIX = "http://activitystrea.ms/schema/1.0"; //$NON-NLS-1$

    /**
     * JSON object keys used to construct Activity Stream results
     */
    static public final String KEY_ID = "id"; //$NON-NLS-1$
    static public final String KEY_PUBLISHED = "published"; //$NON-NLS-1$
    static public final String KEY_TITLE = "title"; //$NON-NLS-1$
    static public final String KEY_ACTOR = "actor"; //$NON-NLS-1$
    static public final String KEY_VERB = "verb"; //$NON-NLS-1$
    static public final String KEY_CONTENT = "content"; //$NON-NLS-1$
    static public final String KEY_DISPLAYNAME = "displayName"; //$NON-NLS-1$
    static public final String KEY_OBJECT = "object"; //$NON-NLS-1$
    static public final String KEY_OBJECTTYPE = "objectType"; //$NON-NLS-1$
    static public final String KEY_SUMMARY = "summary"; //$NON-NLS-1$
    static public final String KEY_VALUE = "value"; //$NON-NLS-1$
    static public final String KEY_URL = "url"; //$NON-NLS-1$
    static public final String KEY_AUTHOR = "author"; //$NON-NLS-1$
    static public final String KEY_WIDTH = "width"; //$NON-NLS-1$
    static public final String KEY_HEIGHT = "height"; //$NON-NLS-1$
    static public final String KEY_IMAGE = "image"; //$NON-NLS-1$
    static public final String KEY_ITEMS = "items";//$NON-NLS-1$
    static public final String KEY_TARGET = "target"; //$NON-NLS-1$
    static public final String KEY_TOTALITEMS = "totalItems"; //$NON-NLS-1$
    static public final String KEY_SUGGESTEDVERBS = "suggestedVerbs"; //$NON-NLS-1$
    static public final String KEY_HYPERNYMS = "hypernyms"; //$NON-NLS-1$

    static public final String VERB_POST = "post"; //$NON-NLS-1$
    
    static public final String OBJECTTYPE_PERSON = "person"; //$NON-NLS-1$
    
    /**
     * Custom element names in the {@link #NS_URI} namespace, possibly included
     * in returned feed entries.
     */
    static public final String REPOSITORY_URL= "repositoryURL"; //$NON-NLS-1$
    static public final String REPOSITORY_ID= "repositoryId"; //$NON-NLS-1$
    static public final String MODIFIED= "modified"; //$NON-NLS-1$
    static public final String TYPE_NAMESPACE= "typeNamespace"; //$NON-NLS-1$
    static public final String TYPE_NAME= "typeName"; //$NON-NLS-1$
    static public final String ITEMID= "itemId"; //$NON-NLS-1$
    static public final String STATEID= "stateId"; //$NON-NLS-1$
    static public final String TARGET_URL= "targetUrl"; //$NON-NLS-1$
    static public final String TARGET_SUMMARY= "targetSummary"; //$NON-NLS-1$
    static public final String PUBLIC_URI="publicUri"; //$NON-NLS-1$
    static public final String REFTYPES= "reference_types"; //$NON-NLS-1$

    /**
     * The supported feed service query parameter name specifying the UUID 
     * identifier for the process area to query ChangeEvents for.  Legal values 
     * for this parameter are defined as {@link UUID#getUuidValue()}, as
     * available from {@link IItemHandle#getItemId()}.
     * <p>
     * This parameter may be present in the query string in the URI of the feed 
     * service, and is provided here as a possible key value in the map returned
     * by {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_PROCESS_AREA = "processArea"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter name specifying the type of
     * the requested feed.
     * <p>
     * This parameter may be present in the query string in the URI of the feed 
     * service, and is provided here as a possible key value in the map returned
     * by {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_FEEDTYPE = "feedType"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter name specifying the "id" 
     * attribute of the "feedItemQuery" extension point element, as defined by 
     * extensions of that extension point.
     * <p>
     * This parameter may be present in the query string in the URI of the feed 
     * service, and is provided here as a possible key value in the map returned
     * by {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_FEED_PROVIDER = "provider"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter name specifying the type of
     * {@link Item} to query ChangeEvents for.
     * <p>
     * This parameter may be present in the query string in the URI of the feed 
     * service, and is provided here as a possible key value in the map returned
     * by {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_ITEMTYPE = "itemType"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter name specifying the username
     * to query ChangeEvents for (as the value of {@link IContributor#getName()} associated
     * to the contributor that appears in either the author or contributor fields
     * of the ChangeEvent). Specifying this field as a query parameter is equivalent
     * to specifying both the {@link IFeedServiceConstants#QUERY_AUTHOR} and {@link IFeedServiceConstants#QUERY_CONTRIBUTOR} fields.
     * <p>
     * This parameter may be present in the query string in the URI of the feed
     * service, and is provided here as a possible key value in the map returned
     * by {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_USER = "user"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter name specifying the username to query
     * ChangeEvents for (as the value of {@link IContributor#getName()} associated
     * to the {@link IChangeEvent#AUTHOR_PROPERTY author} field of the ChangeEvent.
     * <p>
     * This parameter may be present in the query string in the URI of the feed 
     * service, and is provided here as a possible key value in the map returned by
     * {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_AUTHOR = "author"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter name specifying the username to query
     * ChangeEvents for (as the value of {@link IContributor#getName()} associated
     * to the {@link IChangeEvent#CONTRIBUTORS_PROPERTY contributors} field of the ChangeEvent.
     * <p>
     * This parameter may be present in the query string in the URI of the feed 
     * service, and is provided here as a possible key value in the map returned by
     * {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_CONTRIBUTOR = "contributor"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter specifying the time to query ChangeEvents
     * against.  If specified, the value of {@link IChangeEvent#getTime()} is 
     * compared against this value.  Only ChangeEvents whose time is equal to,
     * or more recent than the specified value are returned.  If specified,
     * this value must be in ISO8601 format. See {@link FeedUtils#getISO8601DateFormat()}.
     * <p>
     * This parameter may be present in the query string in the URI of the feed 
     * service, and is provided here as a possible key value in the map returned by
     * {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_SINCE = "since"; //$NON-NLS-1$
    
    /**
     * The supported feed service query parameter name specifying the maximum number of
     * feed entries to return.<p> 
     * 
     * This parameter may be present in the query string
     * in the URI of the feed service, and is provided here as a possible key
     * value in the map returned by {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_MAXRESULTS = "maxResults"; //$NON-NLS-1$

    /**
     * The supported feed service query parameter name specifying the category to query
     * ChangeEvents for (as found in the category field of the ChangeEvent).
     * <p>
     * This parameter may be present in the query string in the URI of the feed
     * service, and is provided here as a possible key value in the map returned
     * by {@link IFeedFactory#getFeedParameters()}.
     */
    static public final String QUERY_CATEGORY = "category"; //$NON-NLS-1$

}
