/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import java.util.List;

/**
 * The result of a multi-item or multi-state fetch method from
 * {@link com.ibm.team.repository.service.IRepositoryItemService}.
 * @since 0.7
 */
public interface IFetchResult {

    /** Return a list containing all items that the fetch call retrieved. */
    List getRetrievedItems();

    /** Return a list containing all items that the fetch call could not find */
    List getNotFoundItems();

    /** Return a list containing all of the items requested in the fetch for
     *  which the current user was denied permission to read. */
    List getPermissionDeniedItems();

    /**
     * Returns all missing items in a list. This is the union of the
     * permissionDeniedItems and the notFoundItems lists. Provided for backwards
     * compatibility: previous users of FetchResult would expect all items
     * missing for any reason to be in this list.
     */
    List getMissingItems();

    /** Returns true if any of the requested items to the fetch call were not
     *  found. */
    boolean hasNotFoundItems();

    /** Returns true if the user was denied permission to read any of the
     *  requested items. */
    boolean hasPermissionDeniedItems();

    /** Returns true if the fetch could not retrieve any of the requested items
     *  for any reason. */
    boolean hasMissingItems();
}
