/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import com.ibm.team.repository.common.model.FloatingSetupStatusDTO;
import com.ibm.team.repository.common.model.ILicensesInfo2;
import com.ibm.team.repository.common.model.UsedLicenseInfoDTO;

public interface ILicenseAdminService2 extends ILicenseAdminService {

    /**
     * Returns list of floating client access licenses types that
     * can be shared with other servers
     * 
     * @return a list of know client access license types
     */
    public IContributorLicenseType[] getFloatingLicenseTypes() throws TeamRepositoryException;
    
    /**
     * Adds the set of licenses to the server and returns the state of the
     * server after the add.  If submit is false then the licenses are not 
     * really added and the return value is the calculated value of what 
     * the server state <em>would</em> look like if submit had been true
     * 
     * @param localFiles the licenses to be added
     * @param submit whether to apply the licenses or not
     * @throws TeamRepositoryException if anything goes wrong
     */ 
    public ILicensesInfo2 addLicenses2(String[] localFiles, boolean submit, String[] operationIdsToCheck) throws TeamRepositoryException;

    /**
     * Returns the number of licenses available for the given license id.  This total is
     * the sum of the number of included licenses and the number of purchased licenses.
     * 
     * @param licenseId the id of the license
     * @return the total number of licenses
     */
    public int getTotalUserCount(String licenseId) throws TeamRepositoryException;

    /**
     * Returns information about the current license usage.
     * 
     * @return the license usage information
     */
    public UsedLicenseInfoDTO getUsedLicenseInfo() throws TeamRepositoryException;
    
    /**
     * Terminates the lease with the specified id.  
     * 
     * @param leaseId the lease to terminate.
     * @throws TeamRepositoryException 
     */
    public void terminateLease(String leaseId) throws TeamRepositoryException;

    /**
     * Returns the status of the floating license setup.
     * @return a status of the floating license setup
     */
    public FloatingSetupStatusDTO getFloatingSetupStatus() throws TeamRepositoryException;
    
    /**
     * Returns the token cost for the passed in policy id. Returns 0 if there is
     * none.
     * 
     * @param policyId
     * @return
     */
    public int getTokenCost(String policyId) throws TeamRepositoryException;
    
    /**
     * Returns the number of license assignments this floating license has
     * 
     * @param purchaseId
     * @return
     * @throws TeamRepositoryException
     */
    public IContributorLicenseType getContributorLicenseType(String purchaseId) throws TeamRepositoryException;

    void activateTrial(String licenseId) throws TeamRepositoryException;
    
    /**
     * Terminates the floating license leases associated with the specified user.
     * If the specified user is null, then the currently authenticated user is 
     * used.
     * Only a user with the JazzAdmins J2EE role is allowed to terminate the 
     * leases for other users.
     * 
     * @param handle the user to release license leases of, or null to use the 
     *                authenticated user. 
     * @throws TeamRepositoryException if something goes wrong.
     * @since 3.0
     */
    public void terminateLeases(IContributorHandle handle) throws TeamRepositoryException;

    /**
     * Tests whether a user is allowed to have the given licenses assigned 
     * at the same time.  If the assignments are allowed then this method will
     * complete without error.  If the licenses cannot be assigned together, 
     * then an TeamRepositoryException will be thrown to explain the problem.
     *  
     * @param param the licenses to assign together
     * @throws TeamRepositoryException if the licenses are not valid to be
     *          assigned together.
     * @since 3.0
     */    
    public void testLicenseAssignments(String[] licenseIds) throws TeamRepositoryException;
    
}
