/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;


/**
 * The results of assigning a license. Used to express error, warning, or info messages back to the caller
 */
public interface ILicenseAssignmentResult {
    public final static int OK = 0x0;

    public final static int ERROR = 0x1;

    public final static int WARNING = 0x2;
	
    /**
     * Returns a status indicating that the assignment was a success, failure,
     * or a warning
     * 
     * @return
     */
    public int getStatus();

	/**
	 * A message to send back to the client. May be <code>null</code>
	 * 
	 * @return
	 */
	public String getMessage();

	/**
	 * Answers whether or not the result is a warning
	 * 
	 * @return
	 */
	public boolean isWarning();

	/**
	 * Answers whether or not the result is an error
	 * 
	 * @return
	 */
	public boolean isError();

	/**
	 * Answers whether or not the result is a success or ok
	 * 
	 * @return
	 */
	public boolean isOK();
}
