/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import java.sql.Timestamp;
import java.util.List;

import com.ibm.team.repository.common.model.LicenseNoticeDTO;

public interface ILicenseType {

    /**
     * Returns an ID for this license type that can be used in various
     * methods on {@link ILicenseAdminService}
     * 
     * @return an ID for this license type.
     */
    public String getId();
    
    /**
     * Returns the product name that is responsible for contributing this license
     * type. 
     * 
     * @return the product name that is responsible for contributing this license
     *         type. 
     */
    public String getProductName();
    
    /**
     * Returns the product edition name that this license was created for.
     * 
     * @return the product edition that this license was created for or null
     *         if no particular edition was specified.
     */
    public String getEditionName();
    
    /**
     * Returns the product variant name that this license was created for.  When displayed to the 
	 * user, the product variant is less significant than the product name and edition.  A possible
	 * use for the variant field is to explain the trial period.
	 * 
	 * variant would be the trial count
     * 
     * @return the product variant that this license was created for or null
     *         if no particular variant was specified.
     */    
    public String getVariantName();
    
    /**
     * Returns a description of this license type that could be presented to
     * to the user.
     * 
     * @return a description of this license type.
     */
    public String getDescription();
    
    /**
     * Returns a link that could be presented to a user to find information about
     * purchasing this license type.  
     * 
     * @return a link to more information about purchasing, or null
     *         if no link has been provided.
     */
    public String getBuyURL();
    
    /**
     * Returns a link that could be presented to a user to find information about
     * this license type.  
     * 
     * @return a link to more information about this type, or null
     *         if no link has been provided.
     */
    public String getInfoURL();
    
    /**
     * Returns a reason if this license type has been deactivated.  If null is returned then
     * this license type is still active.
     * 
     * @return null if the license type is active, and non-null reason if this license type is 
     *         deactivated.
     */
    public String getDisabledReason();
    
    /**
     * Returns the date when the license trial will expire.  This method
     * will return null if the license is permanent.
     * 
     * @return the date when the license trial will expire or null.
     */
    public Timestamp getExpirationTime();
    
    public boolean isTrial();
    
    // public String getLicenseText();
    
    public List<LicenseNoticeDTO> getNotices();
}
