/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

import com.ibm.team.repository.common.model.RepositoryPackage;

/**
 * Represents the root of the repository.
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IRepositoryRoot extends IRepositoryRootHandle, IAuditable {

    /**
     * Repository root item type.
     */
    public static final IItemType ITEM_TYPE = IItemType.IRegistry.INSTANCE
            .getItemType(RepositoryPackage.eINSTANCE.getRepositoryRoot()
                    .getName(), RepositoryPackage.eNS_URI);

    /*
     * Property names supported by the repository root item type.
     */

    /**
     * The name property.
     * 
     * @see #getName()
     */
    public static final String NAME_PROPERTY = RepositoryPackage.eINSTANCE
            .getRepositoryRoot_Name().getName();

    /**
     * Returns the repository name.
     * 
     * @return the name; never <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public String getName();

    /**
     * Sets the repository name.
     * 
     * @param name
     *            the name; never <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public void setName(String name);
    
    /**
     * Returns true if the RepositoryRoot is marked in production mode
     * 
     * @return a boolean
     */
    public boolean isProductionMode();

}
