/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common;

/**
 * Base class for addressable resource types
 */
public interface IResource {
    
    /**
     * Returns a location that can identify this resource to the
     * server.  Derivations are required to provide an implementation for this that
     * is compatible with {@link Location}.
     * 
     * @return The location for identifying this resource. Must not be <code>null</code>.
     */
    public String getLocation();
    
    /**
     * Returns a unique identifier for this resource that can distinquish a
     * state of this resource instance from any other state of this resource.
     * 
     * @return An identifier that distinquishes a state of this resource from all
     * others.  May be <code>null</code> if derived resources don't make that
     * distinction.
     */
    public String getEtag();
    
}
