/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

/**
 * Represents an unmanaged item. Unmanaged items do not have a global notion of the
 * current state, do not have basic CRUD service available at the repo API layer, and
 * do not have query models. Generally these are not used directly by clients, but rather
 * it is intended to be subclasses for source control and configuration management components.
 * <p>
 * Versionable items (such as file and folders) are
 * created and manipulated in workspaces. 
 * Files and folders are the standard versionable item types. 
 * <p>
 * Item handles and items are passive data objects that are retrieved or created
 * while the client is logged in to a repository. These objects remain valid if
 * the client logs out.
 * </p>
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IUnmanagedItem extends IItem, IUnmanagedItemHandle {

}
