/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common;

/**
 * Represents a versionable item. Versionable items (such as file and folders)
 * are created and manipulated in workspaces. There is no global notion of the
 * current state of a versionable item; only a notion of current state relative
 * to a workspace (or similar context).
 * <p>
 * Files and folders are the standard versionable item types. Repository
 * extensions may define new versionable item types (although not through this
 * client API). Each repository has a registry of the versionable item types it
 * knows about. See <code>ITeamRepository.itemTypeRegistry()</code>.
 * </p>
 * <p>
 * Item handles and items can only be created or retrieved from a repository
 * while the client is logged in. When the client logs out, all objects that
 * they obtained while logged in must be considered invalid. It is unspecified
 * whether these objects come back to life, so to speak, if the client logs back
 * in.
 * </p>
 * 
 * @since 0.5
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IUnmanagedItemHandle extends IItemHandle {
}
