/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.AbstractEnumerator;

/**
 * Type of Marshaller
 * @model
 */
public class MarshallerType extends AbstractEnumerator {

    public MarshallerType(int value, String name) {
        super(value, name);
    }
    
    /**
     * @model
     */
    public static final int PRIMITIVE = 1;
    
    /**
     * @model
     */
    public static final int COMPLEX = 2;
    
    /**
     * @model
     */
    public static final int VOID = 3;
    
    /**
     * @model
     */
    public static final int CHARACTER = 4;
    
    /**
     * @model
     */
    public static final int BYTE = 5;
    
    /**
     * @model
     */
    public static final int STRING = 6;
    
    /**
     * @model
     */
    public static final int INTEGER = 7;
    
    /**
     * @model
     */
    public static final int SHORT = 8;
    
    /**
     * @model
     */
    public static final int LONG = 9;
    
    /**
     * @model
     */
    public static final int FLOAT = 10;
    
    /**
     * @model
     */
    public static final int DOUBLE = 11;
    
    /**
     * @model
     */
    public static final int BOOLEAN = 12;
    
    /**
     * @model
     */
    public static final int TIMESTAMP = 13;
    
    /**
     * @model
     */
    public static final int UUID = 14;
    
    /**
     * @model
     */
    public static final int ENUMERATOR = 15;

    /**
     * @model
     */
    public static final int EOBJECT = 16;

    /**
     * @model
     */
    public static final int JSON = 17;

    /**
     * @model
     */
    public static final int WEB_SERVICES = 18;

    /**
     * @model
     */
    public static final int MIGRATION = 19;

    /**
     * @model
     */
    public static final int ARRAY = 20;

    /**
     * @model
     */
    public static final int PRIMITIVE_ARRAY = 21;

    /**
     * @model
     */
    public static final int COMPLEX_ARRAY = 22;

    /**
     * @model 
     */
    public static final int OBJECT_ARRAY = 23;

    /**
     * @model
     */
    public static final int UUID_ARRAY = 24;

    /**
     * @model
     */
    public static final int BIG_DECIMAL = 25;

    /**
     * The '<em><b>INTEGER ARRAY</b></em>' literal value.
     * <!-- begin-user-doc -->
     * <p>
     * If the meaning of '<em><b>INTEGER ARRAY</b></em>' literal object isn't clear,
     * there really should be more of a description here...
     * </p>
     * <!-- end-user-doc -->
     * @see #INTEGER_ARRAY_LITERAL
     * @model
     * @generated
     * @ordered
     */
    public static final int INTEGER_ARRAY = 26;
    
    
    /**
     * The '<em><b>PRIMITIVE</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #PRIMITIVE
     * @generated
     * @ordered
     */
    public static final MarshallerType PRIMITIVE_LITERAL = new MarshallerType(PRIMITIVE, "PRIMITIVE", "PRIMITIVE"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>COMPLEX</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #COMPLEX
     * @generated
     * @ordered
     */
    public static final MarshallerType COMPLEX_LITERAL = new MarshallerType(COMPLEX, "COMPLEX", "COMPLEX"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>VOID</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #VOID
     * @generated
     * @ordered
     */
    public static final MarshallerType VOID_LITERAL = new MarshallerType(VOID, "VOID", "VOID"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>CHARACTER</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #CHARACTER
     * @generated
     * @ordered
     */
    public static final MarshallerType CHARACTER_LITERAL = new MarshallerType(CHARACTER, "CHARACTER", "CHARACTER"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>BYTE</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #BYTE
     * @generated
     * @ordered
     */
    public static final MarshallerType BYTE_LITERAL = new MarshallerType(BYTE, "BYTE", "BYTE"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>STRING</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #STRING
     * @generated
     * @ordered
     */
    public static final MarshallerType STRING_LITERAL = new MarshallerType(STRING, "STRING", "STRING"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>INTEGER</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #INTEGER
     * @generated
     * @ordered
     */
    public static final MarshallerType INTEGER_LITERAL = new MarshallerType(INTEGER, "INTEGER", "INTEGER"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>SHORT</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #SHORT
     * @generated
     * @ordered
     */
    public static final MarshallerType SHORT_LITERAL = new MarshallerType(SHORT, "SHORT", "SHORT"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>LONG</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #LONG
     * @generated
     * @ordered
     */
    public static final MarshallerType LONG_LITERAL = new MarshallerType(LONG, "LONG", "LONG"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>FLOAT</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #FLOAT
     * @generated
     * @ordered
     */
    public static final MarshallerType FLOAT_LITERAL = new MarshallerType(FLOAT, "FLOAT", "FLOAT"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>DOUBLE</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #DOUBLE
     * @generated
     * @ordered
     */
    public static final MarshallerType DOUBLE_LITERAL = new MarshallerType(DOUBLE, "DOUBLE", "DOUBLE"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>BOOLEAN</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #BOOLEAN
     * @generated
     * @ordered
     */
    public static final MarshallerType BOOLEAN_LITERAL = new MarshallerType(BOOLEAN, "BOOLEAN", "BOOLEAN"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>TIMESTAMP</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #TIMESTAMP
     * @generated
     * @ordered
     */
    public static final MarshallerType TIMESTAMP_LITERAL = new MarshallerType(TIMESTAMP, "TIMESTAMP", "TIMESTAMP"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>UUID</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #UUID
     * @generated
     * @ordered
     */
    public static final MarshallerType UUID_LITERAL = new MarshallerType(UUID, "UUID", "UUID"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>ENUMERATOR</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #ENUMERATOR
     * @generated
     * @ordered
     */
    public static final MarshallerType ENUMERATOR_LITERAL = new MarshallerType(ENUMERATOR, "ENUMERATOR", "ENUMERATOR"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>EOBJECT</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #EOBJECT
     * @generated
     * @ordered
     */
    public static final MarshallerType EOBJECT_LITERAL = new MarshallerType(EOBJECT, "EOBJECT", "EOBJECT"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>JSON</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #JSON
     * @generated
     * @ordered
     */
    public static final MarshallerType JSON_LITERAL = new MarshallerType(JSON, "JSON", "JSON"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>WEB SERVICES</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #WEB_SERVICES
     * @generated
     * @ordered
     */
    public static final MarshallerType WEB_SERVICES_LITERAL = new MarshallerType(WEB_SERVICES, "WEB_SERVICES", "WEB_SERVICES"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>MIGRATION</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #MIGRATION
     * @generated
     * @ordered
     */
    public static final MarshallerType MIGRATION_LITERAL = new MarshallerType(MIGRATION, "MIGRATION", "MIGRATION"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>ARRAY</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #ARRAY
     * @generated
     * @ordered
     */
    public static final MarshallerType ARRAY_LITERAL = new MarshallerType(ARRAY, "ARRAY", "ARRAY"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>PRIMITIVE ARRAY</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #PRIMITIVE_ARRAY
     * @generated
     * @ordered
     */
    public static final MarshallerType PRIMITIVE_ARRAY_LITERAL = new MarshallerType(PRIMITIVE_ARRAY, "PRIMITIVE_ARRAY", "PRIMITIVE_ARRAY"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>COMPLEX ARRAY</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #COMPLEX_ARRAY
     * @generated
     * @ordered
     */
    public static final MarshallerType COMPLEX_ARRAY_LITERAL = new MarshallerType(COMPLEX_ARRAY, "COMPLEX_ARRAY", "COMPLEX_ARRAY"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>OBJECT ARRAY</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #OBJECT_ARRAY
     * @generated
     * @ordered
     */
    public static final MarshallerType OBJECT_ARRAY_LITERAL = new MarshallerType(OBJECT_ARRAY, "OBJECT_ARRAY", "OBJECT_ARRAY"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>UUID ARRAY</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #UUID_ARRAY
     * @generated
     * @ordered
     */
    public static final MarshallerType UUID_ARRAY_LITERAL = new MarshallerType(UUID_ARRAY, "UUID_ARRAY", "UUID_ARRAY"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>BIG DECIMAL</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #BIG_DECIMAL
     * @generated
     * @ordered
     */
    public static final MarshallerType BIG_DECIMAL_LITERAL = new MarshallerType(BIG_DECIMAL, "BIG_DECIMAL", "BIG_DECIMAL"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>INTEGER ARRAY</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #INTEGER_ARRAY
     * @generated
     * @ordered
     */
    public static final MarshallerType INTEGER_ARRAY_LITERAL = new MarshallerType(INTEGER_ARRAY, "INTEGER_ARRAY", "INTEGER_ARRAY"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * An array of all the '<em><b>Marshaller Type</b></em>' enumerators.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    private static final MarshallerType[] VALUES_ARRAY =
        new MarshallerType[] {
            PRIMITIVE_LITERAL,
            COMPLEX_LITERAL,
            VOID_LITERAL,
            CHARACTER_LITERAL,
            BYTE_LITERAL,
            STRING_LITERAL,
            INTEGER_LITERAL,
            SHORT_LITERAL,
            LONG_LITERAL,
            FLOAT_LITERAL,
            DOUBLE_LITERAL,
            BOOLEAN_LITERAL,
            TIMESTAMP_LITERAL,
            UUID_LITERAL,
            ENUMERATOR_LITERAL,
            EOBJECT_LITERAL,
            JSON_LITERAL,
            WEB_SERVICES_LITERAL,
            MIGRATION_LITERAL,
            ARRAY_LITERAL,
            PRIMITIVE_ARRAY_LITERAL,
            COMPLEX_ARRAY_LITERAL,
            OBJECT_ARRAY_LITERAL,
            UUID_ARRAY_LITERAL,
            BIG_DECIMAL_LITERAL,
            INTEGER_ARRAY_LITERAL,
        };

    /**
     * A public read-only list of all the '<em><b>Marshaller Type</b></em>' enumerators.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static final List VALUES = Collections.unmodifiableList(Arrays.asList(VALUES_ARRAY));

    /**
     * Returns the '<em><b>Marshaller Type</b></em>' literal with the specified literal value.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static MarshallerType get(String literal) {
        for (int i = 0; i < VALUES_ARRAY.length; ++i) {
            MarshallerType result = VALUES_ARRAY[i];
            if (result.toString().equals(literal)) {
                return result;
            }
        }
        return null;
    }

    /**
     * Returns the '<em><b>Marshaller Type</b></em>' literal with the specified name.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static MarshallerType getByName(String name) {
        for (int i = 0; i < VALUES_ARRAY.length; ++i) {
            MarshallerType result = VALUES_ARRAY[i];
            if (result.getName().equals(name)) {
                return result;
            }
        }
        return null;
    }

    /**
     * Returns the '<em><b>Marshaller Type</b></em>' literal with the specified integer value.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static MarshallerType get(int value) {
        switch (value) {
            case PRIMITIVE: return PRIMITIVE_LITERAL;
            case COMPLEX: return COMPLEX_LITERAL;
            case VOID: return VOID_LITERAL;
            case CHARACTER: return CHARACTER_LITERAL;
            case BYTE: return BYTE_LITERAL;
            case STRING: return STRING_LITERAL;
            case INTEGER: return INTEGER_LITERAL;
            case SHORT: return SHORT_LITERAL;
            case LONG: return LONG_LITERAL;
            case FLOAT: return FLOAT_LITERAL;
            case DOUBLE: return DOUBLE_LITERAL;
            case BOOLEAN: return BOOLEAN_LITERAL;
            case TIMESTAMP: return TIMESTAMP_LITERAL;
            case UUID: return UUID_LITERAL;
            case ENUMERATOR: return ENUMERATOR_LITERAL;
            case EOBJECT: return EOBJECT_LITERAL;
            case JSON: return JSON_LITERAL;
            case WEB_SERVICES: return WEB_SERVICES_LITERAL;
            case MIGRATION: return MIGRATION_LITERAL;
            case ARRAY: return ARRAY_LITERAL;
            case PRIMITIVE_ARRAY: return PRIMITIVE_ARRAY_LITERAL;
            case COMPLEX_ARRAY: return COMPLEX_ARRAY_LITERAL;
            case OBJECT_ARRAY: return OBJECT_ARRAY_LITERAL;
            case UUID_ARRAY: return UUID_ARRAY_LITERAL;
            case BIG_DECIMAL: return BIG_DECIMAL_LITERAL;
            case INTEGER_ARRAY: return INTEGER_ARRAY_LITERAL;
        }
        return null;
    }

    /**
     * Only this class can construct instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    private MarshallerType(int value, String name, String literal) {
        super(value, name, literal);
    }

}
