/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Marshalling Exception
 */
public class MarshallingException extends Exception {
    
    private static final long serialVersionUID = -7068599227032836423L;

    // Exception Reason Code
    private MarshallingExceptionReasonCode marshallingExceptionReasonCode_;
    
    // nested exceptions
    private Collection<Exception> nestedExceptions_ = new ArrayList<Exception>(1);
    
    public MarshallingException(MarshallingExceptionReasonCode marshallingExceptionReasonCode) {
        super();
        marshallingExceptionReasonCode_ = marshallingExceptionReasonCode;
    }

    public MarshallingException(
            MarshallingExceptionReasonCode marshallingExceptionReasonCode,
            String arg0) {
        super(arg0);
        marshallingExceptionReasonCode_ = marshallingExceptionReasonCode;
    }

    public MarshallingException(
            MarshallingExceptionReasonCode marshallingExceptionReasonCode,
            String arg0, Throwable arg1) {
        super(arg0, arg1);
        marshallingExceptionReasonCode_ = marshallingExceptionReasonCode;
    }

    public MarshallingException(Throwable arg0) {
        super(arg0);
    }
    
    public Collection getNestedExceptions() {
        return nestedExceptions_;
    }
    
    public void addNestedExceptions(Exception exception) {
        if(!nestedExceptions_.contains(exception)) {
            nestedExceptions_.add(exception);
        }
    }
    
    public void removeNestedExceptions(Exception exception) {
        nestedExceptions_.remove(exception);
    }
    
    /**
     * Set reason code
     * @param marshallingExceptionReasonCode
     */
    public void setReasonCode(MarshallingExceptionReasonCode marshallingExceptionReasonCode) {
        marshallingExceptionReasonCode_ = marshallingExceptionReasonCode;
    }
    
    /**
     * Get the marshalling exception reason code
     * @return
     */
    public MarshallingExceptionReasonCode getReasonCode() {
        return marshallingExceptionReasonCode_;
    }
}
