/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.AbstractEnumerator;

/**
 * Marshalling Exception Reason Code
 * @model
 */
public class MarshallingExceptionReasonCode extends AbstractEnumerator {

    public MarshallingExceptionReasonCode(int value, String name) {
        super(value, name);
    }
    
    /**
     * Cannot Marshal the object
     * @model
     */
    public static final int CANNOT_MARSHAL = 1;
    
    /**
     * Marshaller Supported Type is null.
     * @model
     */
    public static final int NULL_SUPPORTED_TYPE = 2;
    
    /**
     * @model 
     */
    public static final int CANNOT_DEMARSHAL = 3;
    /**
     * The '<em><b>CANNOT MARSHAL</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #CANNOT_MARSHAL
     * @generated
     * @ordered
     */
    public static final MarshallingExceptionReasonCode CANNOT_MARSHAL_LITERAL = new MarshallingExceptionReasonCode(CANNOT_MARSHAL, "CANNOT_MARSHAL", "CANNOT_MARSHAL"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>NULL SUPPORTED TYPE</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #NULL_SUPPORTED_TYPE
     * @generated
     * @ordered
     */
    public static final MarshallingExceptionReasonCode NULL_SUPPORTED_TYPE_LITERAL = new MarshallingExceptionReasonCode(NULL_SUPPORTED_TYPE, "NULL_SUPPORTED_TYPE", "NULL_SUPPORTED_TYPE"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * The '<em><b>CANNOT DEMARSHAL</b></em>' literal object.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @see #CANNOT_DEMARSHAL
     * @generated
     * @ordered
     */
    public static final MarshallingExceptionReasonCode CANNOT_DEMARSHAL_LITERAL = new MarshallingExceptionReasonCode(CANNOT_DEMARSHAL, "CANNOT_DEMARSHAL", "CANNOT_DEMARSHAL"); //$NON-NLS-1$ //$NON-NLS-2$

    /**
     * An array of all the '<em><b>Marshalling Exception Reason Code</b></em>' enumerators.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    private static final MarshallingExceptionReasonCode[] VALUES_ARRAY =
        new MarshallingExceptionReasonCode[] {
            CANNOT_MARSHAL_LITERAL,
            NULL_SUPPORTED_TYPE_LITERAL,
            CANNOT_DEMARSHAL_LITERAL,
        };

    /**
     * A public read-only list of all the '<em><b>Marshalling Exception Reason Code</b></em>' enumerators.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static final List VALUES = Collections.unmodifiableList(Arrays.asList(VALUES_ARRAY));

    /**
     * Returns the '<em><b>Marshalling Exception Reason Code</b></em>' literal with the specified literal value.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static MarshallingExceptionReasonCode get(String literal) {
        for (int i = 0; i < VALUES_ARRAY.length; ++i) {
            MarshallingExceptionReasonCode result = VALUES_ARRAY[i];
            if (result.toString().equals(literal)) {
                return result;
            }
        }
        return null;
    }

    /**
     * Returns the '<em><b>Marshalling Exception Reason Code</b></em>' literal with the specified name.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static MarshallingExceptionReasonCode getByName(String name) {
        for (int i = 0; i < VALUES_ARRAY.length; ++i) {
            MarshallingExceptionReasonCode result = VALUES_ARRAY[i];
            if (result.getName().equals(name)) {
                return result;
            }
        }
        return null;
    }

    /**
     * Returns the '<em><b>Marshalling Exception Reason Code</b></em>' literal with the specified integer value.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static MarshallingExceptionReasonCode get(int value) {
        switch (value) {
            case CANNOT_MARSHAL: return CANNOT_MARSHAL_LITERAL;
            case NULL_SUPPORTED_TYPE: return NULL_SUPPORTED_TYPE_LITERAL;
            case CANNOT_DEMARSHAL: return CANNOT_DEMARSHAL_LITERAL;
        }
        return null;
    }

    /**
     * Only this class can construct instances.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    private MarshallingExceptionReasonCode(int value, String name, String literal) {
        super(value, name, literal);
    }

}
