/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;

import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.Iterator;
import java.util.List;

import com.ibm.team.repository.common.LogFactory;
import com.ibm.team.repository.common.internal.marshal.MarshalFactory;
import com.ibm.team.repository.common.internal.marshal.Marshaller;
import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.serialize.internal.nls.Messages;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.NullDataArg;
import com.ibm.team.repository.common.transport.internal.services.ObjectArrayDataArg;
import com.ibm.team.repository.common.transport.internal.services.ServicesFactory;
import com.ibm.team.repository.common.util.NLS;

/**
 * BooleanArrayMarshaller to marshal and demarshal a Boolean array
 */

public class BooleanArrayMarshaller extends MarshallerImpl {

	public BooleanArrayMarshaller() {
		super();
		setSupportedType(Boolean.TYPE);
		setMarshallerType(MarshallerType.OBJECT_ARRAY_LITERAL);
	}

	@Override
	public DataArgType getSupportedDataArgType() {
		return DataArgType.OBJECT_ARRAY_LITERAL;
	}

	@Override
	public boolean canMarshal(Class type) {
		if (type.isArray()) {
			return Boolean.class.equals(type.getComponentType());
		}
		return false;
	}

	public void marshalObjectToOutputStream(Object object, OutputStream outputStream) throws MarshallingException {
		throw new UnsupportedOperationException();
	}

	public void marshalObjectToOutputStream(Object object, OutputStream outputStream, Charset charset) throws MarshallingException {
		throw new UnsupportedOperationException();
	}

	public DataArgType getDataArgType() {
		return DataArgType.OBJECT_ARRAY_LITERAL;
	}

	public String marshalObjectToString(Object object) throws MarshallingException {
		throw new UnsupportedOperationException();
	}

	public Object demarshalInputStreamToObject(InputStream inputStream) throws MarshallingException {
		throw new UnsupportedOperationException();
	}

	public Object demarshalInputStreamToObject(InputStream inputStream, Charset charset) throws MarshallingException {
		throw new UnsupportedOperationException();
	}

	@Override
	public DataArg marshalObjectToDataArg(Object object) {
		if (object == null) {
			return createNullDataArg();
		}
		ObjectArrayDataArg objectArrayDataArg = ServicesFactory.eINSTANCE.createObjectArrayDataArg();
		objectArrayDataArg.setType(DataArgType.OBJECT_ARRAY_LITERAL);

		Boolean[] objectArray = (Boolean[]) object;

		//BooleanMarshaller is registered on Boolean.TYPE (boolean) not Boolean.class
		Marshaller marshaller = MarshalFactory.eINSTANCE.getMarshaller(Boolean.TYPE);
		if (marshaller == null) {
			String pattern = Messages.getServerString("BooleanArrayMarshaller.CannotMarshalObject"); //$NON-NLS-1$
			String message = NLS.bind(pattern, objectArray.length > 0 ? objectArray[0] : objectArray);
			LogFactory.getLog(getClass().getName()).warn(message);
			
			String clientPattern = Messages.getServerString("BooleanArrayMarshaller.CannotMarshalObject"); //$NON-NLS-1$
			String clientMessage = NLS.bind(clientPattern, objectArray.length > 0 ? objectArray[0] : objectArray);
			throw new IllegalStateException(clientMessage);
		}

		for (int i = 0; i < objectArray.length; i++) {
			objectArrayDataArg.getDataArgs().add(marshaller.marshalObjectToDataArg(objectArray[i]));
		}
		return objectArrayDataArg;
	}

	private DataArg createNullDataArg() {
		NullDataArg nullDataArg = ServicesFactory.eINSTANCE.createNullDataArg();
		nullDataArg.setType(DataArgType.NULL_LITERAL);
		return nullDataArg;
	}

	public Object demarshalDataArgToObject(DataArg dataArg, Class type) {
		ObjectArrayDataArg objectArrayDataArg = (ObjectArrayDataArg) dataArg;
		Boolean[] objectArray = new Boolean[objectArrayDataArg.getDataArgs().size()];

		int k = 0;
		Iterator i = objectArrayDataArg.getDataArgs().iterator();
		while (i.hasNext()) {
			DataArg dataArg2 = (DataArg) i.next();
			Marshaller marshaller = MarshalFactory.eINSTANCE.getMarshaller(dataArg2.getType());
			if (marshaller != null) {
				objectArray[k++] = (Boolean) marshaller.demarshalDataArgToObject(dataArg2, Boolean.TYPE);
			}
		}
		return objectArray;
	}

	/**
	 * @param containedReferences  
	 */
	public Object demarshalDataArgToObject(DataArg dataArg, Class type, List containedReferences) {
		return demarshalDataArgToObject(dataArg, type);
	}
}