/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;

import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Array;
import java.nio.charset.Charset;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.team.repository.common.internal.marshal.MarshallerNotDefinedException;
import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.serialize.internal.nls.Messages;
import com.ibm.team.repository.common.transport.internal.services.ComplexArrayDataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.ServicesFactory;
import com.ibm.team.repository.common.util.NLS;

public class ComplexArrayMarshaller extends ComplexTypeMarshallerImpl {

    public ComplexArrayMarshaller() {
        super();
        setSupportedType(EObject.class);
        setMarshallerType(MarshallerType.COMPLEX_ARRAY_LITERAL);
    }
    
    public DataArgType getSupportedDataArgType() {
        // TODO: Add support for complex Arrays
        return DataArgType.EOBJECT_ARRAY_LITERAL;
    }
    
    public boolean canMarshal(Class type) {
        if(type.isArray()) {
            return EObject.class.isAssignableFrom(type.getComponentType());
        }
        return false;
    }
    
    public void marshalObjectToOutputStream(Object object, OutputStream outputStream) throws MarshallingException {
    }
    
    public void marshalObjectToOutputStream(Object object, OutputStream outputStream,
            Charset charset) throws MarshallingException {
    }
    
    public String marshalObjectToString(Object object) throws MarshallingException {
        return null;
    }
    
    public Object demarshalInputStreamToObject(InputStream inputStream) throws MarshallingException {
        return null;
    }
    
    public Object demarshalInputStreamToObject(InputStream inputStream, Charset charset) throws MarshallingException {
        return null;
    }
    
    /**
     * Marshal object to data arg
     * @param object 
     */
    public DataArg marshalObjectToDataArg(Object object) {
        ComplexArrayDataArg complexArrayDataArg = ServicesFactory.eINSTANCE.createComplexArrayDataArg();
        complexArrayDataArg.setType(DataArgType.COMPLEX_LITERAL);
        if (object == null) {
            return complexArrayDataArg;
        }
        
        Object[] eObjectArray = null;
        try {
            eObjectArray = (Object[])object;
        } catch (ClassCastException e) {
            // primitive type that is not supported
            String pattern = Messages.getServerString("ComplexArrayMarshaller.MarshallerNotDefinedForType"); //$NON-NLS-1$
            String componentType = object.getClass().getComponentType().getName();
            String message = NLS.bind(pattern, componentType + "[]"); //$NON-NLS-1$
            throw new MarshallerNotDefinedException(message);
        }
        
        for(int i = 0; i < eObjectArray.length; i++) {
            if (eObjectArray[i] != null) {
                complexArrayDataArg.getValues().add(eObjectArray[i]);
            }
        }
        
        // Add indices if there are duplicates
        if(eObjectArray.length != complexArrayDataArg.getValues().size()) {
            for(int i = 0; i < eObjectArray.length; i++) {
                int index = complexArrayDataArg.getValues().indexOf(eObjectArray[i]);
                complexArrayDataArg.getIndices().add(Integer.valueOf(index));
            }
        }
        return complexArrayDataArg;
    }
    
    public Object demarshalDataArgToObject(DataArg dataArg, Class type) {
        ComplexArrayDataArg complexArrayDataArg = (ComplexArrayDataArg)dataArg;
        if(complexArrayDataArg.getIndices().size() > 0) {
            return demarshalDataArgToObjectArrayUsingIndices(complexArrayDataArg, type);
        }
        List values = complexArrayDataArg.getValues();
        Class componentType = type.getComponentType();
        Object[] eObjectParameter = null;
        try {
            eObjectParameter = (Object[]) Array.newInstance(componentType, values.size());
        } catch (ClassCastException e) {
            // we are using EObjectArray marshaller because a marshaller is not 
            // defined to marshal the componentType object
            String pattern = Messages.getServerString("ComplexArrayMarshaller.MarshallerNotDefinedForType"); //$NON-NLS-1$
            String message = NLS.bind(pattern, type.getCanonicalName());
            throw new MarshallerNotDefinedException(message);
        }
        for(int i = 0; i < values.size(); i++) {
            eObjectParameter[i] = values.get(i);
        }
        return eObjectParameter;
    }
    
    private Object demarshalDataArgToObjectArrayUsingIndices(
            ComplexArrayDataArg complexArrayDataArg,
            Class type) {
        
        List indices = complexArrayDataArg.getIndices();
        List eObjectArray = complexArrayDataArg.getValues();
        Class componentType = type.getComponentType();
        Object[] eObjectParameters = (Object[]) Array.newInstance(componentType, indices.size());
        for(int i = 0; i < indices.size(); i++) {
            int index = ((Integer)indices.get(i)).intValue();
            if(index != -1) {
                eObjectParameters[i] = eObjectArray.get(index);
            } else {
                eObjectParameters[i] = null;
            }
        }
        return eObjectParameters;
    }
}