/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;

import java.util.Date;

import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.internal.util.SafeSimpleDataFormat;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.PrimitiveDataArg;
import com.ibm.team.repository.common.transport.internal.services.ServicesFactory;

public class DateMarshaller extends PrimitiveTypeMarshallerImpl {
    
    public DateMarshaller() {
        super();
        setSupportedType(Date.class);
        setMarshallerType(MarshallerType.TIMESTAMP_LITERAL);
    }
    
    @Override
    public DataArgType getSupportedDataArgType() {
        return DataArgType.DATE_LITERAL;
    }
    
    @Override
    public DataArg marshalObjectToDataArg(Object object) {
        PrimitiveDataArg primitiveDataArg = ServicesFactory.eINSTANCE.createPrimitiveDataArg();
        primitiveDataArg.setType(getSupportedDataArgType());
        if(object == null) {
            primitiveDataArg.setValue(null);
        } else {
            primitiveDataArg.setValue(SafeSimpleDataFormat.DATE_FORMATS.get(0).format(object));
        }
        return primitiveDataArg;
    }
    
    @Override
    public String marshalObjectToString(Object object) throws MarshallingException {
        return SafeSimpleDataFormat.DATE_FORMATS.get(0).format(object);
    }
    
    public Object demarshalDataArgToObject(DataArg dataArg, Class classZ) {
        String stringValue = ((PrimitiveDataArg)dataArg).getValue();
        if(stringValue == null) {
            return null;
        }
        return demarshalObjectFromString(stringValue);
    }
    
    public Object demarshalObjectFromString(String value) {
        return SafeSimpleDataFormat.convertStringToDate(value);
    }
}