/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.internal.marshal.impl;

import java.io.IOException;
import java.io.InputStream;
import java.sql.Timestamp;
import java.util.Map;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl;

import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.model.RepositoryPackage;

/**
 * Marshaler for importing EObjects to the repository. The marshaller provides
 * support for mapping XML elements to the new EStructuralFeatures. 
 */
public class EObjectMigrationMarshaller extends EObjectMarshaller {
    
    public EObjectMigrationMarshaller() {
        super(EObject.class, MarshallerType.MIGRATION_LITERAL);
    }

    public EObjectMigrationMarshaller(Map options) {
        super(EObject.class, MarshallerType.MIGRATION_LITERAL, options);
    }
    
    /**
     * Override to force Timestamps to be encoded as longs.
     */ 
    protected String serializeEAttributeValue(EObject eObject, EAttribute attribute) throws MarshallingException {
        if (attribute.getEType() != RepositoryPackage.eINSTANCE.getTimestamp()) {
            String result = super.serializeEAttributeValue(eObject, attribute);
            return result;
        }

        Object objectValue = eObject.eGet(attribute);
        if (objectValue == null) {
            return null;
        }

        Timestamp ts = (Timestamp) objectValue;
        long ms = ts.getTime();
        String result = Long.toString(ms);
        return result;
    }
    
    @Override
    protected Object internalFastDemarshal(byte[] bytes) throws MarshallingException {
    	//We expect timestamps stored using Long.toString (see above), so use FastDocumentDemarshaller.  
    	//FastDocumentDemarshaller is a generated class, see FastDemarshaller.ppjava in the same package
    	FastDocumentDemarshaller demarshaller = new FastDocumentDemarshaller(bytes);
    	return demarshaller.demarshal();
    }
    
    public Object demarshalInputStreamToObject(InputStream inputStream, 
            Map options, XMLResourceImpl xmlResource) throws MarshallingException {
    	 
    	//xmlResource is a MigrationXMLResourceImpl which handles unknown features in a 
    	//way that is currently unsupported by the fast demarshaller
        options.put(XMLResource.OPTION_USE_PARSER_POOL, xmlParserPool);
        try {
            xmlResource.doLoad(inputStream, options);
            return xmlResource.getContents().get(0);
        } catch (IOException e) {
            throw new MarshallingException(e);
        }
    }
}