/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;

import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.nio.charset.Charset;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.PrimitiveDataArg;

public class EnumeratorMarshaller extends PrimitiveTypeMarshallerImpl {
    
    public static final String GETTER_METHOD_NAME = "get"; //$NON-NLS-1$

    public EnumeratorMarshaller() {
        super();
        setSupportedType(Enumerator.class);
        setMarshallerType(MarshallerType.ENUMERATOR_LITERAL);
    }
    
    @Override
    public boolean canMarshal(Class type) {
        return Enumerator.class.isAssignableFrom(type);
    }
    
    @Override
    public DataArgType getSupportedDataArgType() {
        return DataArgType.ENUMERATOR_LITERAL;
    }

    @Override
    public void marshalObjectToOutputStream(Object object, OutputStream outputStream) throws MarshallingException {
    }

    @Override
    public String marshalObjectToString(Object object) throws MarshallingException {
        return ((Enumerator)object).getLiteral();
    }

    @Override
    public Object demarshalInputStreamToObject(InputStream inputStream) throws MarshallingException {
        return null;
    }
    
    @Override
    public Object demarshalInputStreamToObject(InputStream inputStream, Charset charset) throws MarshallingException {
        return null;
    }
    
    public Object demarshalDataArgToObject(DataArg dataArg, Class classZ) {
        String value = ((PrimitiveDataArg)dataArg).getValue();
        return getEnumerator(value, classZ);
    }
    
    /**
	 * @param dataArg  
	 */
    protected List getTopLevelEObjects(DataArg dataArg) {
        return null;
    }
    
    public Object demarshalObjectFromString(String value) {
        return null;
    }
    
    private Enumerator getEnumerator(String enumeratorValue, Class classZ) {
        Method enumeratorGetterMethod;
        try {
            enumeratorGetterMethod = classZ.getDeclaredMethod(GETTER_METHOD_NAME, new Class[] { String.class });
            return (Enumerator) enumeratorGetterMethod.invoke(null, new Object[] {enumeratorValue});
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
     }
}