/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;

import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.Iterator;
import java.util.List;

import com.ibm.team.repository.common.internal.marshal.MarshalFactory;
import com.ibm.team.repository.common.internal.marshal.Marshaller;
import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.serialize.internal.nls.Messages;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.NullDataArg;
import com.ibm.team.repository.common.transport.internal.services.ObjectArrayDataArg;
import com.ibm.team.repository.common.transport.internal.services.ServicesFactory;
import com.ibm.team.repository.common.util.NLS;

/**
 * ObjectArrayMarshaller to marshal and demarshal an object array
 */
public class IntegerArrayMarshaller extends MarshallerImpl {

    public IntegerArrayMarshaller() {
        super();
        setSupportedType(Integer.TYPE);
        setMarshallerType(MarshallerType.INTEGER_ARRAY_LITERAL);
    }

    @Override
    public DataArgType getSupportedDataArgType() {
        return DataArgType.OBJECT_ARRAY_LITERAL;
    }

    @Override
    public boolean canMarshal(Class type) {
        if (type.isArray()) {
            return Integer.TYPE.equals(type.getComponentType());
        }
        return false;
    }

    public void marshalObjectToOutputStream(Object object, OutputStream outputStream) throws MarshallingException {
        throw new UnsupportedOperationException();
    }

    public void marshalObjectToOutputStream(Object object, OutputStream outputStream, Charset charset) throws MarshallingException {
        throw new UnsupportedOperationException();
    }

    public DataArgType getDataArgType() {
        return DataArgType.OBJECT_ARRAY_LITERAL;
    }

    public String marshalObjectToString(Object object) throws MarshallingException {
        throw new UnsupportedOperationException();
    }

    public Object demarshalInputStreamToObject(InputStream inputStream) throws MarshallingException {
        throw new UnsupportedOperationException();
    }

    public Object demarshalInputStreamToObject(InputStream inputStream, Charset charset) throws MarshallingException {
        throw new UnsupportedOperationException();
    }

    @Override
    public DataArg marshalObjectToDataArg(Object object) {
        if (object == null) {
            return createNullDataArg();
        }
        ObjectArrayDataArg objectArrayDataArg = ServicesFactory.eINSTANCE.createObjectArrayDataArg();
        objectArrayDataArg.setType(DataArgType.OBJECT_ARRAY_LITERAL);

        int[] objectArray = (int[]) object;
        for (int i = 0; i < objectArray.length; i++) {
            Marshaller marshaller = MarshalFactory.eINSTANCE.getMarshaller(Integer.TYPE);
            if (marshaller != null) {
                objectArrayDataArg.getDataArgs().add(marshaller.marshalObjectToDataArg(objectArray[i]));
            } else {
                // print
                String pattern = Messages.getServerString("IntegerArrayMarshaller.CannotMarshalObject"); //$NON-NLS-1$
                String message = NLS.bind(pattern, objectArray[i]);
                System.out.println(message);
            }
        }
        return objectArrayDataArg;
    }

    private DataArg createNullDataArg() {
        NullDataArg nullDataArg = ServicesFactory.eINSTANCE.createNullDataArg();
        nullDataArg.setType(DataArgType.NULL_LITERAL);
        return nullDataArg;
    }

    public Object demarshalDataArgToObject(DataArg dataArg, Class type) {
        ObjectArrayDataArg objectArrayDataArg = (ObjectArrayDataArg) dataArg;
        int[] objectArray = new int[objectArrayDataArg.getDataArgs().size()];

        int k = 0;
        Iterator i = objectArrayDataArg.getDataArgs().iterator();
        while (i.hasNext()) {
            DataArg dataArg2 = (DataArg) i.next();
            Marshaller marshaller = MarshalFactory.eINSTANCE.getMarshaller(dataArg2.getType());
            if (marshaller != null) {
                objectArray[k++] = (Integer) marshaller.demarshalDataArgToObject(dataArg2, Integer.TYPE);
            }
        }
        return objectArray;
    }

    /**
	 * @param containedReferences  
	 */
    public Object demarshalDataArgToObject(DataArg dataArg, Class type, List containedReferences) {
        return demarshalDataArgToObject(dataArg, type);
    }
}