/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;


import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.impl.EFactoryImpl;
import org.eclipse.emf.ecore.plugin.EcorePlugin;

import com.ibm.team.repository.common.internal.marshal.MarshalFactory;
import com.ibm.team.repository.common.internal.marshal.MarshalPackage;
import com.ibm.team.repository.common.internal.marshal.Marshaller;
import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingExceptionReasonCode;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model <b>Factory</b>.
 * <!-- end-user-doc -->
 * @generated
 */
public class MarshalFactoryImpl extends EFactoryImpl implements MarshalFactory {
    
    /**
     * Creates the default factory implementation.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public static MarshalFactory init() {
        try {
            MarshalFactory theMarshalFactory = (MarshalFactory)EPackage.Registry.INSTANCE.getEFactory("com.ibm.team.common.marshaller"); //$NON-NLS-1$ 
            if (theMarshalFactory != null) {
                return theMarshalFactory;
            }
        }
        catch (Exception exception) {
            EcorePlugin.INSTANCE.log(exception);
        }
        return new MarshalFactoryImpl();
    }

    // Registered primitive marshallers
    private HashMap registeredPrimitiveMarshallers;
    
    // Registered complex marshallers
    private List registeredComplexMarshallers;
    
    // Registered Array marshallers
    private List registeredArrayMarshallers;
    
    // Registered Object Marshaller
    private Marshaller objectArrayMarshaller;
    
    /**
     * Creates an instance of the factory.
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated NOT
     */
    public MarshalFactoryImpl() {
        super();
        registeredPrimitiveMarshallers = new HashMap();
        registeredComplexMarshallers = new ArrayList();
        registeredArrayMarshallers = new ArrayList();
        initializeMarshallers();
    }
    
    /**
     * Initialize marshallers
     */
    private void initializeMarshallers() {
        initializePrimitiveMarshallers();
        initializeComplexMarshallers();
        initializeArrayMarshallers();
        initializeObjectArrayMarshaller();
    }
    
    /**
     * Initialize Primitive marshallers
     */
    private void initializePrimitiveMarshallers() {
        
        // Character Type Marshaller
        CharacterMarshaller characterMarshaller = new CharacterMarshaller();
        registeredPrimitiveMarshallers.put(characterMarshaller.getSupportedType(), characterMarshaller);
        registeredPrimitiveMarshallers.put(Character.class, characterMarshaller);
        
        // Byte Marshaller
        ByteMarshaller byteMarshaller = new ByteMarshaller();
        registeredPrimitiveMarshallers.put(byteMarshaller.getSupportedType(), byteMarshaller);
        registeredPrimitiveMarshallers.put(Byte.class, byteMarshaller);
        
        // Boolean Marshaller
        BooleanMarshaller booleanMarshaller = new BooleanMarshaller();
        registeredPrimitiveMarshallers.put(booleanMarshaller.getSupportedType(), booleanMarshaller);
        registeredPrimitiveMarshallers.put(Boolean.class, booleanMarshaller);
        
        // Add String type marshaller
        StringMarshaller stringMarshaller = new StringMarshaller();
        registeredPrimitiveMarshallers.put(stringMarshaller.getSupportedType(), stringMarshaller);
        
        // Add integer type marshaller
        IntegerMarshaller integerMarshaller = new IntegerMarshaller();
        registeredPrimitiveMarshallers.put(integerMarshaller.getSupportedType(), integerMarshaller);
        registeredPrimitiveMarshallers.put(Integer.class, integerMarshaller);
        
        // Add Short type marshaller
        ShortMarshaller shortMarshaller = new ShortMarshaller();
        registeredPrimitiveMarshallers.put(shortMarshaller.getSupportedType(), shortMarshaller);
        registeredPrimitiveMarshallers.put(Short.class, shortMarshaller);
        
        // Add Long type marshaller
        LongMarshaller longMarshaller = new LongMarshaller();
        registeredPrimitiveMarshallers.put(longMarshaller.getSupportedType(), longMarshaller);
        registeredPrimitiveMarshallers.put(Long.class, longMarshaller);
        
        // Add Double type marshaller
        DoubleMarshaller doubleMarshaller = new DoubleMarshaller();
        registeredPrimitiveMarshallers.put(doubleMarshaller.getSupportedType(), doubleMarshaller);
        registeredPrimitiveMarshallers.put(Double.class, doubleMarshaller);
        
        // Add BigDecimal type marshaller
        BigDecimalMarshaller bigDecimalMarshaller = new BigDecimalMarshaller();
        registeredPrimitiveMarshallers.put(bigDecimalMarshaller.getSupportedType(), bigDecimalMarshaller);
        registeredPrimitiveMarshallers.put(BigDecimal.class, bigDecimalMarshaller);
        
        DateMarshaller dateMarshaller = new DateMarshaller();
        registeredPrimitiveMarshallers.put(dateMarshaller.getSupportedType(), dateMarshaller);
        
        // Add Timestamp type marshaller
        TimestampMarshaller timstampMarshaller = new TimestampMarshaller();
        registeredPrimitiveMarshallers.put(timstampMarshaller.getSupportedType(), timstampMarshaller);
        
        // Add UUID type marshaller
        UUIDMarshaller uuidMarshaller = new UUIDMarshaller();
        registeredPrimitiveMarshallers.put(uuidMarshaller.getSupportedType(), uuidMarshaller);
    }
    
    /**
     * Initialize Complex Marshallers
     */
    private void initializeComplexMarshallers() {
        
        // Register Enumber Marshaller
        EnumeratorMarshaller enumeratorMarshaller = new EnumeratorMarshaller();
        registeredComplexMarshallers.add(enumeratorMarshaller);
        
        // Register EObject Marshaller
        EObjectMarshaller eObjectMarshaller = new EObjectMarshaller();
        registeredComplexMarshallers.add(eObjectMarshaller);
        
        // Register JSON EObject Marshaller
        EObjectJSONMarshaller jsonMarshaller = new EObjectJSONMarshaller();
        registeredComplexMarshallers.add(jsonMarshaller);
        
        EObjectMigrationMarshaller migrationMarshaller = new EObjectMigrationMarshaller();
        registeredComplexMarshallers.add(migrationMarshaller);
        
        // Register Web Services
        registeredComplexMarshallers.add(WebServicesMarshaller.INSTANCE);
    }
    
    /**
     * Initialize Array Marshallers
     */
    private void initializeArrayMarshallers() {
        
        // Register String Array Marshaller
        PrimitiveArrayMarshaller primitiveArrayMarshaller = new PrimitiveArrayMarshaller();
        registeredArrayMarshallers.add(primitiveArrayMarshaller);
        
        //Register Boolean Array Marshaller
        BooleanArrayMarshaller booleanArrayMarshaller = new BooleanArrayMarshaller();
        registeredArrayMarshallers.add(booleanArrayMarshaller);
        
        // Register integer array marshaller
        IntegerArrayMarshaller integerArrayMarshaller =  new IntegerArrayMarshaller();
        registeredArrayMarshallers.add(integerArrayMarshaller);
        
        // Register UUID Array Marshaller
        UUIDArrayMarshaller uuidArrayMarshaller = new UUIDArrayMarshaller();
        registeredArrayMarshallers.add(uuidArrayMarshaller);
        
        // Register EObject Marshaller
        ComplexArrayMarshaller complexArrayMarshaller = new ComplexArrayMarshaller();
        registeredArrayMarshallers.add(complexArrayMarshaller);
    }
    
    private void initializeObjectArrayMarshaller() {
        objectArrayMarshaller = new ObjectArrayMarshaller();
    }
    
    /**
     * Get the marshaller given the class name
     */
    public Marshaller getMarshaller(Class classZ) {
        if (classZ.isArray()) {
            return getArrayMarshaller(classZ);
        }
        Marshaller marshaller = (Marshaller)registeredPrimitiveMarshallers.get(classZ);
        if(marshaller == null) {
            // No primitive marshallers registered
            Iterator i = registeredComplexMarshallers.iterator();
            while(i.hasNext()) {
                Marshaller complexMarshaller = (Marshaller)i.next();
                if(complexMarshaller.canMarshal(classZ)) {
                    return complexMarshaller;
                }
            }
        }
        return marshaller;
    }
    
    /**
     * Get the array marshaller
     * @param classZ Class Type
     * @return Array Marshaller
     */
    private Marshaller getArrayMarshaller(Class classZ) {
        
        if (objectArrayMarshaller.canMarshal(classZ)) {
            return objectArrayMarshaller;
        } 
        
        Marshaller arrayMarshaller = null;
        Iterator i = registeredArrayMarshallers.iterator();
        while(i.hasNext()) {
            arrayMarshaller = (Marshaller) i.next();
            if (arrayMarshaller.canMarshal(classZ)) {
                return arrayMarshaller;
            }
        }
        // TODO: From the parameter class types, we will not be able to find out if a 
        // class extends EObject
        return arrayMarshaller;
    }
    
    /**
     * Get the marshaller given the type
     * @param marshallerType Type of the marshaller
     */
    public Marshaller getMarshaller(MarshallerType marshallerType) {
        Iterator i = registeredPrimitiveMarshallers.values().iterator();
        while(i.hasNext()) {
            Marshaller primitiveMarshaller = (Marshaller)i.next();
            if(primitiveMarshaller.getMarshallerType().equals(marshallerType)) {
                return primitiveMarshaller;
            }
        }
        Iterator j = registeredComplexMarshallers.iterator();
        while(j.hasNext()) {
            Marshaller complexMarshaller = (Marshaller)j.next();
            if(complexMarshaller.getMarshallerType().equals(marshallerType)) {
                return complexMarshaller;
            }
        }
        return null;
    }
    
    public Marshaller getMarshaller(DataArgType dataArgType) {
        Iterator i = registeredPrimitiveMarshallers.values().iterator();
        while(i.hasNext()) {
            Marshaller primitiveMarshaller = (Marshaller)i.next();
            if(primitiveMarshaller.getSupportedDataArgType().equals(dataArgType)) {
                return primitiveMarshaller;
            }
        }
        if (objectArrayMarshaller.getSupportedDataArgType().equals(dataArgType)) {
            return objectArrayMarshaller;
        }
        i = registeredComplexMarshallers.iterator();
        while (i.hasNext()) {
            Marshaller complexMarshaller = (Marshaller)i.next();
            if(complexMarshaller.getSupportedDataArgType().equals(dataArgType)) {
                return complexMarshaller;
            }
        }
        return null;
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public EObject create(EClass eClass) {
        switch (eClass.getClassifierID()) {
            default:
                throw new IllegalArgumentException("The class '" + eClass.getName() + "' is not a valid classifier"); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public Object createFromString(EDataType eDataType, String initialValue) {
        switch (eDataType.getClassifierID()) {
            case MarshalPackage.MARSHALLER_TYPE:
                return createMarshallerTypeFromString(eDataType, initialValue);
            case MarshalPackage.MARSHALLING_EXCEPTION_REASON_CODE:
                return createMarshallingExceptionReasonCodeFromString(eDataType, initialValue);
            case MarshalPackage.CLASS:
                return createClassFromString(eDataType, initialValue);
            default:
                throw new IllegalArgumentException("The datatype '" + eDataType.getName() + "' is not a valid classifier"); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public String convertToString(EDataType eDataType, Object instanceValue) {
        switch (eDataType.getClassifierID()) {
            case MarshalPackage.MARSHALLER_TYPE:
                return convertMarshallerTypeToString(eDataType, instanceValue);
            case MarshalPackage.MARSHALLING_EXCEPTION_REASON_CODE:
                return convertMarshallingExceptionReasonCodeToString(eDataType, instanceValue);
            case MarshalPackage.CLASS:
                return convertClassToString(eDataType, instanceValue);
            default:
                throw new IllegalArgumentException("The datatype '" + eDataType.getName() + "' is not a valid classifier"); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public MarshallerType createMarshallerTypeFromString(EDataType eDataType, String initialValue) {
        MarshallerType result = MarshallerType.get(initialValue);
        if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        return result;
    }

    /**
     * <!-- begin-user-doc -->
     * @param eDataType 
     * <!-- end-user-doc -->
     * @generated
     */
    public String convertMarshallerTypeToString(EDataType eDataType, Object instanceValue) {
        return instanceValue == null ? null : instanceValue.toString();
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public MarshallingExceptionReasonCode createMarshallingExceptionReasonCodeFromString(EDataType eDataType, String initialValue) {
        MarshallingExceptionReasonCode result = MarshallingExceptionReasonCode.get(initialValue);
        if (result == null) throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '" + eDataType.getName() + "'"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        return result;
    }

    /**
     * <!-- begin-user-doc -->
     * @param eDataType 
     * <!-- end-user-doc -->
     * @generated
     */
    public String convertMarshallingExceptionReasonCodeToString(EDataType eDataType, Object instanceValue) {
        return instanceValue == null ? null : instanceValue.toString();
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public Class createClassFromString(EDataType eDataType, String initialValue) {
        return (Class)super.createFromString(eDataType, initialValue);
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public String convertClassToString(EDataType eDataType, Object instanceValue) {
        return super.convertToString(eDataType, instanceValue);
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @generated
     */
    public MarshalPackage getMarshalPackage() {
        return (MarshalPackage)getEPackage();
    }

    /**
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * @deprecated
     * @generated
     */
    public static MarshalPackage getPackage() {
        return MarshalPackage.eINSTANCE;
    }

} //MarshalFactoryImpl
