/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;

import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.List;

import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.NullDataArg;
import com.ibm.team.repository.common.transport.internal.services.PrimitiveArrayDataArg;
import com.ibm.team.repository.common.transport.internal.services.ServicesFactory;

public class PrimitiveArrayMarshaller extends MarshallerImpl {

    public PrimitiveArrayMarshaller() {
        super();
        setSupportedType(String.class);
        setMarshallerType(MarshallerType.PRIMITIVE_ARRAY_LITERAL);
    }
    
    public DataArgType getSupportedDataArgType() {
        return DataArgType.PRIMITIVE_ARRAY_LITERAL;
    }
    
    public boolean canMarshal(Class type) {
        if(type.isArray()) {
            return String.class.isAssignableFrom(type.getComponentType());
        }
        return false;
    }

    public void marshalObjectToOutputStream(Object object,
            OutputStream outputStream) throws MarshallingException {
    }
    
    public void marshalObjectToOutputStream(Object object, OutputStream outputStream,
            Charset charset) throws MarshallingException {
        
    }

    public String marshalObjectToString(Object object)
            throws MarshallingException {
        return null;
    }

    /**
	 * @param objects  
     * @param outputStream 
     * @throws MarshallingException 
	 */
    public void marshalArrayToOutputStream(Object[] objects,
            OutputStream outputStream) throws MarshallingException {
    }

    public Object demarshalInputStreamToObject(InputStream inputStream)
        throws MarshallingException {
        return null;
    }

    public Object demarshalInputStreamToObject(InputStream inputStream, Charset charset)
        throws MarshallingException {
        return null;
    }

    public DataArg marshalObjectToDataArg(Object object) {
        
        if(object == null) {
            return createNullDataArg();
        }
        PrimitiveArrayDataArg primitiveArrayDataArg = ServicesFactory.eINSTANCE.createPrimitiveArrayDataArg();
        primitiveArrayDataArg.setType(DataArgType.STRING_LITERAL);
        
        String[] stringArray = (String[])object;
        for(int i = 0; i < stringArray.length; i++) {
            if (stringArray[i] != null) {
                primitiveArrayDataArg.getValues().add(stringArray[i]);
            }
        }
    
        // Add indices if there are duplicates
        if(stringArray.length != primitiveArrayDataArg.getValues().size()) {
            for(int i = 0; i < stringArray.length; i++) {
                int index = primitiveArrayDataArg.getValues().indexOf(stringArray[i]);
                primitiveArrayDataArg.getIndices().add(Integer.valueOf(index));
            }
        }
        return primitiveArrayDataArg;
    }
    
    private DataArg createNullDataArg() {
        NullDataArg nullDataArg = ServicesFactory.eINSTANCE.createNullDataArg();
        nullDataArg.setType(DataArgType.NULL_LITERAL);
        return nullDataArg;
    }
    
    /**
	 * @param containedReferences  
	 */
    public Object demarshalDataArgToObject(DataArg dataArg, Class type, List containedReferences) {
        return demarshalDataArgToObject(dataArg, type);
    }
    
    /**
     * Demarshal the data arg to object
     */
    public Object demarshalDataArgToObject(DataArg dataArg, Class classZ) {
        PrimitiveArrayDataArg primitiveArrayDataArg = (PrimitiveArrayDataArg)dataArg;
        if(primitiveArrayDataArg.getIndices().size() > 0) {
            return demarshalDataArgToObjectUsingIndices(primitiveArrayDataArg, classZ);
        }
        List values = primitiveArrayDataArg.getValues();
        String[] stringArrayParameter = new String[values.size()];
        for(int i = 0; i < values.size(); i++) {
            stringArrayParameter[i] = (String)values.get(i);
        }
        return stringArrayParameter;
    }
    
    /**
     * Demarshal the data arg to an object using indices information
     * @param primitiveArrayDataArg Primitive data argument 
     * @param classZ Primitive type
     * @return
     */
    private Object demarshalDataArgToObjectUsingIndices(
            PrimitiveArrayDataArg primitiveArrayDataArg, 
            Class classZ) {
        
        List indices = primitiveArrayDataArg.getIndices();
        List values = primitiveArrayDataArg.getValues();
        String[] stringArrayParameters = new String[indices.size()];
        for(int i = 0; i < indices.size(); i++) {
            int index = ((Integer)indices.get(i)).intValue();
            if(index != -1) {
                stringArrayParameters[i] = (String)values.get(index);
            } else {
                stringArrayParameters[i] = null;
            }
        }
        return stringArrayParameters;
    }
}