/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.impl;

import java.sql.Timestamp;

import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.model.RepositoryFactory;
import com.ibm.team.repository.common.model.RepositoryPackage;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.PrimitiveDataArg;
import com.ibm.team.repository.common.transport.internal.services.ServicesFactory;
import com.ibm.team.repository.common.utils.DateUtils;

public class TimestampMarshaller extends PrimitiveTypeMarshallerImpl {

    public TimestampMarshaller() {
        super();
        setSupportedType(Timestamp.class);
        setMarshallerType(MarshallerType.TIMESTAMP_LITERAL);
    }
    
    @Override
    public DataArgType getSupportedDataArgType() {
        return DataArgType.TIMESTAMP_LITERAL;
    }
    
    public Object demarshalDataArgToObject(DataArg dataArg, Class classZ) {
        String stringValue = ((PrimitiveDataArg)dataArg).getValue();
        if(stringValue == null) {
            return null;
        }
        return demarshalObjectFromString(stringValue);
    }

    public Object demarshalObjectFromString(String value) {
        Object o = RepositoryFactory.eINSTANCE.createFromString(
                RepositoryPackage.eINSTANCE.getTimestamp(),
                value);
        return o;
    }
    
    @Override
    public DataArg marshalObjectToDataArg(Object object) {
        PrimitiveDataArg primitiveDataArg = ServicesFactory.eINSTANCE.createPrimitiveDataArg();
        primitiveDataArg.setType(getSupportedDataArgType());
        if(object == null) {
            primitiveDataArg.setValue(null);
        } else {
            primitiveDataArg.setValue(convertTimestampToString((Timestamp)object));
        }
        return primitiveDataArg;
    }
    
    @Override
    public String marshalObjectToString(Object object) {
        return convertTimestampToString((Timestamp)object);
    }
    
    private String convertTimestampToString(Timestamp timestamp) {
        return DateUtils.formatTimeRFC3339(timestamp);
    }
}