/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * 
 */
package com.ibm.team.repository.common.internal.marshal.impl;

import com.ibm.team.repository.common.UUID;
import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.transport.internal.services.DataArg;
import com.ibm.team.repository.common.transport.internal.services.DataArgType;
import com.ibm.team.repository.common.transport.internal.services.PrimitiveDataArg;
import com.ibm.team.repository.common.transport.internal.services.ServicesFactory;

/**
 * Marshaller to serialize / deserialize a UUID object
 */
public class UUIDMarshaller extends PrimitiveTypeMarshallerImpl {

    public UUIDMarshaller() {
        super();
        setSupportedType(UUID.class);
        setMarshallerType(MarshallerType.UUID_LITERAL);
    }
    
    public DataArgType getSupportedDataArgType() {
        return DataArgType.UUID_LITERAL;
    }
    
    public DataArg marshalObjectToDataArg(Object object) {
        PrimitiveDataArg primitiveDataArg = ServicesFactory.eINSTANCE.createPrimitiveDataArg();
        primitiveDataArg.setType(getSupportedDataArgType());
        if(object != null) {
            primitiveDataArg.setValue(((UUID)object).getUuidValue());
        }
        return primitiveDataArg;
    }
    
    public String marshalObjectToString(Object object) {
       return ((UUID)object).getUuidValue();
    }
    
    public Object demarshalDataArgToObject(DataArg dataArg, Class classZ) {
        String stringValue = ((PrimitiveDataArg)dataArg).getValue();
        if(stringValue == null) {
            return null;
        }
        return UUID.valueOf(stringValue);
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.marshal.PrimitiveTypeMarshaller#demarshalObjectObjectFromString(java.lang.String)
     */
    public Object demarshalObjectFromString(String value) {
        return UUID.valueOf(value);
    }
}