/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.util;


import java.util.Map;

import org.eclipse.emf.ecore.xmi.XMIException;
import org.eclipse.emf.ecore.xmi.XMLDefaultHandler;
import org.eclipse.emf.ecore.xmi.XMLHelper;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMLHandler;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * This class wraps an XMLHandler with a SAX DefaultHandler.
 * Adapted from deprecated EMF SAXWrapper
 */
public class SAXWrapper extends DefaultHandler implements XMLDefaultHandler
{
  protected XMLHandler handler;
  
  
  public void prepare(XMLResource resource, XMLHelper helper, Map options)
  {
    handler.prepare(resource, helper, options);  
  }

  public void reset()
  {
   handler.reset();    
  }

  /**
   * Constructor for SAXWrapper.
   */
  public SAXWrapper(XMLHandler handler)
  {
    super();
    this.handler = handler;
  }

  public void setDocumentLocator(Locator locator)
  {
    handler.setDocumentLocator(locator);
  }

  public void startDocument() throws SAXException
  {
    handler.startDocument();
  }

  public void endDocument() throws SAXException
  {
    handler.endDocument();
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException
  {
    handler.startPrefixMapping(prefix, uri);
  }

  public void endPrefixMapping(String prefix) throws SAXException
  {
  }

  public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
  {
    handler.startElement(uri, localName, qName, attributes);
  }

  public void endElement (String uri, String localName, String qName) throws SAXException
  {
    handler.endElement(uri, localName, qName);
  }

  public void warning (SAXParseException e) throws SAXException
  {
    XMIException xmi = new XMIException(e.getException() == null ? e : e.getException(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
    handler.warning(xmi);
  }

  public void error (SAXParseException e) throws SAXException
  {
    XMIException xmi = new XMIException(e.getException() == null ? e : e.getException(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
    handler.error(xmi);
  }

  public void fatalError (SAXParseException e) throws SAXException
  {
    XMIException xmi = new XMIException(e.getException() == null ? e : e.getException(), e.getSystemId(), e.getLineNumber(), e.getColumnNumber());
    handler.fatalError(xmi);
    throw e;
  }

  public void characters (char ch[], int start, int length) throws SAXException
  {
    handler.characters(ch, start, length);
  }

  public void ignorableWhitespace (char ch[], int start, int length) throws SAXException
  {
    // handler.ignorableWhitespace(ch, start, length);
  }

  public void processingInstruction (String target, String data) throws SAXException
  {
    handler.processingInstruction(target, data);
  }

  public void skippedEntity (String name) throws SAXException
  {
    // handler.skippedEntity(name);
  }

  public InputSource resolveEntity(String publicId, String systemId) throws SAXException
  {
    // handler.resolveEntity(publicId, systemId);
    return null;
  }

  public void notationDecl(String name, String publicId, String systemId) throws SAXException
  {
    // handler.notationDecl(name, publicId, systemId);
  }

  public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException
  {
    // handler.unparsedEntityDecl(name, publicId, systemId, notationName);
  }

  public void startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)
  {
    handler.startDTD(name, publicId, systemId);
  }

  public void endDTD()
  {
  }

  public void startEntity(java.lang.String name)
  {
  }

  public void endEntity(java.lang.String name)
  {
  }

  public void startCDATA()
  {
    handler.startCDATA();
  }

  public void endCDATA()
  {
    handler.endCDATA();
  }

  public void comment(char [] characters, int start, int length) throws SAXException
  {
    handler.comment(characters, start, length);
  }
}
