/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.util;

import java.util.Map;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.xmi.PackageNotFoundException;
import org.eclipse.emf.ecore.xmi.XMLHelper;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.SAXXMLHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.ibm.team.repository.common.IType;
import com.ibm.team.repository.common.internal.util.ComponentRegistry;
import com.ibm.team.repository.common.internal.util.ItemUtil;
import com.ibm.team.repository.common.serialize.internal.nls.Messages;
import com.ibm.team.repository.common.util.NLS;

public class WebServicesSAXXMLHandler extends SAXXMLHandler {
    
    public static final String MARSHALLER_ANNOTATION = "http:///com/ibm/team/core/marshal"; //$NON-NLS-1$
    public static final String MARSHALLER_CONTAINMENT_ANNOTATION_KEY = "containment"; //$NON-NLS-1$
    public static final String MARSHALLER_ELEMENT_ANNOTATION_KEY = "element"; //$NON-NLS-1$

    public WebServicesSAXXMLHandler(XMLResource xmiResource, XMLHelper helper, Map options) {
        super(xmiResource, helper, options);
    }
    
    @Override
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException {
        // don't resolve any entities. Users can use XML entities to fetch
        // important information from the server machine.
        throw new SAXException("Entities are not allowed in SOAP request"); //$NON-NLS-1$
    }
    
    @Override
    protected void handleFeature(String prefix, String name) {
        if(prefix.equals("soapenv") && name.equals("Body")) {  //$NON-NLS-1$ //$NON-NLS-2$
            prefix = ""; //$NON-NLS-1$
            name = "body"; //$NON-NLS-1$
            super.handleFeature(prefix, name);
            return;
        } 
        if(prefix.equals("soapenv") && name.equals("Fault")) { //$NON-NLS-1$ //$NON-NLS-2$
            prefix = ""; //$NON-NLS-1$
            name = "fault"; //$NON-NLS-1$
        }
        super.handleFeature(prefix, name);
    }
    
    @SuppressWarnings("deprecation")
    @Override
    protected EObject createObjectFromFactory(EFactory factory, String typeName) {
        if (factory != null) {
            EClassifier target = factory.getEPackage().getEClassifier(typeName);
            if (target instanceof EClass) {
                target = ItemUtil.getTargetClassIfFacade((EClass) target);
                typeName = target.getName();
            }
        }
        return super.createObjectFromFactory(factory, typeName);
    }
    
    @Override
    protected EObject createObjectFromTypeName(EObject peekObject, String typeQName, EStructuralFeature feature) {
        
        if (shouldConsiderAsContainment(feature)) {
            return super.createObjectFromTypeName(peekObject, typeQName, feature);
        }

        String typeName = null;
        String prefix = ""; //$NON-NLS-1$
        int index = typeQName.indexOf(":"); //$NON-NLS-1$
        if (index > 0) {
            prefix = typeQName.substring(0, index);
            typeName = typeQName.substring(index + 1);
        } else {
            typeName = typeQName;
        }

        contextFeature = feature;
        EFactory eFactory = getFactoryForPrefix(prefix);
        contextFeature = null;

        if (eFactory == null && prefix.equals("") && helper.getURI(prefix) == null) { //$NON-NLS-1$
            contextFeature = feature;
            EPackage ePackage = handleMissingPackage(null);
            contextFeature = null;
            if (ePackage == null) {
                error(new PackageNotFoundException(null, getLocation(), getLineNumber(), getColumnNumber()));
            } else {
                eFactory = ePackage.getEFactoryInstance();
            }
        }
        
        if (eFactory == null && prefix.length() != 0) {
            String message = NLS.bind(Messages.getServerString("WebServicesSAXXMLHandler.MissingPackage"), prefix); //$NON-NLS-1$
            throw new RuntimeException(message);
        }
        
        String oldTypeName = typeName;
        final String HANDLE = "Handle"; //$NON-NLS-1$
        if (!typeName.endsWith(HANDLE)) {
            typeName = typeName + HANDLE;
        }
        
        EClassifier type = helper.getType(eFactory, typeName);
        EObject obj = null;
        if (type != null) {
            obj = helper.createObject(eFactory, type);
            if (obj == null) {
                typeName = oldTypeName;
            }
        }
        obj = null;
        return super.createObjectFromTypeName(peekObject, typeQName, feature);
    }

	@Override
	protected EObject createObjectFromFeatureType(EObject peekObject, EStructuralFeature feature) {
		// In one specific case where an item has been generalized in an addTables
		// migration, we have to specify its qualified name when creating it.
		// As soon as we determine that we are not currently dealing with that
		// case, bail and call super's implementation.
		if ((feature == null) || (extendedMetaData != null))
			return super.createObjectFromFeatureType(peekObject, feature);
		EClassifier eType = feature.getEType();
		if (eType == null)
			return super.createObjectFromFeatureType(peekObject, feature);
		EClass eClass = (EClass)eType;
		if (eClass.isAbstract() == false)
			return super.createObjectFromFeatureType(peekObject, feature);
		EClass containingClass = feature.getEContainingClass();
		String modelURI = containingClass.getEPackage().getNsURI();
		if (ComponentRegistry.INSTANCE.isValidComponent(modelURI) == false)
			return super.createObjectFromFeatureType(peekObject, feature);
		String containingClassTypeName = containingClass.getName();
		String attribName = feature.getName();
		IType type = ComponentRegistry.INSTANCE.getComponentDescriptorForModelUri(modelURI).getModelElementDescriptor().getNamespaceAndTypeBeforeAttributeWasGeneralized(containingClassTypeName, attribName);
		if (type == null)
			return super.createObjectFromFeatureType(peekObject, feature);
		EPackage ePackage = eClass.getEPackage();
		String prefix = helper.getPrefix(ePackage);
		String fqName = prefix + ":" + type.getName(); //$NON-NLS-1$
		return super.createObjectFromTypeName(peekObject, fqName, feature);
	}

    /**
     * Should the reference a containment?
     * @param eObject EObject that contains the reference
     * @param reference Reference
     * @return
     */
    private boolean shouldConsiderAsContainment(EStructuralFeature structuralFeature) {
        boolean containment = true;
        if(structuralFeature.getEType() instanceof EClass) {
            String marshallerAnnotation = getAnnotation(structuralFeature, MARSHALLER_ANNOTATION, MARSHALLER_CONTAINMENT_ANNOTATION_KEY);
            containment = ((EReference)structuralFeature).isContainment() || 
                ((marshallerAnnotation != null && marshallerAnnotation.equals(String.valueOf(true))));
            if (!containment) {
                // this is to support shared reference contributor contained in a LoginResult class
                return !(ItemUtil.isHandle((EReference)structuralFeature));
            }
        }
        return containment;
    }
    
    /**
     * Get Annotation given the source and key name
     */
    private String getAnnotation(EStructuralFeature structuralFeature, String sourceAnnotationName, String detailsKey) {
        EAnnotation annotation = structuralFeature.getEAnnotation(sourceAnnotationName);
        if (annotation == null) {
            return null;
        }
        return annotation.getDetails().get(detailsKey);
    }
}