/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.internal.marshal.util;

import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMLHelperImpl;

/**
 * A specialized XMLHelper that allows for the deserialization of character-based
 * control sequences that are illegal in XML (and have been encoded).
 */
public class WebServicesXMLHelperImpl extends XMLHelperImpl {

    public WebServicesXMLHelperImpl() {
        super();
    }

    public WebServicesXMLHelperImpl(XMLResource resource) {
        super(resource);
    }

    @Override
    protected Object createFromString(EFactory factory, EDataType dataType, String value) {
        // If it's a string, decode any special control characters we may have
        // encoded (those control characters that are illegal in XML).
        //
        //Workitem 38591
        
        if (dataType.getClassifierID() == EcorePackage.ESTRING) {
            value = XMLEscapeUtil.decodeControlChars(value);
        }
        
        return doCreateFromString(factory, dataType, value);
    }
    
    protected Object doCreateFromString(EFactory factory, EDataType dataType, String value) {
        return super.createFromString(factory, dataType, value);
    }

}
