/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

import org.eclipse.emf.ecore.xmi.XMLHelper;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMLLoadImpl;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.DefaultHandler;

public class WebServicesXMLLoadImpl extends XMLLoadImpl {
    private WebServicesXMLResourceImpl xmlResource;

    public WebServicesXMLLoadImpl(WebServicesXMLResourceImpl resource, XMLHelper helper) {
        super(helper);
        this.xmlResource = resource;
    }
    
    protected DefaultHandler makeDefaultHandler() {
        return new WebServicesSAXXMLHandler(xmlResource, helper, options);
    }

    /* (non-Javadoc)
     * The parent class implementation does an expensive (in memory
     * allocation terms, at least) lookahead in the input stream to
     * determine the encoding of the XML document.  This override
     * uses the encoding specified by the creator of the
     * WebServicesXMLResourceImpl, if one has been specified.
     * See bug 29454, "XMLLoadImpl.getEncoding creates BufferedInputStream of default 8K size when reading item states"
     * 
     * @see org.eclipse.emf.ecore.xmi.impl.XMLLoadImpl#getEncoding()
     */
    @Deprecated
    @Override
    protected String getEncoding() throws IOException {
        String encoding = null;
        if (xmlResource == null) {
            encoding = super.getEncoding();
        }
        else if (xmlResource.isEncodingSet()) {
            encoding = xmlResource.getEncoding();
        }
        else {
            encoding = super.getEncoding();
        }

        return encoding;
    }

    @Override
    public void load(XMLResource r, InputSource inputSource,
            Map<?, ?> optionMap) throws IOException {
        try {
            super.load(r, inputSource, optionMap);
        }
        finally {
            r = null;
        }
    }

    @Override
    public void load(XMLResource r, InputStream inputStream,
            Map<?, ?> optionMap) throws IOException {
        try {
            super.load(r, inputStream, optionMap);
        }
        finally {
            r = null;
        }
    }

    @Override
    public void load(XMLResource r, Node node, Map<?, ?> optionMap)
            throws IOException {
        try {
            super.load(r, node, optionMap);
        }
        finally {
            r = null;
        }
    }
}
