/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.internal.marshal.util;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.xmi.XMLHelper;
import org.eclipse.emf.ecore.xmi.XMLLoad;
import org.eclipse.emf.ecore.xmi.impl.XMLResourceImpl;

import com.ibm.team.repository.common.serialize.internal.nls.Messages;
import com.ibm.team.repository.common.util.NLS;

public class WebServicesXMLResourceImpl extends XMLResourceImpl {
    private boolean haveSetEncoding = false;

    public WebServicesXMLResourceImpl() {
        super();
    }

    public WebServicesXMLResourceImpl(URI uri) {
        super(uri);
    }
    
    @Override
    protected XMLLoad createXMLLoad() {
        return new WebServicesXMLLoadImpl(this, createXMLHelper());
    }

    @Override
    public String getEncoding() {
        return super.getEncoding();
    }

    @Override
    public void setEncoding(String encoding) {
        if (haveSetEncoding) {
            String oldEncoding = super.getEncoding();
            if (oldEncoding.equals(encoding))
                return;

            // I don't know what it would mean if the encoding were to change,
            // but I suspect it means that the method that created this object 
            // has set an encoding that does not match the encoding found in the 
            // input stream.
            String pattern = Messages.getServerString("WebServicesXMLResourceImpl.CantChangeEncoding"); //$NON-NLS-1$
            String msg = NLS.bind(pattern, oldEncoding, encoding);
            System.err.println(msg);
            return;
        }
        super.setEncoding(encoding);
        haveSetEncoding = true;
    }

    public boolean isEncodingSet() {
        return haveSetEncoding;
    }

    @Override
    protected XMLHelper createXMLHelper() {
        return new WebServicesXMLHelperImpl(this);
    }
    
}
