/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.json.internal;

/**
 *
 */
public class Token {

    static final public Token TokenEOF = new Token();
    static final public Token TokenBraceL = new Token();
    static final public Token TokenBraceR = new Token();
    static final public Token TokenBrackL = new Token();
    static final public Token TokenBrackR = new Token();
    static final public Token TokenColon = new Token();
    static final public Token TokenComma = new Token();
    static final public Token TokenTrue = new Token();
    static final public Token TokenFalse = new Token();
    static final public Token TokenNull = new Token();

    private String valueString;
    private Number valueNumber;

    public Token() {
        super();
    }

    public Token(String value) {
        super();
        valueString = value;
    }

    public Token(Number value) {
        super();
        valueNumber = value;
    }

    public String getString() {
        return valueString;
    }

    public Number getNumber() {
        return valueNumber;
    }

    public boolean isString() {
        return null != valueString;
    }

    public boolean isNumber() {
        return null != valueNumber;
    }

    @Override
    public String toString() {
        if (this == TokenEOF)
            return "Token: EOF"; //$NON-NLS-1$
        if (this == TokenBraceL)
            return "Token: {";  //$NON-NLS-1$
        if (this == TokenBraceR)
            return "Token: }";  //$NON-NLS-1$
        if (this == TokenBrackL)
            return "Token: [";  //$NON-NLS-1$
        if (this == TokenBrackR)
            return "Token: ]";  //$NON-NLS-1$
        if (this == TokenColon)
            return "Token: :";  //$NON-NLS-1$
        if (this == TokenComma)
            return "Token: ,";  //$NON-NLS-1$
        if (this == TokenTrue)
            return "Token: true";  //$NON-NLS-1$
        if (this == TokenFalse)
            return "Token: false";  //$NON-NLS-1$
        if (this == TokenNull)
            return "Token: null";  //$NON-NLS-1$

        if (this.isNumber())
            return String.format("Token: Number - %s", getNumber().toString()); //$NON-NLS-1$
        if (this.isString())
            return String.format("Token: String - \"%s\"", getString()); //$NON-NLS-1$

        return "Token: unknown"; //$NON-NLS-1$
    }
}