/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize;

import com.ibm.team.repository.common.serialize.internal.PrimitiveRegistry;


/**
 * Registry of IPrimitiveHandlers.
 */
public interface IPrimitiveRegistry {

	/**
	 * The global primitive registry.
	 */
	static final public IPrimitiveRegistry INSTANCE = new PrimitiveRegistry();

	/**
	 * Register a class and it's associated handler.
	 */
	public void register(Class clazz, IPrimitiveHandler handler);

	/**
	 * Return a handler given the class.
	 */
	public IPrimitiveHandler getHandler(Class clazz);
	
    /**
     * Return a handler given the xsi type (namespace uri and local name).
     */
    public IPrimitiveHandler getHandler(String[] xsiType);
    
	/**
	 * Return indication of whether a class is a primitive.
	 */
	public boolean isPrimitive(Class clazz);
	
}
