/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize;

import java.io.Writer;

import org.eclipse.emf.ecore.EObject;

/**
 * Describes operations available to serializers.
 * 
 * Serializers are thread-safe.  You can reuse a Serializer 
 * across multiple threads simultaneously.  The verbosity 
 * of the serializer, as set by the setverbose() method, 
 * is determined by a serializer when it starts, and won't
 * be changed if the verbosity is changed after a 
 * serialization has started.
 */
public interface ISerializer {

	/**
	 *
	 */
	public void setVerbose(boolean value);
	
	/**
	 *
	 */
	public boolean isVerbose();
	
    /**
     *
     */
    public String getFormat();
    
    /**
     * 
     */
    public String getVersion();
    
    
    /**
     * Sets the repository URI that may be referenced by the serializer to
     * construct absolute item URIs on output.
     * 
     * @param uri A URI to the repository (e.g. "http://jazzdev.ibm.com:9080/jazz").
     * May be <code>null</code>.
     * @return
     */
    public void setRepoUri(String uri);
    
    /**
     * Returns the repository URI associated to this serializer.
     * 
     * @return The repository URI associated to this serializer. May be
     * <code>null</code>.
     */
    public String getRepoUri();
    
    /**
     *
     */
    public void serialize(Writer writer, EObject eObject) throws SerializeException;
}
