/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize;

import com.ibm.team.repository.common.serialize.internal.SerializerFactory;

/**
 * Factory to create ISerializer and IDeserializer objects.
 */
public interface ISerializerFactory {

	/**
	 * 
	 */
	static public final ISerializerFactory INSTANCE = new SerializerFactory();

	/**
	 * 
	 */
	static public final String SERIALIZED_FORMAT_XML  = "xml"; //$NON-NLS-1$

	/**
	 * 
	 */
	static public final String SERIALIZED_FORMAT_JSON = "json"; //$NON-NLS-1$
	
    /**
     * 
     */
    static public final String SERIALIZED_VERSION_0_3 = "0.3"; //$NON-NLS-1$

    /**
     * 
     */
    static public final String SERIALIZED_VERSION_0_4 = "0.4"; //$NON-NLS-1$

	/**
	 * Return an ISerializer for a specified format, for the specified version.
	 * If the version is null, use the default version.  
	 */
	public ISerializer getSerializer(String serializedFormat, String serializedVersion);
	
    /**
     * Return an ISerializer for a specified format, using the most default version.
     */
    public ISerializer getSerializer(String serializedFormat);
    
	/**
	 * Return an IDeserializer for a specified format.
     * IDeserializers can handle any version.
	 */
	public IDeserializer getDeserializer(String serializedFormat);
	
}
