/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize;

import java.net.URI;

import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.IType;

/**
 * Abstract interface for converting between object references and URIs.
 */
public interface IURISerializer {
    
    /**
     * Given the specified object, returns a relative URI representing the 
     * object.
     * 
     * @param object The object to build a URI representation for.  Must not
     * be <code>null</code>.
     * 
     * @return The URI representing the object resource.  Will be <code>null</code>
     * if a URI can't be determined.
     * 
     * @throws SerializeException Thrown on syntax errors with the specified uri.
     */
    URI getUri(Object object) throws SerializeException;
    
    /**
     * Returns the item handle of the resource identified by the URI.
     * 
     * @param loc The URI identifying a resource.  The URI may or
     * may not be of a well-known item identifier.
     * 
     * @return The item handle of the resource identified by the referenced
     * location.  Will be <code>null</code> if this location does not 
     * identify an {@link IItem}.
     * 
     * @throws SerializeException Thrown on syntax errors with the specified uri.
     */
    IItemHandle getItemHandle(URI loc) throws SerializeException;
    
    /**
     * Returns the type of the resource identified by the specified URI.
     * 
     * @param loc The uri identifying a resource.  The uri may or
     * may not be of a well-known item or virtual type identifier.
     * 
     * @return The type encoded in the URI.  May be <code>null</code> if no
     * type is recognized.
     * 
     * @throws SerializeException Thrown on syntax errors with the specified uri.
     */
    IType getType(URI loc) throws SerializeException;

}
