/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.repository.common.serialize;


/**
 * Placeholder exception class to be used to pass text messages to serializers. Passing an instance of this class
 * will cause the exception not to be logged but the message will still be serialized. This class returns zero-length
 * stack trace.
 * @author alexbern
 * @since 4.0.5
 */
public final class MessagePlaceholder extends Exception
{
	private static final long serialVersionUID = 1L;
	public MessagePlaceholder( String msg ) {
		super( msg );
	}
	@Override
	public StackTraceElement[] getStackTrace() {
		if( StackUtil.shouldAvoidSerializingExceptions())
			return new StackTraceElement[0];
		return super.getStackTrace();
	}
	@Override
	public Throwable fillInStackTrace() {
		if( StackUtil.shouldAvoidSerializingExceptions())
			return this;
		return super.fillInStackTrace();
	}
	@Override
	public void setStackTrace(StackTraceElement[] arg0) {
		if( StackUtil.shouldAvoidSerializingExceptions() == false ) {
			super.setStackTrace(arg0);
		}
	}
};
