/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize.internal;

import java.io.Reader;

import com.ibm.team.repository.common.serialize.IDeserializer2;
import com.ibm.team.repository.common.serialize.IStackAdjuster;
import com.ibm.team.repository.common.serialize.IURISerializer;
import com.ibm.team.repository.common.serialize.SerializeException;
import com.ibm.team.repository.common.transport.HttpUtil.MediaType;

/**
 * The abstract base class from which {@link MediaType} deserializers are to derive.
 */
public abstract class AbstractDeserializer implements IDeserializer2{

    private final MediaType _mType;
    
    protected final IURISerializer _uriSerializer;

    /**
     * Constructs a new instance.
     * 
     * @param mType The media type for this deserializer. Must not be <code>null</code>.
     * 
     * @param The serializer used to convert between objects and URIs. Must not
     * be <code>null</code>.
     */
    AbstractDeserializer(MediaType mType, IURISerializer uriSerializer) {
        
        if (mType == null)
            throw new IllegalArgumentException("MediaType must not be null"); //$NON-NLS-1$
        
        _mType = mType;
        _uriSerializer = uriSerializer;
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#getMediaType()
     */
    public MediaType getMediaType() {
        return _mType;
    }
    
    public Throwable deserializeException(Reader r, ClassLoader loader, IStackAdjuster adj) throws SerializeException {
    	throw new IllegalStateException("Not implemented."); //$NON-NLS-1$
    }

}
