/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.common.serialize.internal;

import com.ibm.team.repository.common.serialize.IDeserializer;
import com.ibm.team.repository.common.serialize.IPrimitiveHandler;
import com.ibm.team.repository.common.serialize.IPrimitiveRegistry;
import com.ibm.team.repository.common.serialize.SerializeException;
import com.ibm.team.repository.common.serialize.internal.nls.Messages;
import com.ibm.team.repository.common.util.NLS;

/**
 *
 */
public abstract class Deserializer implements IDeserializer {

    private static final IPrimitiveRegistry PrimitiveReg = IPrimitiveRegistry.INSTANCE;

    public Deserializer() {
        super();
    }
    
    public Object deserializeString(String string, Class expectedType) throws SerializeException {
        
        IPrimitiveHandler primHandler = PrimitiveReg.getHandler(expectedType);
        if (null == primHandler) {
            String msg = NLS.bind(Messages.getServerString("Deserializer.UnknownPrimitiveType"), expectedType.getName()); //$NON-NLS-1$
            throw new SerializeException(msg);
        }
        
        return primHandler.stringToObject(string);
    }
    
}
