/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize.internal;

import java.io.IOException;
import java.io.Writer;

import org.eclipse.emf.ecore.EObject;

import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.serialize.ISerializer;
import com.ibm.team.repository.common.serialize.SerializeException;

/**
 *
 */
public abstract class Serializer implements ISerializer {
    private boolean verbose;
    private String repoUri;

    /**
     * 
     */
    protected Serializer() {
        super();

        verbose = false;
    }

    /**
     *
     */
    public boolean isVerbose() {
        return verbose;
    }

    /**
     *
     */
    public void setVerbose(boolean value) {
        verbose = value;
    }

    
    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.ISerializer#setRepoUri(java.lang.String)
     */
    public void setRepoUri(String uri) {
        repoUri = uri;
    }

    
    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.ISerializer#getRepoUri()
     */
    public String getRepoUri() {
        return repoUri;
    }

    /**
     *
     */
    public void serialize(Writer writer, EObject eObject) throws SerializeException {
        if (null == eObject)
            throw new IllegalArgumentException("object to serialize must not be null"); //$NON-NLS-1$

        SerializingWriter sWriter;

        if (isVerbose()) {
            sWriter = new SerializingWriterVerbose(writer);
        }

        else {
            sWriter = new SerializingWriter(writer);
        }

        Class expectedType = eObject.eClass().getInstanceClass();
        
        serializeEObject(sWriter, eObject, expectedType);

        try {
            sWriter.flush();
        } catch (IOException e) {
            throw SerializeException.from(e);
        }
    }

    /**
     *
     */
    public boolean shouldMorphItemToHandle(EObject eObject, Class expectedType) {
        if (null == eObject)
            return false;

        // if object isn't an Item, can't morph it
        if (!(eObject instanceof IItem))
            return false;

        // it's an item, but check to see if expected type is;
        // if expected type is an Item, don't morph it
        if (IItem.class.isAssignableFrom(expectedType))
            return false;

        // object is an item and expected type is a handle
        return true;
    }

    /**
     *
     */
    public EObject morphItemToHandle(EObject item) {
        if (null == item)
            return null;

        return (EObject) ((IItem) item).getItemHandle();
    }

    /**
     *
     */
    public Class getExpectedType(EObject eObject) {
        if (null == eObject) return null;
        
        return eObject.eClass().getInstanceClass();
    }
    
    /**
     *
     */
    abstract protected void serializeEObject(SerializingWriter sWriter, EObject eObject, Class expectedType) throws SerializeException;

}
