/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize.internal;

import com.ibm.team.repository.common.serialize.IDeserializer;
import com.ibm.team.repository.common.serialize.ISerializer;
import com.ibm.team.repository.common.serialize.ISerializerFactory;
import com.ibm.team.repository.common.serialize.internal.json.JSONDeserializer;
import com.ibm.team.repository.common.serialize.internal.json.JSONSerializer_03;
import com.ibm.team.repository.common.serialize.internal.xml.XMLDeserializer;
import com.ibm.team.repository.common.serialize.internal.xml.XMLSerializer_03;
import com.ibm.team.repository.common.serialize.internal.xml.XMLSerializer_04;

/**
 *
 */
public class SerializerFactory implements ISerializerFactory {

    /**
     * 
     */
    static public final String AttributeSerializeVersion = "serializeVersion"; //$NON-NLS-1$

    public SerializerFactory() {
		super();
	}
	
	/**
	 * 
	 */
	public IDeserializer getDeserializer(String serializedFormat) {
		if (null == serializedFormat) throw new IllegalArgumentException("serializedFormat must not be null"); //$NON-NLS-1$
		
		
		if (SERIALIZED_FORMAT_XML.equals(serializedFormat))  return new XMLDeserializer();
		if (SERIALIZED_FORMAT_JSON.equals(serializedFormat)) return new JSONDeserializer();
			
		return null;
	}

	/**
	 * 
	 */
	public ISerializer getSerializer(String serializedFormat, String serializedVersion) {
		if (null == serializedFormat) throw new IllegalArgumentException("serializedFormat must not be null"); //$NON-NLS-1$
		
		if (SERIALIZED_FORMAT_XML.equals(serializedFormat)) {
			if (null == serializedVersion) serializedVersion = SERIALIZED_VERSION_0_4;
            if (SERIALIZED_VERSION_0_3.equals(serializedVersion))  return new XMLSerializer_03();
            if (SERIALIZED_VERSION_0_4.equals(serializedVersion))  return new XMLSerializer_04();
		}
        else if (SERIALIZED_FORMAT_JSON.equals(serializedFormat)) {
			if (null == serializedVersion) serializedVersion = SERIALIZED_VERSION_0_3;
			if (SERIALIZED_VERSION_0_3.equals(serializedVersion)) return new JSONSerializer_03();
        }
        
		return null;
	}
    
    /**
     * 
     */
    public ISerializer getSerializer(String serializedFormat) {
        return getSerializer(serializedFormat, null);
    }
    
}
