/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize.internal;

import java.net.URI;

import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.IType;
import com.ibm.team.repository.common.IVirtual;
import com.ibm.team.repository.common.Location;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.serialize.internal.nls.Messages;
import com.ibm.team.repository.common.serialize.IURISerializer;
import com.ibm.team.repository.common.serialize.SerializeException;
import com.ibm.team.repository.common.util.NLS;

/**
 * Generic implementation of a URI serializer.
 */
public class UriSerializer implements IURISerializer {

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IURISerializer#getItemHandle(java.net.URI)
     */
    public IItemHandle getItemHandle(URI loc) throws SerializeException{
        Location itemLoc = null;
        try {
            itemLoc = Location.location(loc);
        } catch (final TeamRepositoryException e) {
            final String msg = NLS.bind(Messages.getServerString("UriSerializer.0"),  //$NON-NLS-1$
                    loc, e.getMessage());
            throw new SerializeException(msg);
        }
        return itemLoc.getItemHandle();
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IURISerializer#getUri(java.lang.Object)
     */
    public URI getUri(Object item) throws SerializeException {
        if (item == null)
            throw new IllegalArgumentException("item must not be null"); //$NON-NLS-1$
        
        if (!(item instanceof IItemHandle) && !(item instanceof IVirtual))
            throw new IllegalArgumentException("item must be of type IType or IVirtual"); //$NON-NLS-1$
        
        if (item instanceof IItemHandle)
            return Location.itemLocation((IItemHandle)item, null).toRelativeUri();
        
        String uri = ((IVirtual)item).getLocation();
        if (uri == null)
            throw new IllegalArgumentException("referenced virtual contains no location information"); //$NON-NLS-1$
        
        try {
            return Location.location(new URI(uri)).toRelativeUri();
        } catch (Exception e) {
            throw new SerializeException(e.getMessage());
        }
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IURISerializer#getType(java.net.URI)
     */
    public IType getType(URI loc) throws SerializeException {
        if (loc == null)
            throw new IllegalArgumentException("location must not be null"); //$NON-NLS-1$
        
        Location itemLoc = null;
        try {
            itemLoc = Location.location(loc);
        } catch (final TeamRepositoryException e) {
            final String msg = NLS.bind(Messages.getServerString("UriSerializer.0"),  //$NON-NLS-1$
                    loc, e.getMessage());
            throw new SerializeException(msg);
        }
        
        return itemLoc.getType();
    }
}
