/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.common.serialize.internal;

import java.io.Reader;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.team.repository.common.IItem;
import com.ibm.team.repository.common.IItemType;
import com.ibm.team.repository.common.IVirtual;
import com.ibm.team.repository.common.IVirtualType;
import com.ibm.team.repository.common.serialize.IURISerializer;
import com.ibm.team.repository.common.serialize.SerializeException;
import com.ibm.team.repository.common.transport.HttpUtil.MediaType;

/**
 * Implementation of the deserialization interface for XML representations.
 */
public class XMLDeserializer extends AbstractDeserializer {

    /**
     * Constructs an XML deserializer.
     * 
     * @param type The media type associated to this deserializer. Must not
     * be <code>null</code>.
     * 
     * @param The serializer used to convert between objects and URIs. Must not
     * be <code>null</code>.
     */
    public XMLDeserializer(final MediaType type, IURISerializer uriSerializer) {
        super(type, uriSerializer);
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserialize(org.eclipse.emf.ecore.EObject, java.io.Reader)
     */
    public Object deserialize(final EObject object, final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("deserialize not implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeItem(java.io.Reader)
     */
    public IItem deserializeItem(final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("deserializeItem not implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeItems(java.io.Reader)
     */
    public List<IItem> deserializeItems(final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("deserializeItems not implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeNewItem(com.ibm.team.repository.common.IItemType, java.io.Reader)
     */
    public IItem deserializeNewItem(final IItemType itemType, final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("deserializeNewItem not implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeWorkingCopy(com.ibm.team.repository.common.IItem, java.io.Reader)
     */
    public IItem deserializeWorkingCopy(final IItem item, final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("deserializeWorkingCopy not implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeVirtual(java.io.Reader)
     */
    public IVirtual deserializeVirtual(final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("not yet implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeVirtuals(java.io.Reader)
     */
    public List<IVirtual> deserializeVirtuals(final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("not yet implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeVirtual(com.ibm.team.repository.common.IVirtualType, java.io.Reader)
     */
    public IVirtual deserializeVirtual(final IVirtualType type, final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("not yet implemented"); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see com.ibm.team.repository.common.serialize.IDeserializer2#deserializeVirtuals(com.ibm.team.repository.common.IVirtualType, java.io.Reader)
     */
    public List<IVirtual> deserializeVirtuals(final IVirtualType type, final Reader reader) throws SerializeException {
        throw new UnsupportedOperationException("not yet implemented"); //$NON-NLS-1$
    }
    
    
}
