/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.repository.transport.client;

import org.apache.http.auth.AuthScheme;
import org.apache.http.impl.auth.SPNegoSchemeFactory;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HttpContext;

/**
 * This is a workaround for a problem in the IBM JRE. The problem is tracked in PMR 50074,001,866. The solution has been fixed and tested on our end, i.e. we
 * can get rid of both {@link AdaptedSPNegoSchemeFactory} and {@link AdaptedSPNegoScheme} as soon as we have a guarantee that the JRE's are at a sufficient
 * support level. Here are the details on that suppport level:
 * 
 * The fix was delivered for Java 160_SR16FP3, 626_SR8FP3, 170_SR8FP1, and 727_SR2FP1. These versions will be released on the Q1 of 2015. The fix will be
 * available in ibmjgssprovider.jar (level 20141031).
 */
@SuppressWarnings("deprecation")
public class AdaptedSPNegoSchemeFactory extends SPNegoSchemeFactory {

    /**
     * 
     */
    public AdaptedSPNegoSchemeFactory() {
        super();
    }

    /**
     * @param stripPort
     */
    public AdaptedSPNegoSchemeFactory(boolean stripPort) {
        super(stripPort);
    }

    public AuthScheme newInstance(final HttpParams params) {
        return new AdaptedSPNegoScheme(this.isStripPort());
    }

    public AuthScheme create(final HttpContext context) {
        return new AdaptedSPNegoScheme(this.isStripPort());
    }
}
