/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.repository.transport.client;

import org.apache.http.Header;
import org.apache.http.HttpRequest;
import org.apache.http.auth.AuthenticationException;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.MalformedChallengeException;
import org.apache.http.impl.auth.AuthSchemeBase;
import org.apache.http.util.CharArrayBuffer;

/**
 *
 */
public class BearerScheme extends AuthSchemeBase {
    private final static String BEARER = "bearer"; //$NON-NLS-1$
    private final static String JSA_REALM = "jsa"; //$NON-NLS-1$

    public BearerScheme() {
    }

    @Override
    public String getSchemeName() {
        return BEARER;
    }

    @Override
    public String getParameter(String name) {
        return null;
    }

    @Override
    public String getRealm() {
        return JSA_REALM;
    }

    @Override
    public boolean isConnectionBased() {
        return false;
    }

    @Override
    public boolean isComplete() {
        return true;
    }

    @Override
    public Header authenticate(Credentials credentials, HttpRequest request) throws AuthenticationException {
        // return null means the Authorization header is not set to begin with. This will lead to a 401 which is reported back to the 
        // RemoteTeamServer as no other AuthScheme's will be attempted. Therefore, this avoid that credentials are sent to the RP
        return null;
    }

    @Override
    protected void parseChallenge(CharArrayBuffer buffer, int beginIndex, int endIndex) throws MalformedChallengeException {
        // nothing to parse
    }

}
