/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.transport.client;

import java.io.IOException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;

/**
 * @since 0.7
 */
public interface ITrustStoreProvider {

	/**
	 * Returns the store of all certificates trusted for the lifetime
	 * of this trust provider
	 */
	CertificateStore getSessionTrustStore();
	
	/**
	 * Returns the store of all permanently trusted
	 * certificates.
	 */
	CertificateStore getRuntimeTrustStore();
	
    /**
     * Install a certificate in the user's application-specific on-disk key
     * store, if possible.
     * 
     * @param cert
     * @throws NoSuchAlgorithmException
     * @throws CertificateException
     * @throws KeyStoreException
     * @throws IOException
     */
    public void installCertificate(Certificate cert) throws NoSuchAlgorithmException, CertificateException, KeyStoreException, IOException;
	
}
