/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.transport.client;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.nio.channels.SocketChannel;

import org.apache.http.HttpHost;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.protocol.HttpContext;

import com.ibm.team.repository.common.transport.TeamServiceCallContextImpl;

/**
 * A derivation of {@link InterruptableSocketFactory} in support of HttpClient 4.3.x
 */
public class InterruptableSocketFactory2 extends PlainConnectionSocketFactory {

    static private final LogUtil _logger = new LogUtil(InterruptableSocketFactory2.class);
    
    @Override
    public Socket createSocket(final HttpContext context) {
        _logger.debug("Entering createSocket"); //$NON-NLS-1$
        
        SocketChannel channel;
        try {
            channel = SocketChannel.open();
        } catch (IOException e) {
            throw new IllegalStateException(String.format("Unexpected IOException thrown: %s", e.getMessage())); //$NON-NLS-1$
        }
        
        Socket socket = channel.socket();
        _logger.debug("Created socket from channel"); //$NON-NLS-1$
        
        return socket;
    }

    @Override
    public Socket connectSocket(
            final int connectTimeout,
            final Socket sock,
            final HttpHost host,
            final InetSocketAddress remoteAddress,
            final InetSocketAddress localAddress,
            final HttpContext context) throws IOException
    {
        _logger.debug("Entering connectSocket"); //$NON-NLS-1$
        
        SocketChannel channel = sock.getChannel();
        if (channel == null)
            throw new IllegalStateException("Unexpected unavailable socket channel"); //$NON-NLS-1$
        
        channel.configureBlocking(true);
        Thread currentThread = Thread.currentThread();
        TeamServiceCallContextImpl.registerConnectingSocket(channel, currentThread);
        boolean connectionFailed = true;

        Socket newSock;
        try {
            newSock = InterruptableSocket2.newInterruptableSocket(sock, channel, context);
            channel.connect(remoteAddress);
            _logger.debug("Created interruptable socket and connected"); //$NON-NLS-1$
            connectionFailed = false;
        }
        finally {
            TeamServiceCallContextImpl.unregisterConnectingSocket(currentThread);           
            if (connectionFailed) {
                _logger.debug("Failed connecting socket"); //$NON-NLS-1$
                channel.close();
            }
        }
        
        return newSock;
    }

}