/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *
 */
package com.ibm.team.repository.transport.client;

import com.ibm.team.repository.common.transport.TeamServiceException;

/**
 * Thrown when the server does not return a response.
 * 
 * The server may have timed out, or otherwise become unavailable.
 */
@SuppressWarnings("serial")
public class NoResponseException extends TeamServiceException {

    /**
     * Constructs an instance of this class with the specified detail message.
     *
     * @param message The optional message.  May be <code>null</code>.
     */
	public NoResponseException(String message) {
		super(message);
	}
	
    /**
     * Creates an exception with an additional status/transport-layer status code.
     * 
     * @param message The optional message.  May be <code>null</code>.
     * @param statusCode The transport-layer status code.  The domain/range of
     * the value is assumed by the caller - there is no validation done here.
     */
    public NoResponseException(String message, int statusCode) {
        super(message, statusCode);
    }

    /**
     * Creates an exception with a message and cause.
     *
     * @param message The optional message.  May be <code>null</code>.
     * @param cause The optional originating cause. May be <code>null</code>.
     */
	public NoResponseException(String message, Throwable cause) {
		super(message, cause);
	}
	
    /**
     * Creates an exception with an additional status/transport-layer status code.
     *
     * @param message The optional message.  May be <code>null</code>.
     * @param cause The optional originating cause. May be <code>null</code>.
     * @param statusCode The transport-layer status code.  The domain/range of
     * the value is assumed by the caller - there is no validation done here.
     */
    public NoResponseException(String message, Throwable cause, int statusCode) {
        super(message, cause, statusCode);
    }
	
}
