/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.repository.transport.client;

import org.eclipse.core.runtime.IStatus;

/**
 * Abstract base implementation of {@link IOAuthHandler}
 * @since 0.7
 */
public abstract class OAuthHandler implements IOAuthHandler {
    
    /* (non-Javadoc)
     * @see com.ibm.team.repository.transport.client.IOAuthHandler#authorize(java.lang.String)
     */
    abstract public IResponse authorize(String authorizeURL);
    
    /**
     * Simple implementation of {@link IResponse}
     */
    static public class Response implements IResponse {

        private final String _callbackUri;
        private final IStatus _status;
        
        /**
         * Constructs a Response object with the specified callback string and status.
         * 
         * @param callbackUri The callback URI specified by the handler. 
         * Can be <code>null</code> if the authorization was cancelled or denied.
         * 
         * @param status The status associated to the response.
         * Can be <code>null</code> if the authorization was cancelled or denied.
         */
        public Response(String callbackUri, IStatus status) {
            _callbackUri = callbackUri;
            _status = status;
        }
        
        /* (non-Javadoc)
         * @see com.ibm.team.repository.transport.client.IOAuthHandler.IResponse#getCallbackURL()
         */
        public String getCallbackURL() {
            return _callbackUri;
        }

        /* (non-Javadoc)
         * @see com.ibm.team.repository.transport.client.IOAuthHandler.IResponse#getStatus()
         */
        public IStatus getStatus() {
            return _status;
        }
        
    }
}

