/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *
 */
package com.ibm.team.repository.transport.client;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.SocketException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLDecoder;
import java.net.UnknownHostException;
import java.nio.channels.UnresolvedAddressException;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;
import java.security.cert.X509Certificate;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.zip.GZIPInputStream;

import javax.net.ssl.KeyManager;
import javax.servlet.http.HttpServletResponse;

import org.apache.http.Consts;
import org.apache.http.Header;
import org.apache.http.HeaderElement;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.NoHttpResponseException;
import org.apache.http.ProtocolException;
import org.apache.http.RequestLine;
import org.apache.http.auth.AuthScheme;
import org.apache.http.auth.AuthSchemeProvider;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.AuthState;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.NTCredentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.AuthCache;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CookieStore;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.HttpRequestRetryHandler;
import org.apache.http.client.config.AuthSchemes;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.client.methods.HttpOptions;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.client.methods.HttpRequestWrapper;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.client.utils.HttpClientUtils;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.client.utils.URIUtils;
import org.apache.http.config.ConnectionConfig;
import org.apache.http.config.Lookup;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.config.SocketConfig;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.conn.routing.RouteInfo;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.auth.BasicSchemeFactory;
import org.apache.http.impl.auth.win.WindowsNegotiateScheme;
import org.apache.http.impl.auth.win.WindowsNegotiateSchemeFactory;
import org.apache.http.impl.client.BasicAuthCache;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.CookieSpecRegistries;
import org.apache.http.impl.client.DefaultRedirectStrategy;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.client.WinHttpClients;
import org.apache.http.impl.conn.DefaultProxyRoutePlanner;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpCoreContext;
import org.apache.http.util.EntityUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.xmlsoap.schemas.soap.envelope.Body;
import org.xmlsoap.schemas.soap.envelope.Envelope;
import org.xmlsoap.schemas.soap.envelope.Fault;

import com.ibm.team.repository.common.internal.marshal.MarshalFactory;
import com.ibm.team.repository.common.internal.marshal.MarshallerType;
import com.ibm.team.repository.common.internal.marshal.MarshallingException;
import com.ibm.team.repository.common.internal.marshal.MarshallingExceptionReasonCode;
import com.ibm.team.repository.common.internal.marshal.impl.WebServicesMarshaller;
import com.ibm.team.repository.common.internal.marshal.util.MarshallerUtil;
import com.ibm.team.repository.common.internal.util.ComponentConfigurationRegistry;
import com.ibm.team.repository.common.serialize.IDeserializer2;
import com.ibm.team.repository.common.serialize.SerializeException;
import com.ibm.team.repository.common.transport.AbstractTeamServer;
import com.ibm.team.repository.common.transport.ComponentConfiguration;
import com.ibm.team.repository.common.transport.ConnectionException;
import com.ibm.team.repository.common.transport.HostUnknownException;
import com.ibm.team.repository.common.transport.HostUtil;
import com.ibm.team.repository.common.transport.HttpUtil;
import com.ibm.team.repository.common.transport.HttpUtil.MediaType;
import com.ibm.team.repository.common.transport.ICertificateValidator;
import com.ibm.team.repository.common.transport.ITeamRestService;
import com.ibm.team.repository.common.transport.ITeamService;
import com.ibm.team.repository.common.transport.InsecureProtocolException;
import com.ibm.team.repository.common.transport.ServiceNotAvailableException;
import com.ibm.team.repository.common.transport.TeamServerConfiguration;
import com.ibm.team.repository.common.transport.TeamServiceException;
import com.ibm.team.repository.common.transport.UriUtil;
import com.ibm.team.repository.common.transport.VersionMismatchException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.transport.auth.TransportAuthException;
import com.ibm.team.repository.transport.internal.nls.Messages;

/**
 * @since 0.5
 */
public class RemoteTeamServer extends AbstractTeamServer {

    private static final String UNKNOWN = "UNKNOWN"; //$NON-NLS-1$ 

	private static final boolean DO_NOT_STRIP_PORT_FOR_KERBEROS = Boolean.getBoolean("com.ibm.team.repository.client.kerberos.donotstripport"); //$NON-NLS-1$

    // Externalized Strings
    private static final String NLS_REMOTE_TEAM_SERVER_CLOSING_CONNECTIONS = "_NoId.RemoteTeamServer.ClosingConnections"; //$NON-NLS-1$

    // this is attached to the user-agent to indicate we are spnego enabled
    public static final String SPNEGO_ENABLED = " spnego-enabled"; //$NON-NLS-1$
    private static final String JAZZ_NATIVE_CLIENT = "Jazz Native Client" + SPNEGO_ENABLED; //$NON-NLS-1$
    // String constants
    private static final String BACK_SLASH_REGEXP = "\\\\"; //$NON-NLS-1$
    private static final String OS_NAME_PROP = "os.name"; //$NON-NLS-1$
    private static final String OS_WINDOWS = "WINDOWS"; //$NON-NLS-1$
    private static final String COMPUTER_NAME_EV = "ComputerName"; //$NON-NLS-1$
    private static final String EMPTY_STRING = ""; //$NON-NLS-1$
    private final static String BEARER_CHALLENGE_NAME = "bearer"; //$NON-NLS-1$
    private final static String NEGOTIATE_CHALLENGE_NAME = "negotiate"; //$NON-NLS-1$
    private final static String BASIC_CHALLENGE_NAME = "basic"; //$NON-NLS-1$

    private final static String AuthSchemeBearer = "Bearer"; //$NON-NLS-1$
    private final static Collection<String> authSchemePreferences = Arrays
            .asList(new String[] { AuthSchemeBearer, AuthSchemes.SPNEGO, AuthSchemes.BASIC });

    // State relative to HTTP communication
    private volatile CloseableHttpClient _httpClient;
    private volatile CookieStore _cookieStore;
    private volatile CredentialsProvider _credsProvider;
    private volatile SecureInterruptableSocketFactory2 _secureSocketFactory;
    private volatile PoolingHttpClientConnectionManager _connectionMgr;

    // the default is USERIDPASSWD. This is mostly to cater for JUNITS
    private volatile AuthenticationType _authenticationType = AuthenticationType.USERIDPASSWD;

    // State relative to the origin server
    private volatile String _repoPath;
    private volatile HttpHost _targetHost;
    private volatile HttpHost _idpHost;
    private volatile HttpHost _proxy;

    // State relative to secure socket creation
    private volatile KeyManager[] _keyManagers;

    // Team server configuration information
    private volatile int _socketTimeout = 0;
    private volatile int _connectTimeout = 0;

    // The optional OAuth authorization challenge handler
    private volatile IOAuthHandler _oauthHandler;

    private final ICertificateValidator _certificateValidator;

    static private final LogUtil _logger = new LogUtil(RemoteTeamServer.class);
    
    /**
     * This registry keeps all created {@link WindowsNegotiateScheme} objects in a list, 
     * so they cannot be garbage-collected. When the WindowsNegotiateScheme#finalize() method
     * is called, it in turn calls {@link WindowsNegotiateScheme#dispose()} method. While
     * <code>dispose</code> can be called multiple times, when it is called by GC it sometimes 
     * causes a GPF crash from inside a native call made by JNA. The references to created 
     * schemes will be disposed of and cleared for garbage collection, when our http 
     * client is no longer needed. 
     * @author alexbern
     */
    private static class RtsWindowsNegotiateSchemeFactory extends WindowsNegotiateSchemeFactory
    {
    	ArrayList<WindowsNegotiateScheme> all = new ArrayList<WindowsNegotiateScheme>();
    	Object lock = new Object();
    	public RtsWindowsNegotiateSchemeFactory() {
			super( null );
		}
    	@Override
    	public AuthScheme create(HttpContext context) {
    		WindowsNegotiateScheme as = (WindowsNegotiateScheme) super.create(context);
    		synchronized (lock) {
    			all.add(as);
			}
    		return as;
    	}
		public void dispose() {
			synchronized (lock) {
				for (WindowsNegotiateScheme s : all ) {
					s.dispose();
				}
				all.clear();
			}
		}
    };
    
    RtsWindowsNegotiateSchemeFactory _windowsNegotiateSchemeFactory = null;

    /**
     * Construct an instance for the specified server connection without a certificate handler, such that problematic SSL certificates will be rejected.
     * 
     * @param urlString
     *            The team repository URI. Must not be <code>null</code> - must be absolute.
     */
    public RemoteTeamServer(String urlString) {
        this(urlString, null);
    }

    /**
     * Construct an instance for the specified server that will consult the specified <code>validator</code> regarding problematic SSL certificates.
     * 
     * @param urlString
     *            The team repository URI. Must not be <code>null</code> - must be absolute.
     * 
     * @param handler
     *            certificate handler, or <code>null</code> to reject all problematic SSL certificates.
     */
    public RemoteTeamServer(String urlString, ICertificateValidator validator) {
        super(urlString);
        setRepositoryURLUnsafe(urlString);
        _certificateValidator = validator;
    }

    /**
     * Resets the repository URL associated with this instance, causing all open connections to be closed and the mechanisms support network transport to be
     * reset.
     * 
     * @param urlString
     *            The repository URL for this instance. Must not be <code>null</code>.
     */
    @Override
    synchronized public void setRepositoryURL(String urlString)
    {
        _logger.debug("Entering setRepositoryURL"); //$NON-NLS-1$

        setRepositoryURLUnsafe(urlString);
        closeConnections();
    }

    /**
     * Unsynchronized setting of the repository URL associated to this server instance.
     * 
     * @param urlString
     *            The repository URL for this instance. Must not be <code>null</code>.
     */
    private void setRepositoryURLUnsafe(String urlString) {
        super.setRepositoryURL(urlString);
        URL url = null;
        try {
            url = new URL(urlString);
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException(String.format("Invalid URL \"%s\" given to setRepositoryURL: %s", urlString, e)); //$NON-NLS-1$
        }

        String host = url.getHost();
        if (host == null)
            throw new IllegalArgumentException("Repository URL missing host"); //$NON-NLS-1$

        int port = url.getPort();
        if (-1 == port)
            port = url.getDefaultPort();
        if (-1 == port) {
            throw new IllegalStateException("No port available or specified in repository URL"); //$NON-NLS-1$
        }

        _targetHost = new HttpHost(host, port, url.getProtocol());
        _repoPath = url.getPath();
    }

    /**
     * Returns the httpClient associated to this server instance, creating it if necessary.
     * 
     * @return The httpClient. Never <code>null</code>.
     */
    private CloseableHttpClient getHttpClient() {
        CloseableHttpClient result = _httpClient;
        if (result == null) {
            synchronized (this) {
                result = _httpClient;
                if (result == null) {
                    result = _httpClient = buildHttpClient();
                }
            }
        }

        _logger.debug("Entering getHttpClient: returning %s", result); //$NON-NLS-1$
        return result;
    }

    /**
     * Returns the global cookie store associated to this server instance, creating it if necessary. The creation is synchronized on <code>this</code>.
     * 
     * @return The cookie store. Never <code>null</code>.
     */
    private CookieStore getCookieStore() {
        CookieStore result = _cookieStore;
        if (result == null) {
            synchronized (this) {
                result = _cookieStore;
                if (result == null)
                    result = _cookieStore = new BasicCookieStore();
            }
        }

        _logger.debug("Entering getCookieStore: returning %s", result); //$NON-NLS-1$
        return result;
    }

    /**
     * Returns the credentials provider used by this server instance, creating it if necessary. 
     * The creation is synchronized on <code>this</code>.
     * 
     * @return The credentials provider. Never <code>null</code>.
     */
    private CredentialsProvider getCredsProvider() {
        CredentialsProvider result = _credsProvider;
        if (result == null) {
            synchronized (this) {
                result = _credsProvider;
                if (result == null)
                    result = _credsProvider = new BasicCredentialsProvider();
            }
        }

        return result;
    }

    /**
     * A custom redirect strategy that allows redirects on the following methods. The assumption here is that a redirect on a non-idempotent method is for the
     * purposes of authentication redirect, and will not cause the original method to replay. E.G. when a POST request is redirected by the team server, that is
     * because: a) the container is redirected from http to https; b) the caller is being redirected to an URL that requires authentication. The subsequent
     * return to the original request in that case will be a GET, which will get replayed in subsequent authentication logic in this class.
     */
    private static class AuthRedirectStrategy extends DefaultRedirectStrategy {

        static private String[] METHODS = new String[] {
                HttpDelete.METHOD_NAME,
                HttpGet.METHOD_NAME,
                HttpHead.METHOD_NAME,
                HttpOptions.METHOD_NAME,
                HttpPost.METHOD_NAME,
                HttpPut.METHOD_NAME
        };

        @Override
        protected boolean isRedirectable(String method) {
            _logger.debug("Entering isRedirectable: %s", method); //$NON-NLS-1$
            for (String name : METHODS)
                if (name.equalsIgnoreCase(method))
                    return true;

            return false;
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.apache.http.impl.client.DefaultRedirectStrategy#isRedirected(org.apache.http.HttpRequest, org.apache.http.HttpResponse,
         * org.apache.http.protocol.HttpContext)
         * 
         * Overridden here to not redirect on 301. The previous version of this rich client library did not redirect on 301s, and we're following that to
         * maintain compatibility (which at this time, appears to be only tests). Note also the the location header returned via the ContentMovedException on
         * the server is typically not one amenable to a GET redirect.
         * 
         * For example, in the test that I observed, a DELETE was sent to the a URL that mapped to the ItemRestService on a fronting-app, which failed with a
         * ContentMovedException, containing a location to the itemName/Contributor/ADMIN URI, which will map on the server to the ItemRenderService. The
         * ItemRenderService doesn't expect to be called with the rich client and service versioning information, and fails that versioning check.
         */
        @Override
        public boolean isRedirected(
                final HttpRequest request,
                final HttpResponse response,
                final HttpContext context) throws ProtocolException
        {
            final int statusCode = response.getStatusLine().getStatusCode();
            final String method = request.getRequestLine().getMethod();
            final Header locationHeader = response.getFirstHeader("location"); //$NON-NLS-1$

            _logger.debug("Entering isRedirected: statusCode=%d; method=%s; locationHeader=%s", statusCode, method, locationHeader); //$NON-NLS-1$

            switch (statusCode) {
            case HttpStatus.SC_MOVED_TEMPORARILY:
                /*
                 * If the original request used a non-secure URL (i.e. http instead of https), and the new Location does use a secure URL, we do not allow the
                 * redirect. This allows executeMethod(), below, to see the redirect and throw InsecureProtocolException, which is expected by callers when the
                 * server is configured to require secure connections. In 5.0.2 and earlier, redirects were followed manually, and this same logic was applied
                 * to each 302 response. Note that in 5.0.2 we did not check if the redirect target URI was secure. This is different now because we may have
                 * redirections for other reasons and if a server is set up in non-secure mode, this would cause problems with the other authentication
                 * protocols
                 */
                HttpHost targetHost = (HttpHost) context.getAttribute(HttpCoreContext.HTTP_TARGET_HOST);
                if (targetHost != null && HttpUtil.HTTP.equalsIgnoreCase(targetHost.getSchemeName())) {
                    if (locationHeader != null) {
                        URI redirectTarget = URI.create(locationHeader.getValue());
                        if (HttpUtil.HTTPS.equalsIgnoreCase(redirectTarget.getScheme())) {
                            return false;
                        }
                    }
                }

                return isRedirectable(method) && locationHeader != null;
            case HttpStatus.SC_TEMPORARY_REDIRECT:
                return isRedirectable(method);
            case HttpStatus.SC_SEE_OTHER:
                return true;
            default:
                return false;
            }
        }

        /**
         * This method is overridden so that any service version header can be stripped from the request when following redirects. It doesn't make sense to sent
         * the version for the originally requested server to redirect targets, because they will not be the same service, in general.
         */
        @Override
        public HttpUriRequest getRedirect(HttpRequest request, HttpResponse response, HttpContext context) throws ProtocolException {
            /*
             * The base DefaultRedirectStrategy class will generate the redirect request without any headers. Then, by default, the redirect executor class will
             * copy all the headers from the original request to the redirect request, but only if the redirect request doesn't have any headers. So we do the
             * copy ourselves, without the service version header.
             * 
             * The request should be wrapped, so we want the original headers, without the request-specific headers that would be added by protocol
             * interceptors.
             */
            Header[] originalHeaders;
            if (request instanceof HttpRequestWrapper) {
                originalHeaders = ((HttpRequestWrapper) request).getOriginal().getAllHeaders();
            } else {
                originalHeaders = request.getAllHeaders();
            }

            HttpUriRequest redirectRequest = super.getRedirect(request, response, context);
            redirectRequest.setHeaders(originalHeaders);

            Header serviceVersionHeader = redirectRequest.getFirstHeader(HttpUtil.X_SERVICE_VERSION);
            if (serviceVersionHeader != null) {
                redirectRequest.removeHeader(serviceVersionHeader);
            }

            return redirectRequest;
        }
    }

    /**
     * Returns a new instance of a configured httpclient.
     */
    private synchronized CloseableHttpClient buildHttpClient() {
        _logger.debug("Entering buildHttpClient"); //$NON-NLS-1$

        HttpClientBuilder builder = getHttpClientBuilder();

        
        /*
         * Pooling connection manager
         */
        if (_keyManagers == null)
            _secureSocketFactory = new SecureInterruptableSocketFactory2(_certificateValidator);
        else
            _secureSocketFactory = new SecureInterruptableSocketFactory2(_keyManagers, _certificateValidator);

        Registry<ConnectionSocketFactory> connectionRegistry = RegistryBuilder.<ConnectionSocketFactory> create()
                .register("http", new InterruptableSocketFactory2()) //$NON-NLS-1$
                .register("https", _secureSocketFactory) //$NON-NLS-1$
                .build();
        _connectionMgr = new PoolingHttpClientConnectionManager(connectionRegistry);

        // Socket config
        _connectionMgr.setDefaultSocketConfig(
                SocketConfig.custom()
                        .setSoLinger(-1)
                        .setTcpNoDelay(true)
                        .build());

        // Connection config
        _connectionMgr.setDefaultConnectionConfig(
                ConnectionConfig.custom()
                        .setBufferSize(8192)
                        .build());

        // The defaults as established by previous versions of RemoteTeamServer (which seem excessive,
        // but retained here for backwards compatibility.
        _connectionMgr.setDefaultMaxPerRoute(1000);
        _connectionMgr.setMaxTotal(1000);

        builder.setConnectionManager(_connectionMgr);

        /*
         * We override the auth registry to encode basic credentials as UTF-8
         */
        RegistryBuilder<AuthSchemeProvider> authSchemeRegistryBuilder = RegistryBuilder.<AuthSchemeProvider> create();

        // we are always at least supporting bearer
        authSchemeRegistryBuilder.register(AuthSchemeBearer, new BearerSchemeProvider());

        switch (this._authenticationType) {
        case WINDOWSINTEGRATED:
        	authSchemeRegistryBuilder.register(AuthSchemes.SPNEGO, getWindowsNegotiateSchemeFactory());
        	break;
        case KERBEROSSPNEGO:
            // if we use spnego, we don't want to handle 401s, the main code will catch this and throw the appropriate exception
            authSchemeRegistryBuilder.register(AuthSchemes.SPNEGO, new AdaptedSPNegoSchemeFactory(!DO_NOT_STRIP_PORT_FOR_KERBEROS));
            break;
        case USERIDPASSWD:
            // if we use user creds, we do not want to handle spnego, but deal with it in the main body, but we do want to support basic
            authSchemeRegistryBuilder.register(AuthSchemes.BASIC, new BasicSchemeFactory(Charset.forName("UTF-8"))); //$NON-NLS-1$
            break;
        default:
            // in all other cases, we don't want to deal with basic or spnego, notably clien creds
            break;
        }

        Lookup<AuthSchemeProvider> authSchemeRegistry = authSchemeRegistryBuilder.build();
        builder.setDefaultAuthSchemeRegistry(authSchemeRegistry);

        /*
         * Set the proxy route if a proxy has been specified
         */
        if (_proxy != null) {
            DefaultProxyRoutePlanner routePlanner = new DefaultProxyRoutePlanner(_proxy);
            builder.setRoutePlanner(routePlanner);
        }

        /*
         * Default request information.
         */
        builder.setDefaultRequestConfig(getRequestConfigBuilder().build());
        builder.setRedirectStrategy(new AuthRedirectStrategy());
        // if we use Windows Integrated, the builder comes with all the required stuff.
        if( !_authenticationType.equals( AuthenticationType.WINDOWSINTEGRATED )) 
        	builder.setDefaultCredentialsProvider(getCredsProvider());

        builder.setUserAgent(JAZZ_NATIVE_CLIENT);
        
        // See wi 368181. The PublicSuffixDomainFilter uses a default PublicSuffixMatcher
        // that rejects hostnames lacking a dot, such as "ccmserver", so needed
        // cookies don't get put on outgoing requests.
        // Here, we create a cookie spec registry with handlers that don't have a PublicSuffixMatcher.
        
        if (!Boolean.getBoolean("com.ibm.team.repository.transport.client.useDefaultPublicSuffixMatcher")) { //$NON-NLS-1$
            builder.setDefaultCookieSpecRegistry(CookieSpecRegistries.createDefault(null));
        }

		/* We occasionally see NoHttpResponseException.  Evidently this can
		 * happen when the server shuts down a connection but the client has not
		 * noticed yet.  One suggested workaround is to retry. */
		HttpRequestRetryHandler retryHandler = new HttpRequestRetryHandler() {
			@Override
			public boolean retryRequest(IOException exception, int executionCount, HttpContext context) {
				if (executionCount > 3) {
					_logger.debug("httpClient retry overflow"); //$NON-NLS-1$
					return false;
				} else if (exception instanceof NoHttpResponseException) {
					_logger.debug("httpClient retry on NoHttpResponseException"); //$NON-NLS-1$
					return true;
				} else
					return false;
			}
		};
		builder.setRetryHandler(retryHandler);

        return builder.build();
    }

	private RtsWindowsNegotiateSchemeFactory getWindowsNegotiateSchemeFactory() {
		if( _windowsNegotiateSchemeFactory == null )
			_windowsNegotiateSchemeFactory = new RtsWindowsNegotiateSchemeFactory();
		
		return _windowsNegotiateSchemeFactory;
	}

	private HttpClientBuilder getHttpClientBuilder() {
		HttpClientBuilder builder = _authenticationType.equals( AuthenticationType.WINDOWSINTEGRATED ) ?
				WinHttpClients.custom() : HttpClients.custom();
		return builder; 
	}

    /**
     * Return a builder for the default request configuration used in request execution. By default, redirects and authentication is enabled.
     * 
     * @return The builder. Never <code>null</code>.
     */
    @SuppressWarnings("deprecation")
	synchronized private RequestConfig.Builder getRequestConfigBuilder() {
        int[] timeouts = getTimeouts();
        return RequestConfig.custom()
                .setConnectionRequestTimeout(timeouts[0])
                .setConnectTimeout(timeouts[0]) // changed form timeouts[1]
                .setCookieSpec(CookieSpecs.BROWSER_COMPATIBILITY)
                .setSocketTimeout(timeouts[1])
                .setStaleConnectionCheckEnabled(true)
                .setAuthenticationEnabled(true)
                .setCircularRedirectsAllowed(true)
                .setTargetPreferredAuthSchemes(authSchemePreferences)
                .setRedirectsEnabled(true);
    }

    /**
     * Set the credentials for a kerberos-style login.
     * @deprecated. Use {@link #setCredentials(boolean)}
     */
    @Override
    synchronized public void setCredentials() {
    	setCredentials( false );
    }

    
    /**
     * Set the credentials for a kerberos-style login
     */
    @Override
    synchronized public void setCredentials( boolean useIntegratedWindows ) {
        super.setCredentials(EMPTY_STRING, EMPTY_STRING);
        _logger.debug("Entering setCredentials()"); //$NON-NLS-1$
        closeHttpClient(false);
        this._authenticationType = useIntegratedWindows ? AuthenticationType.WINDOWSINTEGRATED : AuthenticationType.KERBEROSSPNEGO;
        _logger.debug("Authentication type is set to " + _authenticationType.toString()); //$NON-NLS-1$
        this.setInternalCredentials( !useIntegratedWindows );
    }

    /**
     * Set the credentials userId/Password style login
     */
    @Override
    synchronized public void setCredentials(String userid, String password) {
        super.setCredentials(userid, password != null ? password : EMPTY_STRING);
        _logger.debug("Entering setCredentials(userid=%s, password=%s)", userid, password); //$NON-NLS-1$
        closeHttpClient(false);
        this._authenticationType = AuthenticationType.USERIDPASSWD;
        this.setInternalCredentials(!isBlank(userid));
    }

    private void resetInternalCredentials() {
        closeHttpClient(true);
        switch (this._authenticationType) {
        case WINDOWSINTEGRATED:
            _logger.debug("Resetting internal credentials for WINDOWSINTEGRATED"); //$NON-NLS-1$
            this.setInternalCredentials(false);
            break;
        case KERBEROSSPNEGO:
            _logger.debug("Resetting internal credentials for SPNEGO"); //$NON-NLS-1$
            this.setInternalCredentials(true);
            break;
        default:
            _logger.debug("Resetting internal credentials"); //$NON-NLS-1$
            this.setInternalCredentials(!isBlank(this.getUserid()));
            break;
        }
    }

    /**
     * The auth scope is derived from the _targetHost and is only based on the <code>host</code> - port,
     * realm and scheme are not considered
     */
    private void setInternalCredentials(boolean setTargetCredentials) {
        BasicCredentialsProvider newCredsProvider = new BasicCredentialsProvider();

        String userId = this.getUserid();
        String password = this.getPassword();

        if (setTargetCredentials) {
            if (this._idpHost != null) {
                AuthScope idpScope = new AuthScope(_idpHost.getHostName(), AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthScope.ANY_SCHEME);
                newCredsProvider.setCredentials(idpScope, new UsernamePasswordCredentials(userId, password));
            } else {
                AuthScope targetScope = new AuthScope(_targetHost.getHostName(), AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthScope.ANY_SCHEME);
                newCredsProvider.setCredentials(targetScope, new UsernamePasswordCredentials(userId, password));
            }
        }

        if (_proxy != null) {
            // carry-over any proxy
            Credentials proxyCreds = _credsProvider.getCredentials(new AuthScope(_proxy));
            if (proxyCreds != null) {
                newCredsProvider.setCredentials(new AuthScope(_proxy),
                        new UsernamePasswordCredentials(
                        		proxyCreds.getUserPrincipal().getName(),
                                proxyCreds.getPassword()));
            }
        }

        _credsProvider = newCredsProvider;
    }

    /**
     * Set the keystore and its password to be used for authentication
     */
    synchronized public void setCredentials(KeyManager[] keyManagers) {
        _logger.debug("Entering setCredentials(keyManagers)"); //$NON-NLS-1$
        super.setCredentials(EMPTY_STRING, EMPTY_STRING);
        closeHttpClient(false);
        this._authenticationType = AuthenticationType.CLIENTCERTS;
        _keyManagers = keyManagers;
    }

    /**
     * !!DO NOT USE!!
     * 
     * For internal use only by form-based auth class. Will be removed in subsequent releases. See Workitem 54754.
     * 
     * @deprecated
     */
    @Deprecated
    synchronized public String[] getHttpCredentials() {
        _logger.debug("Entering getHttpCredentials"); //$NON-NLS-1$

        String result[] = new String[2];

        UsernamePasswordCredentials storedCredentials = null;
        if (_credsProvider != null) {
            AuthScope scope = new AuthScope(_targetHost.getHostName(), AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthScope.ANY_SCHEME);
            storedCredentials = (UsernamePasswordCredentials) _credsProvider.getCredentials(scope);
        }

        if (storedCredentials == null)
            return result;

        result[0] = storedCredentials.getUserName();
        result[1] = storedCredentials.getPassword();
        if (result[0] == null)
            result[0] = ""; //$NON-NLS-1$
        if (result[1] == null)
            result[1] = ""; //$NON-NLS-1$
        return result;
    }

    /**
     * Build credentials for the specified user, attempting to create credentials that will support NTLM authentication as well as basic authentication if on
     * Windows.
     */
    private static Credentials buildProxyCredentials(String userName, String password) {
        _logger.debug("Entering getHttpCredentials"); //$NON-NLS-1$

        Credentials credentials = null;
        // Attempt to build credentials on Windows to support NTML auth
        // (These will still support basic auth)
        if (System.getProperty(OS_NAME_PROP).toUpperCase(Locale.ROOT).startsWith(OS_WINDOWS)) {
            // Get user's the domain name
            String domainName = ""; //$NON-NLS-1$
	            String[] s = userName.split(BACK_SLASH_REGEXP);
	            if (s.length == 2) {
	                domainName = s[0];
	                userName = s[1];
	            }
            // Get the local host name
            String hostName = System.getenv(COMPUTER_NAME_EV);
            if (hostName == null) {
                hostName = ""; //$NON-NLS-1$
            }
            credentials = new NTCredentials(userName, password, hostName, domainName);
            _logger.debug("Using NTCredentials"); //$NON-NLS-1$
        } else {
            credentials = new UsernamePasswordCredentials(userName, password);
            _logger.debug("Using UsernamePasswordCredentials"); //$NON-NLS-1$
        }
        return credentials;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.team.repository.common.transport.ITeamServer#setProxyServer(java.lang.String, int, java.lang.String, java.lang.String)
     */
    synchronized public void setProxy(String host, String type, int port, String userName, String password) {
        _logger.debug("setProxy() - host: %s - type: %s - port: %d - userName: %s - password: %s", host, type, port, userName, password); //$NON-NLS-1$

        closeConnections();

        if (host == null || port < 1) {
            _logger.debug("empty host or negative port given to setProxy"); //$NON-NLS-1$
            _proxy = null;       
            setInternalCredentials( !isBlank( getUserid()));
            return;
        }

        _proxy = new HttpHost(host, port, type);
        if (userName == null) {
        	getCredsProvider().clear();
        	_logger.debug("Cleared proxy credentials"); //$NON-NLS-1$
        } else {
        	getCredsProvider().setCredentials(new AuthScope(_proxy), buildProxyCredentials(userName, password));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.team.repository.common.transport.ITeamServer#setProxy(java.lang.String, int, java.lang.String, java.lang.String)
     */
    synchronized public void setProxy(String host, int port, String userName, String password) {
        setProxy(host, null, port, userName, password);
    }

    /**
     * Set the TeamServer configuration.
     */
    @Override
    synchronized public void setConfiguration(TeamServerConfiguration configuration) {
        super.setConfiguration(configuration);
        _logger.debug("Entering setConfiguration"); //$NON-NLS-1$

        closeConnections();
        _connectTimeout = configuration.getSocketTimeout();
        _socketTimeout = configuration.getSocketTimeout();
    }

    /**
     * Returns the configured connection and socket timeouts.
     * 
     * @return A two-element array with the connection timeout in slot 0, and socket timeout in slot 1.
     */
    synchronized private int[] getTimeouts() {

        int[] retval = new int[2];
        if (_connectTimeout != 0 && _socketTimeout != 0) {
            retval[0] = _connectTimeout;
            retval[1] = _socketTimeout;
        }
        else {
            TeamServerConfiguration config = new TeamServerConfiguration();
            retval[0] = config.getConnectTimeout();
            retval[1] = config.getSocketTimeout();
        }

        return retval;
    }

    @Override
    public ITeamService createTeamService(Class<?> serviceClass, Object implementation, String version,
            List<ComponentConfiguration> componentConfigurations)
    {
        _logger.debug("Entering createTeamService"); //$NON-NLS-1$

        if (null == serviceClass)
            throw new IllegalArgumentException("serviceClass must not be null"); //$NON-NLS-1$
        if (null != implementation)
            throw new IllegalArgumentException(NLS.bind("{0} implementation must be null", serviceClass)); //$NON-NLS-1$

        ClassLoader serviceClassLoader = serviceClass.getClassLoader();

        if (ITeamRestService.class.isAssignableFrom(serviceClass)) {
            _logger.debug("creating RemoteTeamRestService from %s", serviceClass.getName()); //$NON-NLS-1$
            return new RemoteTeamRestService(this, serviceClass, serviceClassLoader, version);
        }
        else {
            _logger.debug("creating RemoteTeamService from %s", serviceClass.getName()); //$NON-NLS-1$
            return new RemoteTeamService(this, serviceClass, serviceClassLoader, version, componentConfigurations);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.team.repository.common.transport.AbstractTeamServer#buildServiceImplementation(java.lang.Class)
     */
    @Override
    protected Object buildServiceImplementation(Class<?> serviceClass) {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.team.repository.common.transport.ITeamServer#isLocal()
     */
    public boolean isLocal() {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.team.repository.common.transport.ITeamServer#closeConnections()
     */
    public void closeConnections() {
        closeHttpClient(false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.ibm.team.repository.common.transport.ITeamServer#closeConnections()
     */
    public void closeConnections(boolean savePrefs) {
        closeConnections();
    }

    /**
     * Closes the HttpClient connections in the background, nullifying the httpclient instance and cookie store.
     */
    private void closeHttpClient(boolean preserveCookieStore) {
        _logger.debug("Entering closeHttpClient"); //$NON-NLS-1$
        if (_logger.isDebugEnabled())
            _logger.debug("Value of _httpclient: %s", _httpClient); //$NON-NLS-1$

        final PoolingHttpClientConnectionManager connectionManager;
        synchronized (this) {
            connectionManager = _connectionMgr;
            if( _windowsNegotiateSchemeFactory != null )
            	_windowsNegotiateSchemeFactory.dispose();
            _httpClient = null;
            if (!preserveCookieStore) {
                _cookieStore = null;
            }
            _secureSocketFactory = null;
            _connectionMgr = null;
            _keyManagers = null;
            // fall back to the default
            _authenticationType = AuthenticationType.USERIDPASSWD;
        }

        // when we nullify the http client, it may still be in use, so we have to wait a substantial time until we actually close the corresponding connection
        // manager in practice 10 seconds is enough
        long delay = 5000;

        /*
         * When RemoteTeamServer (RTS) is running in "local" test mode, it is shared between the client test/driver and the jetty server running in the same
         * process (Jetty's socket listeners are run in threads). Typically that shared RTS is mutated with one set of credentials when it starts the test,
         * passes that test to the Jetty server/thread, where that thread mutates the same RTS for a different set of credentials. Resetting those credentials
         * has the side effect of close the connections of the original request, which we want to stay open until the original client request completes. To
         * support that case, we extend the delay to start the closing of the original http client even further. Looking at some process tests that run in the
         * build as "local", some can push the 30 second mark, so we delay by 40 seconds.
         */

        String testType = System.getProperty("com.ibm.team.core.tests.repo"); //$NON-NLS-1$
        if ("local".equalsIgnoreCase(testType)) { //$NON-NLS-1$
            delay = 40000;
        }

        /*
         * Run the shutdown in the background using Eclipse's job scheduler.
         */
        Job job = new Job(Messages.getClientString(NLS_REMOTE_TEAM_SERVER_CLOSING_CONNECTIONS)) {
        	private final AtomicBoolean returnedAsync = new AtomicBoolean(false);
        	private final AtomicBoolean finishedAsync = new AtomicBoolean(false);

            @Override
            protected IStatus run(IProgressMonitor arg0) {
            	synchronized(RemoteTeamServer.this) {
            		// Run shutdown asynchronously so we can cancel it.
            		new Thread(new Runnable() {
            			@Override
            			public void run() {
            				_logger.debug("Starting HttpClient background close"); //$NON-NLS-1$

            				/*
            				 * Note that we get a reference to the connection manager directly, as we want the JVM to know that a reference to the connection manager
            				 * remains so that its finalizer is not called. In HC4.X, the finalizer for the connection manager calls shutdown, as we do in this job runner.
            				 * We want control over the shutdown - not the GC.
            				 */
            				connectionManager.shutdown();

            				_logger.debug("Finished HttpClient background close"); //$NON-NLS-1$
            				finishedAsync.set(true);
            				// Asynchronously mark the job as completed.
            				done(Status.OK_STATUS);
            			}
            		}).start();
            		returnedAsync.set(true);
                    return ASYNC_FINISH;
            	}
            }
            
            @Override
            public void canceling() {
            	synchronized(RemoteTeamServer.this) {
            		// Done can only be used after returning ASYNC_FINISH.
            		// Check returnedAsync to make sure this has been returned.
            		// Check finishedAsync to make sure done has not been called already.
            		if (returnedAsync.get() && !finishedAsync.getAndSet(true)) {
            			// Asynchronously mark the Job as canceled.
            			done(Status.CANCEL_STATUS);
            		}
            	}
    			super.canceling();
            }
        };
        job.setRule(RepositoryJobSchedulingRule.getWriteAccessRule(getRepositoryURL()));
        job.setPriority(Job.BUILD);

        if (connectionManager != null) {
            _logger.debug("Started eclipse background job to close the client"); //$NON-NLS-1$
            job.schedule(delay);
        } else {
            _logger.debug("httpclient already closed"); //$NON-NLS-1$
        }

    }

    /**
     * Returns the URL path of the encapsulated repository URL.
     * 
     * @return The repository URL path. Never <code>null</code>.
     */
    public String getRepositoryPath() {
        return _repoPath;
    }

    /**
     * @since 0.7
     */
    public X509Certificate[] getServerCertificatePath() {
        SecureInterruptableSocketFactory2 factory = _secureSocketFactory;
        if (factory == null) {
            return null;
        }
        return factory.getLastKnownServerCertificatePath();
    }

    /**
     * Sets the OAuth authorization challenge handler for this server.
     * 
     * @param handler
     *            The authorization challenge handler. May be <code>null</code>.
     */
    public void setOAuthHandler(IOAuthHandler handler) {
        _logger.debug("Entering setOAuthHandler - handler:%s", handler); //$NON-NLS-1$
        _oauthHandler = handler;
    }

    /**
     * Returns the OAuth authorization challenge handler for this server.
     * 
     * @return The authorization challenge handler for this server. May be <code>null</code>.
     */
    public IOAuthHandler getOAuthHandler() {
        return _oauthHandler;
    }

    /**
     * A class for string-izing HTTP headers.
     */
    static private class StringAdaptor extends Object {
        Header[] _headers;

        public StringAdaptor(Header[] headers) {
            _headers = headers;
        }

        @Override
        public String toString() {
            StringBuilder builder = new StringBuilder();
            for (Header header : _headers) {
                builder.append(header);
                builder.append('\n');
            }
            return builder.toString();
        }
    }

    /**
     * The preferred method for executing requests. By default exceptions are decoded/serialized. The response from the origin server, if available, is set on
     * the referenced <code>method</code>.
     * 
     * @param service
     *            The service associated to the execution context. May be <code>null</code>.
     * @param method
     *            The request/response pair to execute. Must not be <code>null</code>. The request associated to the method must not have a RequestConfig set.
     *            If an exception is not thrown as a result of the execution, <code>method</code> will contain an HttpResponse object.
     * @param handle404
     *            Whether or not this routine should throw an exception for 404 status returned from the origin server.
     * @param wrapIOException
     *            Whether or not IOExceptions should be wrapped in TeamServiceExceptions.
     * @return The final http status code returned by the origin server. The same value available in response encapsulated by <code>method</code>.
     * 
     * @throws IOException
     * @throws TeamServiceException
     */
    public int executeMethod(RemoteTeamService service, RequestResponsePair method, boolean handle404,
            boolean wrapIOException) throws IOException, TeamServiceException
    {
        return executeMethod(service, method, handle404, wrapIOException, true);
    }

    /**
     * The preferred method for executing requests. By default exceptions are decoded/serialized. The response from the origin server, if available, is set on
     * the referenced <code>method</code>.
     * 
     * @param service
     *            The service associated to the execution context. May be <code>null</code>.
     * @param method
     *            The request/response pair to execute. Must not be <code>null</code>. The request associated to the method must not have a RequestConfig set.
     *            If an exception is not thrown as a result of the execution, <code>method</code> will contain an HttpResponse object.
     * @param handle404
     *            Whether or not this routine should throw an exception for 404 status returned from the origin server.
     * @param wrapIOException
     *            Whether or not IOExceptions should be wrapped in TeamServiceExceptions.
     * @param decodeExceptions
     *            Whether or not origin response exceptions should be decoded and thrown. <code>true</code> to decode/throw exceptions, <code>false</code> to
     *            return then in the method's response body.
     * @return The final http status code returned by the origin server. The same value available in response encapsulated by <code>method</code>.
     * 
     * @throws IOException
     * @throws TeamServiceException
     */
    public int executeMethod(RemoteTeamService service, RequestResponsePair method, boolean handle404,
            boolean wrapIOException, boolean decodeExceptions) throws IOException, TeamServiceException
    {
        _logger.debug("Entering executeMethod - decodeExceptions == %s", Boolean.toString(decodeExceptions)); //$NON-NLS-1$

        /*
         * The caller is not allow to set a request config. If one is set, err-out.
         */
        if (method.getRequest().getConfig() != null) {
            throw new IllegalStateException("Callers of executeMethod are not allowed to set a RequestConfig on the request"); //$NON-NLS-1$
        }

        /*
         * Associate our cookie store with the request (for previous authentication state). Save the original request in case it needs to be replayed.
         */
        HttpClientContext httpContext = HttpClientContext.create();
        method.setContext(httpContext);
        httpContext.setCookieStore(getCookieStore());
        httpContext.setAttribute(SERVICE_CONTEXT, service);

        /*
         * Override the default request configuration, in case the socket timeout has been changed.
         */
        RequestConfig requestConfig = getRequestConfigBuilder().setSocketTimeout(getConfiguration().getSocketTimeout()).build();
        httpContext.setRequestConfig(requestConfig);

        // now execute request and handle challenges
        this.executeWithAuthHandling(method, httpContext, wrapIOException, 0);

        int statusCode = method.getResponse().getStatusLine().getStatusCode();

        _logger.debug("Execute with authHandler returned " + statusCode); //$NON-NLS-1$

        /*
         * Special case for the JTS proxy. If the response is coming from the target of the proxy, ignore the status checking here
         * (https://jazz.net/wiki/bin/view/Main/JFAMAuthProxy) In support of tests only.
         */
        if (isResponseFromProxyTarget(method.getRequest())) {
            _logger.debug("Response from JFAM proxy - exiting"); //$NON-NLS-1$
            return statusCode;
        }

        switch (statusCode) {

        case HttpServletResponse.SC_UNAUTHORIZED: {
            _logger.debug("Handling SC_UNAUTHORIZED"); //$NON-NLS-1$

            ChallengeType challengeType = method.getChallengeType();

            String message;
            AuthenticationException exp;
            switch (this._authenticationType) {
            case KERBEROSSPNEGO:
                switch (challengeType) {
                case SPNEGO:
                    message = NLS.bind(Messages.getClientString(NLS_KERBEROS_TICKET_INVALID), method.getRequest().getURI());
                    exp = new KerberosInvalidTicketException(message, statusCode);
                    break;
                default:
                    // if the server comes back with a 401 which is not negotiate, it simply cannot handle Kerberos.
                    // Note that we never engage fallback if we are not using user credentials, so e.g. a 401/basic would never be the
                    // result of fallback in this situation
                    message = NLS.bind(Messages.getClientString(NLS_NO_TARGET_KERBEROS), method.getRequest().getURI());
                    exp = new NoKerberosTargetException(message, statusCode);
                    break;
                }
                break;
            case USERIDPASSWD:
                switch (challengeType) {
                case BASIC:
                    // trying to use user creds, but if we now see a 401, it means the provided creds are wrong
                    message = NLS.bind(Messages.getClientString(NLS_INVALID_CREDENTIALS), method.getRequest().getURI());
                    exp = new InvalidUserCredentialsException(message, statusCode);
                    break;
                default:
                    // any other challenge at this point implies user credentials cannot be used
                    message = NLS.bind(Messages.getClientString(NLS_USERCREDENTIALS_NOT_POSSIBLE), method.getRequest().getURI());
                    exp = new NoUserCredentialsAcceptedException(message, statusCode);
                    break;
                }

                break;
            case CLIENTCERTS:
                // The user is trying to use client certificates, but if we see a 401 of any kind
                // it means the server does not accept the certificate
                message = NLS.bind(Messages.getClientString(NLS_CLIENTCERTIFICATE_NOT_ACCEPTED), method.getRequest().getURI());
                exp = new ClientCertificateNotAcceptedException(message, statusCode);
                break;
            default:
                // should not be possible
                message = NLS.bind(Messages.getClientString(NLS_UNAUTHORIZED_ERROR), method.getRequest().getURI());
                exp = new AuthenticationException(message, statusCode);
                break;
            }
            exp.setRequestUri(getAppropriateRequestURL(httpContext));
            exp.addHeaders(getHttp3ResponseHeaders(httpContext));
            throw exp;
        }

        case HttpServletResponse.SC_FORBIDDEN: {
            _logger.debug("Handling SC_FORBIDDEN"); //$NON-NLS-1$
            String message = NLS.bind(Messages.getClientString(NLS_FORBIDDEN_ERROR), method.getRequest().getURI());
            AuthenticationException exp = new InsufficientRightsException(message, statusCode);

            exp.setRequestUri(getAppropriateRequestURL(httpContext));
            exp.addHeaders(getHttp3ResponseHeaders(httpContext));
            throw exp;
        }

        case HttpServletResponse.SC_NOT_IMPLEMENTED: {
            _logger.debug("Handling SC_NOT_IMPLEMENTED"); //$NON-NLS-1$
            String message = NLS.bind(Messages.getClientString(NLS_NOT_IMPLEMENTED_ERROR), method.getRequest().getURI());
            ServiceNotAvailableException exp = new ServiceNotAvailableException(message, statusCode);
            exp.setRequestUri(getAppropriateRequestURL(httpContext));
            exp.addHeaders(getHttp3ResponseHeaders(httpContext));
            throw exp;
        }

        case HttpServletResponse.SC_SERVICE_UNAVAILABLE: {
            _logger.debug("Handling SC_SERVICE_UNAVAILABLE"); //$NON-NLS-1$
            String message = NLS.bind(Messages.getClientString(NLS_UNAVAILABLE_ERROR), method.getRequest().getURI());
            ServiceNotAvailableException exp = new ServiceNotAvailableException(message, statusCode);
            exp.setRequestUri(getAppropriateRequestURL(httpContext));
            exp.addHeaders(getHttp3ResponseHeaders(httpContext));
            throw exp;
        }

        case HttpServletResponse.SC_MOVED_TEMPORARILY: {
            _logger.debug("Handling SC_MOVED_TEMPORARILY"); //$NON-NLS-1$
            /*
             * Per Workitem 49159, check to see if the requested scheme is http, and suggest that failed authentication might be the issue.
             */
            String protocol = _targetHost.getSchemeName();
            if (protocol.compareToIgnoreCase(HttpUtil.HTTP) == 0) {
                String message = NLS.bind(Messages.getClientString("ClientHttpUtil.ErrorInsecureHttpConnection"), //$NON-NLS-1$
                        method.getRequest().getURI(), HttpUtil.HTTP, HttpUtil.HTTPS);
                throw new InsecureProtocolException(message, statusCode);
            }
            else {
                throwHttpClientError(httpContext, decodeExceptions);
                throw new RuntimeException("Should never happen, above call to throwHttpClientError will always throw an exception"); //$NON-NLS-1$
            }
        }
        default:
            if (!handle404 && (404 == statusCode)) {
                _logger.debug("Immediately returning 404"); //$NON-NLS-1$
                return statusCode;
            }

            if (statusCode >= 300) {
                _logger.debug("Other > 300 status: " + statusCode); //$NON-NLS-1$
                throwHttpClientError(httpContext, decodeExceptions);
            }
        }

        return statusCode;
    }

    /**
     * If we reach this stage, we know that auth-code flow involves a challenge which needs to be dealt with. 
     * However, we do not know what kind of challenge. The strategy is to first execute the auth code flow 
     * again <b>without</b> prompt=none, where we make sure that the http client is able to react to known
     * 401 challenges from the OP, e.g. if it was configured for spnego or basic. This is done by setting the
     *  _idpHost and credentials for it (we also turn off the credentials for the RP at this point if not 
     *  already done). If we get a 200, we are either getting a form challenge from the OP or and IdP in which
     * case we will retry the flow by explicitly setting the credentials for the _idpHost. If the IdP is not 
     * able to deal with basic, the OP is expected to have configured an authentication instruction handler
     * which sends back JSON with a special header to recognize. The rich client will then follow those
     * instructions instead.
     */
    private void handleAuthorizationServerChallenge(HttpHost opHost, HttpGet authorizationRequest, RequestResponsePair method,
            HttpClientContext httpContext, boolean wrapIOException) throws TeamServiceException, IOException {
        // we set the idpHost to be the OP for now so we can react to challenges from it
        _idpHost = opHost;

        // we reset the credentials, which this time will set them for the IdP, not the RP
        this.resetInternalCredentials();

        // now we re-execute the auth code flow *without* prompt=none
        _logger.debug("OIDC resending authorization request without prompt = none"); //$NON-NLS-1$
        CloseableHttpResponse closeableResponse = executePrimitiveRequest(authorizationRequest, opHost, httpContext, wrapIOException, false);
        method.setResponse(closeableResponse);

        if (closeableResponse.getStatusLine().getStatusCode() == HttpServletResponse.SC_UNAUTHORIZED) {
            // in the case of spnego and the desire to use user credentials (or possible client certificates), we need to check for fallback
            ChallengeType challengeType = getChallengeType(method);
            _logger.debug("OIDC challenge type is %s", challengeType); //$NON-NLS-1$
            if (challengeType == ChallengeType.SPNEGO && this._authenticationType != AuthenticationType.KERBEROSSPNEGO) {
                this.handleSpnegoChallengeForNonSpnegoConnection(method, httpContext, wrapIOException);
            }
        }

        // so if we see a 200 at this stage, we have 2 possibilities 1) we are dealing with a FORM produced by the OP or some other IDP
        // in which case we will simply set http client to send Basic Creds to the last request preemptively or 2) we are given instructions by the OP
        // on how to deal with SAML (or some other unconventional IdP).
        if (closeableResponse.getStatusLine().getStatusCode() == HttpServletResponse.SC_OK) {
            // TODO option 2) is not yet implemented, so for now, we always assume we are in the option 1) scenario

            HttpClientUtils.closeQuietly(closeableResponse);

            // fetch the last request from the context (note that this is not necessarily the
            _idpHost = httpContext.getTargetHost();

            // we again reset the credentials because the idpHost may have changed once more, this time possibly to an external IdP
            this.resetInternalCredentials();

            // on top of that, for the duration of this context, we will send the basic credentials to any request towards the IdP
            AuthCache authCache = new BasicAuthCache();
            authCache.put(_idpHost, new BasicScheme(Charset.forName("UTF-8"))); //$NON-NLS-1$
            CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
            AuthScope idpScope = new AuthScope(_idpHost.getHostName(), AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthScope.ANY_SCHEME);
            credentialsProvider.setCredentials(idpScope, new UsernamePasswordCredentials(this.getUserid(), this.getPassword()));
            httpContext.setAuthCache(authCache);
            httpContext.setCredentialsProvider(credentialsProvider);
            httpContext.setAttribute(HttpClientContext.TARGET_AUTH_STATE, new AuthState());
            closeableResponse = executePrimitiveRequest(authorizationRequest, opHost, httpContext, wrapIOException, true);
            method.setResponse(closeableResponse);
            httpContext.removeAttribute(HttpClientContext.AUTH_CACHE);
        }

    }

    /**
     * If we detect a 401 with WWW-Authenticate Bearer, we initiate an auth-code flow with prompt=none. If this succeeds, we do not have to deal with
     * authorization server challenges. If this fails, we figure out how the authorization server (OP) wants to deal with the challenge. For example, it may
     * come from an external IdP such as a SAML provider.
     */
    private void handleBearerChallenge(RequestResponsePair method, HttpClientContext httpContext, boolean wrapIOException) throws TeamServiceException,
            IOException {
        // first we make a primitive request to the provided auth code flow header with prompt=none.
        CloseableHttpResponse response = method.getResponse();

        // construct request for initial authorization request with prompt=none
        Header opAuthorizationRequestHeader = response.getFirstHeader(AUTHORIZATION_REDIRECT_URL_HEADER);

        // if the header were to be null because target application is not developed with the OIDC SDK, we return and let the 401 reach the client
        if (opAuthorizationRequestHeader == null) {
            _logger.debug("Bearer challenge received but target not developed with OIDC SDK"); //$NON-NLS-1$
            return;
        }

        try {
            URIBuilder authorizationRequestUriBuilder = new URIBuilder(opAuthorizationRequestHeader.getValue());
            HttpClientUtils.closeQuietly(response);

            // auth code flow uri without prompt=none
            URI authorizationRequestUri = authorizationRequestUriBuilder.build();
            _logger.debug("OIDC Authorization uri: %s", authorizationRequestUri); //$NON-NLS-1$

            // auth code flow uri with prompt=none
            URIBuilder authorizationRequestUriBuilderWithNoPrompt = authorizationRequestUriBuilder.addParameter("prompt", "none"); //$NON-NLS-1$ //$NON-NLS-2$
            // get the OP host
            HttpHost opHost = new HttpHost(authorizationRequestUriBuilder.getHost(), authorizationRequestUriBuilder.getPort(),
                    authorizationRequestUriBuilder.getScheme());

            _logger.debug("OIDC OP Host: %s", opHost); //$NON-NLS-1$

            // (1) first make a call with prompt=none
            HttpGet noPromptAuthorizationRequest = new HttpGet(authorizationRequestUriBuilderWithNoPrompt.build());
            _logger.debug("OIDC Authorization uri with prompt is none : %s", noPromptAuthorizationRequest.getURI()); //$NON-NLS-1$

            // This request will fail if this client is connecting for the first time. It may succeed
            // if an OP session is already in place in which case some previous level-setting had already been done
            CloseableHttpResponse closeableResponse = executePrimitiveRequest(noPromptAuthorizationRequest, opHost, httpContext, wrapIOException, false);
            method.setResponse(closeableResponse);
            if (closeableResponse.getStatusLine().getStatusCode() == HttpServletResponse.SC_OK) {
                _logger.debug("OIDC Authorization request with prompt is none returned 200"); //$NON-NLS-1$
                // check if we have to deal with challenge
                Header login_required_header = closeableResponse.getFirstHeader(AUTHORIZATION_LOGIN_REQUIRED_HEADER);
                if (login_required_header != null) {
                    _logger.debug("OIDC login required header returned"); //$NON-NLS-1$
                    // close the response
                    HttpClientUtils.closeQuietly(closeableResponse);

                    // (1) this means we haven't yet signed into the authorization server, so deal with this challenge
                    HttpGet authorizationRequest = new HttpGet(authorizationRequestUri);
                    this.handleAuthorizationServerChallenge(opHost, authorizationRequest, method, httpContext,
                            wrapIOException);

                } // otherwise, we are good and reached the protected resource
                _logger.debug("OIDC auth sequence reached protected resource"); //$NON-NLS-1$
            } else {
                // if there is another problem, we simply let it propagate
                _logger.debug("OIDC Authorization request with prompt is none returned %s", closeableResponse.getStatusLine().getStatusCode()); //$NON-NLS-1$
            }

        } catch (URISyntaxException e) {
            throw new IOException(e);
        }

    }

    /**
     * If the server indicates it handles SPNeGO, but the client has not been set up to use SPNeGO, we will re-execute with a bad SPNeGO authorization to permit
     * a possible fall-back by the server. There are four possibilities, but only 1 needs special attention:
     * <ul>
     * <li>The server is not enabled for fallback: The response will be a 403. This will lead to an exception downstream, but we track this scenario to improve
     * the error message</li>
     * <li>The server is enabled for fallback but has client certificates: The response will be the desired result, so we let things go through</li>
     * <li>The server is enabled for fallback with FORM: The response will be a FORM challenge, which is handled downstream</li>
     * <li>The server is enabled for fallback with BASIC: The response will be a 401 with WWW-Authenticate set to BASIC. Normally, basic challenges are handled
     * by Apache's auth scheme, but in this scenario, the original presence of the Authorization:Negotiate header confuses the scheme handling so authentication
     * fails. In that case, we simply actively set the Basic auth header</li>
     * </ul>
     */
    private void handleSpnegoChallengeForNonSpnegoConnection(RequestResponsePair method, HttpClientContext httpContext, boolean wrapIOException)
            throws TeamServiceException, IOException {

        _logger.debug("Handling SPNEGO challenge for non-SPNEGO connection type"); //$NON-NLS-1$
        // obtain request/response
        HttpRequestBase originalRequest = method.getRequest();
        CloseableHttpResponse closeableResponse = method.getResponse();

        // close old response
        HttpClientUtils.closeQuietly(closeableResponse);

        // add Authorization:Negotiate to force trigger fallback
        originalRequest.setHeader(HttpHeaders.AUTHORIZATION, NEGOTIATE_CHALLENGE_NAME);
        _logger.debug("Resending request %s with authorization header %s", originalRequest.getURI(), NEGOTIATE_CHALLENGE_NAME); //$NON-NLS-1$
        closeableResponse = executePrimitiveRequest(originalRequest, _targetHost, httpContext, wrapIOException, true);

        // update the method
        method.setResponse(closeableResponse);

        int status = closeableResponse.getStatusLine().getStatusCode();

        if (status == HttpStatus.SC_FORBIDDEN) {

            switch (this._authenticationType) {
            case USERIDPASSWD:
                // if we see a 403 at this point, we either have spnego with no fallback or the server demands a client certificate
                // unfortunately, we cannot know which one it is, so we simply state that user credentials are not accepted

                String message = NLS.bind(Messages.getClientString(NLS_USERCREDENTIALS_NOT_POSSIBLE), method.getRequest().getURI());
                AuthenticationException exp = new NoUserCredentialsAcceptedException(message, 403);
                exp.setRequestUri(getAppropriateRequestURL(httpContext));
                exp.addHeaders(getHttp3ResponseHeaders(httpContext));
                throw exp;
            case CLIENTCERTS:
                // if we see a 403 at this point, we either have spnego with no fallback or the provided client certificate was not acceptable
                // NOTE: NORMALLY, client certificates are immediately accepted by the server. However, we have seen customers where that
                // was not the case, for some reason, the first request ignored it, and the second took it. Still, in those cases, we should not
                // be hitting a negotiate challenge yet because of how the foundation server handles certificates. However, because Websphere
                // by default permits fallback to certificates, we'll defensively cater for this scenario

                message = NLS.bind(Messages.getClientString(NLS_CLIENTCERTIFICATE_NOT_ACCEPTED), method.getRequest().getURI());
                exp = new ClientCertificateNotAcceptedException(message, 403);
                exp.setRequestUri(getAppropriateRequestURL(httpContext));
                exp.addHeaders(getHttp3ResponseHeaders(httpContext));
                throw exp;
            default:
                // not possible
                throw new TeamServiceException("Unexpected authentication type leading to a 403 : " + this._authenticationType); //$NON-NLS-1$
            }
        } else if (status == HttpStatus.SC_UNAUTHORIZED) {

            ChallengeType challengeType = getChallengeType(method);
            _logger.debug("Challenge type is %s ", challengeType); //$NON-NLS-1$
            switch (challengeType) {
            case BASIC:
                // close old response
                HttpClientUtils.closeQuietly(closeableResponse);

                // if we see a 401/basic, we got a fallback to basic

                // massage the HttpContext to pro-actively set the basic auth header for this request
                AuthCache authCache = new BasicAuthCache();
                authCache.put(_targetHost, new BasicScheme(Charset.forName("UTF-8"))); //$NON-NLS-1$
                CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
                AuthScope idpScope = new AuthScope(_targetHost.getHostName(), AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthScope.ANY_SCHEME);
                credentialsProvider.setCredentials(idpScope, new UsernamePasswordCredentials(this.getUserid(), this.getPassword()));
                httpContext.setAuthCache(authCache);
                httpContext.setCredentialsProvider(credentialsProvider);
                httpContext.setAttribute(HttpClientContext.TARGET_AUTH_STATE, new AuthState());
                originalRequest.removeHeaders(HttpHeaders.AUTHORIZATION);
                _logger.debug("Pro-actively set basic credentials and re-executed original request"); //$NON-NLS-1$
                closeableResponse = executePrimitiveRequest(originalRequest, _targetHost, httpContext, wrapIOException, true);
                httpContext.removeAttribute(HttpClientContext.AUTH_CACHE);

                // update the method
                method.setResponse(closeableResponse);
                break;
            default:
                // all other cases we propagate up (not sure if they are even possible)
                break;
            }
        }
    }

    private void executeWithAuthHandling(RequestResponsePair method, HttpClientContext httpContext, boolean wrapIOException, int attempts)
            throws TeamServiceException, IOException {
        HttpRequestBase originalRequest = method.getRequest();
        if (_logger.isDebugEnabled()) {
            _logger.debug("Value of originalRequest URI: %s", originalRequest.getRequestLine()); //$NON-NLS-1$
            _logger.debug("Value of originalRequest headers:\n%s", new StringAdaptor(originalRequest.getAllHeaders())); //$NON-NLS-1$
        }

        /*
         * Execute the request and save the response
         */

        CloseableHttpResponse closeableResponse = executePrimitiveRequest(originalRequest, _targetHost, httpContext, wrapIOException, true);
        _logger.debug("Finished executing primitive request"); //$NON-NLS-1$
        method.setResponse(closeableResponse);
        int status = closeableResponse.getStatusLine().getStatusCode();

        if (status == HttpServletResponse.SC_UNAUTHORIZED) {

            ChallengeType challengeType = getChallengeType(method);
            _logger.debug("ChallengeType is " + challengeType); //$NON-NLS-1$

            switch (challengeType) {
            case BEARER:
                this.handleBearerChallenge(method, httpContext, wrapIOException);
                break;
            case SPNEGO:
                if (this._authenticationType != AuthenticationType.KERBEROSSPNEGO) {
                    // if we have user creds or client creds, we need to permit fallback
                    this.handleSpnegoChallengeForNonSpnegoConnection(method, httpContext, wrapIOException);

                    // note that fallback may lead to redirections and FORM challenges, so don't return but permit
                    // code to proceed to handleAuthentication

                } else {
                    // in all other cases, we immediately return since we have a 401.
                    // Also note that for client certificates, if we reach a 401/negotiate, client certificates were never an option
                    return;
                }

                break;
            default:
                // nothing to do in all other cases, but note that with a 401, we should simply return at this point
                return;
            }
        }

        /*
         * Handle any legacy authentication challenges, and capture the final status.
         */
        handleAuthentication(method, httpContext, wrapIOException, attempts);

    }

    /**
     * Prints HttpContext information to the logger's debugging stream.
     * 
     * @param httpContext
     *            The context to print. Must not be <code>null</code>.
     */
    static private void debug(String msg, HttpClientContext httpContext) {
        if (!_logger.isDebugEnabled())
            return;

        HttpResponse contextResponse = httpContext.getResponse();
        HttpRequest finalRequest = httpContext.getRequest();
        HttpHost contextHost = httpContext.getTargetHost();
        RouteInfo httpRoute = httpContext.getHttpRoute();
        RequestConfig requestConfig = httpContext.getRequestConfig();

        if (msg != null)
            _logger.debug(msg);

        if (finalRequest != null)
            _logger.debug("Value of request: %s", finalRequest.getRequestLine()); //$NON-NLS-1$
        if (contextResponse != null)
            _logger.debug("Value of context response headers:\n%s", new StringAdaptor(contextResponse.getAllHeaders())); //$NON-NLS-1$
        if (contextHost != null)
            _logger.debug("Value of context host: %s", contextHost); //$NON-NLS-1$
        if (httpRoute != null)
            _logger.debug("Value of route info: %s", httpRoute); //$NON-NLS-1$
        if (requestConfig != null)
            _logger.debug("Value of request config: %s", requestConfig); //$NON-NLS-1$
    }

    public enum ChallengeType {
        SPNEGO, BEARER, BASIC, NA
    }

    /**
     * We always give priority to bearer or spnego, so if we encounter basic, we keep looking for bearer or spnego. However, if we encounter bearer or spnego,
     * we give that priority and quit.
     * 
     * The result is never null
     */
    private ChallengeType getChallengeType(RequestResponsePair method) {

        // by default, the challenge type is NA (this avoids numerous null-checks)
        ChallengeType challengeType = ChallengeType.NA;
        CloseableHttpResponse response = method.getResponse();
        Header[] wwwAuthHeaders = response.getHeaders(HttpHeaders.WWW_AUTHENTICATE);
        for (Header wwwAuthHeader : wwwAuthHeaders) {
            String challenge = wwwAuthHeader.getValue().toLowerCase(Locale.ENGLISH);
            if (challenge.startsWith(NEGOTIATE_CHALLENGE_NAME)) {
                challengeType = ChallengeType.SPNEGO;
                break;
            }
            if (challenge.startsWith(BEARER_CHALLENGE_NAME)) {
                challengeType = ChallengeType.BEARER;
                break;
            }
            if (challenge.startsWith(BASIC_CHALLENGE_NAME)) {
                challengeType = ChallengeType.BASIC;
            }
        }

        method.setChallengeType(challengeType);
        return challengeType;
    }

    private final static int MAXIMUM_REPLAY_ATTEMPTS = 10; // in theory the number of replays required is unbounded because in practice, replays of this kind
                                                           // are caused by concurrency and it is
                                                           // very unlikely that as many a 10 replay attempts are ever required. However, we put the bound in to
                                                           // avoid an infinite loop if a server
                                                           // is misbehaving

    /**
     * Checks the response on the referenced method for authentication challenges, responding to those challenges and updating the response on
     * <code>method</code> accordingly.
     * 
     * @param method
     *            The method to inspect. Must not be <code>null</code>.
     * @param httpContext
     *            The httpContext state to share across invocations.
     * @param wrapIOException
     *            Whether or not to wrap IO exceptions.
     */
    private void handleAuthentication(RequestResponsePair method, HttpClientContext httpContext, boolean wrapIOException, int attempts)
            throws TeamServiceException, IOException
    {
        _logger.debug("Entering handleAuthentication"); //$NON-NLS-1$

        // we obtain the last request and host from a possible redirection sequence
        HttpRequest lastRequest = httpContext.getRequest();
        HttpHost lastHost = httpContext.getTargetHost();

        CloseableHttpResponse response = method.getResponse();
        int statusCode = method.getResponse().getStatusLine().getStatusCode();
        _logger.debug("Replay initially set to false"); //$NON-NLS-1$
        boolean replay = false; // by default we do not replay any requests

        // we may have encountered a (Jazz) container FORM challenge
        if (statusCode == 200 && hasAuthRequiredHeader(response)) {
            _logger.debug("Handling form-based authentication"); //$NON-NLS-1$

            HttpClientUtils.closeQuietly(response);
            AuthenticationException exp;
            switch (this._authenticationType) {
            case WINDOWSINTEGRATED:
                // we are dealing with a form challenge, but we are trying to use WINDOWSINTEGRATED, we should indicate that the server is not set up for SPNEGO
                exp = new WindowsLoginException(
                		NLS.bind(Messages.getClientString("ClientHttpUtil.FailedWinAuth"), //$NON-NLS-1$ 
                				method.getRequest().getURI().toString(),
                				getLoggedInUserCredentials()));

                exp.setRequestUri(getAppropriateRequestURL(httpContext));
                exp.addHeaders(getHttp3ResponseHeaders(httpContext));
                throw exp;
            case KERBEROSSPNEGO:
                // we are dealing with a form challenge, but we are trying to use SPNEGO, we should indicate that the server is not set up for SPNEGO
                exp = new NoKerberosTargetException(
                		NLS.bind(Messages.getClientString(NLS_NO_TARGET_KERBEROS), 
                				method.getRequest().getURI().toString()));
                exp.setRequestUri(getAppropriateRequestURL(httpContext));
                exp.addHeaders(getHttp3ResponseHeaders(httpContext));
                throw exp;

            case CLIENTCERTS:
                exp = new ClientCertificateNotAcceptedException(NLS.bind(Messages.getClientString(NLS_CLIENTCERTIFICATE_NOT_ACCEPTED), method.getRequest()
                        .getURI().toString()));
                exp.setRequestUri(getAppropriateRequestURL(httpContext));
                exp.addHeaders(getHttp3ResponseHeaders(httpContext));
                throw exp;
            default:
                // we are obviously fine if we are trying to use user creds
                break;
            }

            // Use the last request and host to do form-based auth, since it's the one that returned the challenge.
            String lastPath = getContextPath(lastRequest);
            AuthScope scope = new AuthScope(_targetHost.getHostName(), AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthScope.ANY_SCHEME);
            UsernamePasswordCredentials credentials = (UsernamePasswordCredentials) getCredsProvider().getCredentials(scope);
            formBasedAuthenticate(lastPath, lastHost, getHttpClient(), credentials, httpContext);

            // if we are dealing with tomcat and we have a successful form-based authentication, we re-execute the last request which will
            // lead to a redirect to the protected resource. This obviously happens as a GET. We can absorb that response and a subsequent
            // replay will then perform the original method correctly.

            // NOTE: We could also ignore this aspect and let the replay logic do its job. The first replay would lead to a situation where
            // the POST ends up being redirected again to a GET. Then another replay is executed which eventually does the right thing.
            boolean isTomcat = isTomcat(response);
            if (isTomcat) {
                _logger.debug("Running on tomcat requires us to re-execute the last request %s" + lastRequest.getRequestLine().getUri()); //$NON-NLS-1$
                response = executePrimitiveRequest(lastRequest, lastHost, httpContext, wrapIOException, true);
                method.setResponse(response);
                HttpClientUtils.closeQuietly(response);
            }

            // we always force a replay after we have dealt with a form authentication
            _logger.debug("Setting replay to true"); //$NON-NLS-1$
            replay = true;
        }

        // we replay if we went through form auth, but we also replay if the last request was a GET but the original method was not (provided the last
        // status we're seeing is 2xx)
        String methodName = method.getRequest().getMethod();
        statusCode = response.getStatusLine().getStatusCode(); // obtain again
        boolean nonGet = methodName.equalsIgnoreCase("post") || methodName.equalsIgnoreCase("put") || methodName.equalsIgnoreCase("delete"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        _logger.debug("Last request was a nonGet: " + nonGet); //$NON-NLS-1$
        replay = replay || (nonGet && lastRequest.getRequestLine().getMethod().equalsIgnoreCase("get") && statusCode < 300); //$NON-NLS-1$
        _logger.debug("Setting replay to " + replay); //$NON-NLS-1$

        if (replay) {

            if (attempts > MAXIMUM_REPLAY_ATTEMPTS) {
                _logger.debug("Replaying has reached more than maximum attempts: " + attempts); //$NON-NLS-1$
                // this should really no occur and will be treated as an internal error
                throw new TeamServiceException(
                        "Replaying the original POST/PUT/DELETE request on a redirection sequence more than " + MAXIMUM_REPLAY_ATTEMPTS + " times"); //$NON-NLS-1$ //$NON-NLS-2$
            }

            // we need to check if the post was to oauth-authorize, which is exempt from replaying
            if (!isPostToOAuthAuthorize(method.getRequest())) {
                _logger.debug("Replaying the original POST/PUT/DELETE request on a redirection sequence, attempt " + attempts); //$NON-NLS-1$

                HttpClientUtils.closeQuietly(response);

                // because the replay may lead to a form challenge again, we execute the request again with possible authentication handling

                this.executeWithAuthHandling(method, httpContext, wrapIOException, attempts + 1);
                return;
            } else {
                _logger.debug("a post to oauth authorize is exempt from replaying. Request was %s", method.getRequest().getURI()); //$NON-NLS-1$
            }
        }

        _logger.debug("No further authentication required"); //$NON-NLS-1$
    }

    /**
     * Returns the name of the currently logged in user in the form of DOMAIN/USERNAME. May only be called 
     * when in {@link #_authenticationType}={@link AuthenticationType#WINDOWSINTEGRATED}. 
     * @return String user name.
     */
    static public String getLoggedInUserCredentials() {
    	String userName = null;
        String os = System.getProperty("os.name"); //$NON-NLS-1$
        os = os != null ? os.toLowerCase(Locale.ROOT) : null;
        if (os != null && os.contains("windows")) { //$NON-NLS-1$
	    	userName = System.getenv("USERNAME"); //$NON-NLS-1$
			String userDomain = System.getenv("USERDOMAIN"); //$NON-NLS-1$
			userName = String.format( "%s\\%s", userDomain == null ? UNKNOWN : userDomain, userName == null ? UNKNOWN : userName ); //$NON-NLS-1$
        } else {
        	userName = System.getenv("USER"); //$NON-NLS-1$
        	userName = userName == null ? UNKNOWN : userName;
        }
		return userName;
	}

	private static boolean isPostToOAuthAuthorize(HttpRequest request) {
        RequestLine line = request.getRequestLine();
        boolean isPost = line.getMethod().equalsIgnoreCase("post"); //$NON-NLS-1$
        if (isPost) {
            URI requestURI = URI.create(line.getUri());
            String path = requestURI.getPath();
            int indexOfSecondSlash = path.indexOf('/', 1);
            if (indexOfSecondSlash > 0) {
                // if there is one, we check that it is oauth-authorize
                String servletPath = path.substring(indexOfSecondSlash);
                return servletPath.toLowerCase(Locale.ENGLISH).startsWith(OAUTH_AUTHORIZE_PREFIX);
            }
        }

        return false;
    }

    private static final String OAUTH_AUTHORIZE_PREFIX = "/oauth-authorize"; //$NON-NLS-1$

    /**
     * Return the context path for a request URI (that is, the first segment of the request path).
     */
    private String getContextPath(HttpRequest request) {
        URI requestURI = URI.create(request.getRequestLine().getUri());
        String contextPath = requestURI.getPath();
        int index = contextPath.indexOf('/', 1);
        if (index > 0) {
            contextPath = contextPath.substring(0, index);
        }

        return contextPath;
    }

    /**
     * Returns whether or not the specified response originates from a Tomcat server.
     * 
     * @param response
     *            The response to inspect. Must not be <code>null</code>.
     * @return <code>true</code> if the response originated from a Tomcat server, <code>false</code> if it did not.
     */
    private boolean isTomcat(CloseableHttpResponse response) {
        boolean isTomcat = false;
        Header serverHeader = response.getFirstHeader("Server"); //$NON-NLS-1$
        if (serverHeader != null) {
            if (serverHeader.getValue().startsWith("Apache-Coyote")) //$NON-NLS-1$
                isTomcat = true;
        }

        return isTomcat;
    }

    private CloseableHttpResponse executePrimitiveRequest(HttpRequest request, HttpHost targetHost, HttpClientContext context, boolean wrapIOException,
            boolean addComponentInformation) throws TeamServiceException, IOException {
        _logger.debug("Entering executePrimitiveRequest"); //$NON-NLS-1$

        try {
            if (addComponentInformation) {
                addComponentConfigurationHeaders(request);
            }
            debug("PRIOR REQUEST EXECUTION CONTEXT", context); //$NON-NLS-1$

            // replace user-agent with one which indicates it can handle spnego
            Header userAgent = request.getFirstHeader(HTTP.USER_AGENT);
            if (userAgent != null) {
                String newUserAgent = userAgent.getValue() + SPNEGO_ENABLED;
                _logger.debug("Adjusted user agent to: " + newUserAgent); //$NON-NLS-1$
                request.setHeader(HTTP.USER_AGENT, newUserAgent);
            }

            CloseableHttpResponse response = getHttpClient().execute(targetHost, request, context);
            debug("AFTER REQUEST EXECUTION CONTEXT", context); //$NON-NLS-1$
            return response;
        }

        catch (ClientProtocolException e) {
            TeamServiceException ex = new TeamServiceException(httpErrorMessage(NLS_HTTP_ERROR_SERVICE, context, e), e);
            ex.addHeaders(getHttp3ResponseHeaders(context));
            ex.setRequestUri(getRequestUri(context));
            _logger.debug("executePrimitiveRequest - Throwing HttpException"); //$NON-NLS-1$
            throw ex;
        } catch (ConnectTimeoutException e) {
            ConnectionException ex = new ConnectionException(httpErrorMessage(NLS_HTTP_ERROR_TIMEOUT, context, e), e);
            ex.setRequestUri(getRequestUri(context));
            _logger.debug("executePrimitiveRequest - Throwing ConnectTimeoutException"); //$NON-NLS-1$
            throw ex;
        } catch (NoHttpResponseException e) {
            NoResponseException ex = new NoResponseException(httpErrorMessage(NLS_HTTP_ERROR_NO_RESPONSE, context, e), e);
            ex.setRequestUri(getRequestUri(context));
            _logger.debug("executePrimitiveRequest - Throwing NoHttpResponseException"); //$NON-NLS-1$
            throw ex;
        }

        catch (SocketException e) {
            ConnectionException c = new ConnectionException(httpErrorMessage(NLS_HTTP_ERROR_SOCKET, context, e), e);
            c.setRequestUri(getRequestUri(context));
            _logger.debug("executePrimitiveRequest - Throwing SocketException"); //$NON-NLS-1$
            throw c;
        }

        catch (UnresolvedAddressException e) {
            HostUnknownException ex = new HostUnknownException(httpErrorMessage(NLS_HTTP_ERROR_INVALID, context, e), e);
            ex.setRequestUri(getRequestUri(context));
            _logger.debug("executePrimitiveRequest - Throwing UnresolvedAddressException"); //$NON-NLS-1$
            throw ex;
        } catch (UnknownHostException e) {
            _logger.debug("Handling UnknownHostException"); //$NON-NLS-1$
            HostUnknownException ex = new HostUnknownException(httpErrorMessage(NLS_HTTP_ERROR_UNKNOWN_HOST, context, e), e);
            ex.setRequestUri(getRequestUri(context));
            _logger.debug("executePrimitiveRequest - Throwing UnknownHostException"); //$NON-NLS-1$
            throw ex;
        }

        // catch-all IOException
        catch (IOException e) {
            if (wrapIOException) {
                TeamServiceException ex = new TeamServiceException(httpErrorMessage(NLS_HTTP_ERROR, context, e), e);
                ex.setRequestUri(getRequestUri(context));
                _logger.debug("executePrimitiveRequest - Throwing TeamServiceException"); //$NON-NLS-1$
                throw ex;
            }
            _logger.debug("executePrimitiveRequest - Throwing catch-all IOException"); //$NON-NLS-1$
            throw e;
        }

    }

    /**
     * Returns the request URI encapsulated by the referenced context. It is assumed that the context has been used in at least one httpclient request
     * execution.
     * 
     * @param context
     *            The request context. Must not be <code>null</code>.
     * @return The request URI referenced by the context. Never <code>null</code>. Will be empty on parsing errors.
     */
    static private String getRequestUri(HttpContext context) {

        HttpClientContext clientContext = HttpClientContext.adapt(context);
        HttpRequest request = clientContext.getRequest();
        HttpHost host = clientContext.getTargetHost();

        URI requestUri;
        try {
            requestUri = URIUtils.rewriteURI(URI.create(request.getRequestLine().getUri()), host);
        } catch (URISyntaxException e) {
            _logger.debug("Error determining request Uri: %s", e.getMessage()); //$NON-NLS-1$
            return EMPTY_STRING;
        }
        return requestUri.toString();
    }

    /**
     * Returns the response headers encapsulated in the referenced http context, in HttpClient v3 form.
     */
    static private org.apache.commons.httpclient.Header[] getHttp3ResponseHeaders(HttpContext context) {

        HttpClientContext clientContext = HttpClientContext.adapt(context);
        HttpResponse response = clientContext.getResponse();

        if (response == null)
            return null;

        Header[] headers = response.getAllHeaders();
        if (headers == null)
            return null;

        org.apache.commons.httpclient.Header[] newHeaders = new org.apache.commons.httpclient.Header[headers.length];
        for (int i = 0; i < headers.length; i++) {
            newHeaders[i] = new org.apache.commons.httpclient.Header(headers[i].getName(), headers[i].getValue());
        }
        return newHeaders;
    }

    /**
     * Formats an error message including the URI referenced by the http client method, the exception method and the message for the user.
     */
    private String httpErrorMessage(String messageKey, HttpContext context, Exception e) {
        HttpClientContext clientContext = HttpClientContext.adapt(context);
        String msg = e.getMessage() == null ? e.getClass().getName() : e.getMessage();

        String uri = EMPTY_STRING;
        HttpRequest request = clientContext.getRequest();
        if (request != null)
            uri = request.getRequestLine().getUri();

        String s = NLS.bind(Messages.getServerString(messageKey), msg, uri);
        return s;
    }

    /**
     * Adds component version information to the referenced <code>request</code> if that version information is available and not already present.
     */
    private static void addComponentConfigurationHeaders(HttpRequest request) {
        if (request == null) {
            _logger.debug("addComponentConfigurations - null method argument"); //$NON-NLS-1$
            return;
        }

        Header requestHeader = request.getFirstHeader(HttpUtil.X_COMPONENT_CONFIGURATION_VERSIONS);
        if (requestHeader != null) {
            _logger.debug("addComponentConfigurations - component configuration header present"); //$NON-NLS-1$
            return;
        }

        List<ComponentConfiguration> componentConfigurations =
                ComponentConfigurationRegistry.INSTANCE.getComponentConfigurations();
        if (componentConfigurations.size() < 1) {
            _logger.debug("addComponentConfigurations - null component configuration"); //$NON-NLS-1$
            return;
        }

        String headerValue = ComponentConfigurationRegistry.formatConfigurationsForHttpHeader(componentConfigurations);
        request.setHeader(HttpUtil.X_COMPONENT_CONFIGURATION_VERSIONS, headerValue);

        _logger.debug("addComponentConfigurations - component configuration header added"); //$NON-NLS-1$
    }

    /**
     * Returns the proxy URI if present, else the original request URI.
     */
    private String getAppropriateRequestURL(HttpContext context) {
        URI u = URI.create(getRequestUri(context));

        if (!isProxyUri(u))
            return u.toString();

        HttpResponse response = HttpClientContext.adapt(context).getResponse();
        if (response == null)
            return u.toString();

        /*
         * In 401 situations the presence of the WEBUI proxy header means that the 401 originates from the target, and not the proxy itself. Thus if we have a
         * 401, but no proxy header, then return the proxy URI, and not the target.
         */
        if (response.getStatusLine().getStatusCode() == 401) {
            Header[] headers = response.getHeaders(FROM_WEBUI_PROXY_HEADER);
            if (headers == null || headers.length == 0)
                return u.toString();
        }

        Map<String, String[]> parms = UriUtil.parseQueryParameters(u.getQuery());
        String[] uriParm = parms.get(PROXY_URI_PARM);
        if (uriParm == null || uriParm.length == 0)
            return u.toString();

        return uriParm[0];
    }

    /**
     * Returns whether or not the specified uri references the proxy that this client can talk to.
     */
    private boolean isProxyUri(URI requestUri) {
        URI proxyUri = null;
        try {
            proxyUri = new URI(buildProxyUri());
        } catch (URISyntaxException e) {
            _logger.error("Error resolving proxy URI: %s", e.getMessage()); //$NON-NLS-1$
            return false;
        }

        if (!HostUtil.authorityEquiv(requestUri, proxyUri)) {
            return false;
        }

        if (UriUtil.startsWithPath(proxyUri, requestUri)) {
            return true;
        }

        return false;
    }

    /**
     * Builds a proxy URI root from the referenced team server's repository URI.
     */
    private String buildProxyUri() {
        String repoUri = this.getRepositoryURL();
        String proxyUri = repoUri + PROXY_CONTEXT_SEGMENT;

        return proxyUri;
    }

    /**
     * Formats and throws an exception for HTTP status codes >= 300.
     */
    @SuppressWarnings("deprecation")
    private void throwHttpClientError(HttpContext context, boolean decodeExceptions) throws TeamServiceException {

        HttpClientContext clientContext = HttpClientContext.adapt(context);
        HttpResponse response = clientContext.getResponse();
        RemoteTeamService service = (RemoteTeamService) clientContext.getAttribute(SERVICE_CONTEXT);

        String decodedStatusText = URLDecoder.decode(response.getStatusLine().getReasonPhrase());
        int statusCode = response.getStatusLine().getStatusCode();

        String message = null;
        if (service == null) {
            String template = Messages.getServerString(NLS_REMOTE_HTTP_ERROR);
            message = NLS.bind(template, statusCode, decodedStatusText);
        } else {
            String template = Messages.getServerString(NLS_GENERAL_ERROR);
            message = NLS.bind(template, service.toString(), statusCode, decodedStatusText);
        }

        TeamServiceException exp = new TeamServiceException(message, statusCode);
        exp.setRequestUri(getAppropriateRequestURL(context));
        exp.addHeaders(getHttp3ResponseHeaders(clientContext));

        // Append exception in response body, if there is one
        if (decodeExceptions) {
            if (service == null) {
                appendReadException(exp, context);
            } else {
                appendNestedException(exp, context);
            }
        }

        Throwable cause = exp.getCause();
        if (cause instanceof VersionMismatchException) {
            exp = new VersionMismatchException(cause.getMessage(), exp, statusCode);
        }

        throw exp;
    }

    /**
     * Examine the HttpMethod for a serialized exception in the response and add it as a cause to the given exception.
     */
    private static void appendReadException(TeamServiceException exp, HttpContext context) {

        HttpClientContext clientContext = HttpClientContext.adapt(context);
        HttpResponse response = clientContext.getResponse();

        Header responseHeader = response.getFirstHeader(HttpUtil.CONTENT_TYPE);

        // Get the content type and encoding
        String encoding = HttpUtil.CharsetEncoding.ISO8859.toString();
        if (responseHeader != null) {
            HeaderElement[] elements = responseHeader.getElements();

            if (elements.length != 1) {
                // We should only have the one element.
                return;
            }

            // If there's a content type other than JSON, bail.
            String type = elements[0].getName();
            if (!HttpUtil.MediaType.JSON.toString().equals(type)) {
                return;
            }

            // Guess the encoding.
            NameValuePair nvp = elements[0].getParameterByName("charset"); //$NON-NLS-1$
            if (nvp != null) {
                encoding = nvp.getValue();
            }
        }

        // Generate the exception
        Throwable cause;
        HttpEntity entity = null;
        try {
            // If there is no response body, just return
            entity = response.getEntity();
            if (entity == null)
                return;

            /*
             * Deserialize the body, and then reset it on the entity so that it can be re-read by consumers. This has to happen immediately, in case exceptions
             * are thrown, leaving a response that has already been consumed and no longer available.
             */
            byte[] body = EntityUtils.toByteArray(entity);
            response.setEntity(new ByteArrayEntity(body));

            Charset charset = Charset.forName(encoding);
            ByteArrayInputStream stream = new ByteArrayInputStream(body);
            Reader r = new InputStreamReader(stream, charset);

            // Parse the object into an exception
            IDeserializer2 deserializer = IDeserializer2.FACTORY.newInstance(MediaType.JSON, null);
            cause = deserializer.deserializeException(r, RemoteTeamServer.class.getClassLoader(), null);

        } catch (IllegalCharsetNameException e) {
            return;
        } catch (UnsupportedCharsetException e) {
            return;
        } catch (IOException e) {
            return;
        } catch (SerializeException e) {
            return;
        }

        // Assign the exception as the root cause
        Throwable walker = exp;
        while (walker.getCause() != null) {
            walker = walker.getCause();
        }

        walker.initCause(cause);
    }

    /**
     * Try to append the server side stack trace to the new failure exception.
     */
    private void appendNestedException(TeamServiceException exception, HttpContext context) {

        WebServicesMarshaller webServicesMarshaller = (WebServicesMarshaller)
                MarshalFactory.eINSTANCE.getMarshaller(MarshallerType.WEB_SERVICES_LITERAL);
        try {
            Envelope envelope = demarshallEnvelope(webServicesMarshaller, context);
            Body body = envelope.getBody();
            if (body != null) {
                Fault fault = body.getFault();
                if (fault != null) {
                    Throwable throwable = constructExceptionFromFault(fault, context);
                    if (throwable != null) {
                        exception.initCause(throwable);
                        exception.setFromServer(true);
                    }
                }
            }
        } catch (Throwable e) {
            _logger.debug("Exception thrown during the marshalling of appendNestedException: %s", e.getMessage()); //$NON-NLS-1$
        }
    }

    /**
     * Returns an exception constructed from a fault.
     */
    private Throwable constructExceptionFromFault(Fault fault, HttpContext context) {
        RemoteTeamService service = (RemoteTeamService) context.getAttribute(SERVICE_CONTEXT);
        Throwable result = MarshallerUtil.decodeFault(fault, service.getServiceClassLoader());
        return result;
    }

    /**
     * Demarshalls the response referenced by the context.
     */
    private static Envelope demarshallEnvelope(WebServicesMarshaller webServicesMarshaller, HttpContext context) throws IOException, MarshallingException {
        HttpClientContext clientContext = HttpClientContext.adapt(context);
        HttpResponse response = clientContext.getResponse();
        HttpRequest request = clientContext.getRequest();

        Header[] headers = response.getHeaders(HttpUtil.CONTENT_ENCODING);
        if (null == headers) {
            headers = new Header[0];
        }

        boolean gzipped = false;
        for (int i = 0; i < headers.length; i++) {
            if (headers[i].getValue().equalsIgnoreCase(HttpUtil.GZIP)) {
                gzipped = true;
            }
        }

        HttpEntity entity = response.getEntity();
        if (entity == null) {
            String message = NLS.bind(Messages.getServerString("ClientHttpUtil.EmptyResponse"), request.getRequestLine()); //$NON-NLS-1$
            throw new MarshallingException(MarshallingExceptionReasonCode.CANNOT_DEMARSHAL_LITERAL, message);
        }

        /*
         * Reset our entity to a byte array input stream so that its repeatable, once its been absorbed by the marshaller.
         */
        byte[] content = EntityUtils.toByteArray(entity);
        response.setEntity(new ByteArrayEntity(content));
        InputStream responseStream = new ByteArrayInputStream(content);

        if (gzipped) {
            responseStream = new GZIPInputStream(responseStream, 8192);
        }

        return (Envelope) webServicesMarshaller.demarshalInputStreamToObject(responseStream);
    }

    /**
     * Returns whether or not the response contained in the referenced request is from the target of the JFAM proxy, rather than the JFAM proxy itself.
     * 
     * @param request
     *            The request containing a request response. Must not <code>null</code>.
     * 
     * @return <code>true</code> if the response referenced by the specified request is for the target of the proxy, <code>false</code> if not, or if the
     *         request contains no response information.
     */
    private static boolean isResponseFromProxyTarget(HttpRequest request) {

        Header proxyHeader = request.getFirstHeader(VIA_PROXY_HEADER);
        boolean retVal = proxyHeader != null && Boolean.parseBoolean(proxyHeader.getValue());

        _logger.debug("isResponseFromProxyTarget - return = %s", retVal); //$NON-NLS-1$
        return retVal;
    }

    static protected boolean hasAuthRequiredHeader(HttpResponse response) {
        Header formAuthMsg = response.getFirstHeader(FORM_AUTH_HEADER);
        boolean retval = (formAuthMsg != null && formAuthMsg.getValue().equals(FORM_AUTH_REQUIRED_MSG));

        return retval;
    }

    /**
     * Send a form-based authentication request to the specified host/repository.
     * 
     * @param repoPath
     *            The path to the repository. Must not be <code>null</code>.
     * @param targetHost
     *            The repository host. Must not be <code>null</code>.
     * @param httpClient
     *            The httpclient to use. Must not be <code>null</code>.
     * @param storedCredentials
     *            The credentials to use. May be <code>null</code>.
     * @param context
     *            The shared http context. Must not be <code>null</code>.
     * @return The request used to successfully authenticate.
     * 
     * @throws AuthenticationException
     */
    private void formBasedAuthenticate(String repoPath, HttpHost targetHost, CloseableHttpClient httpClient,
            UsernamePasswordCredentials storedCredentials, HttpClientContext context) throws AuthenticationException
    {
        _logger.debug("Entering formBasedAuthenticate"); //$NON-NLS-1$

        final int expectedRedirectResponse = 302;
        final int expectedOkResponse = 200;

        if (!repoPath.endsWith(SLASH))
            repoPath += SLASH;

        String formAuthURI = repoPath + FORM_AUTH_URI;
        _logger.debug("formAuthURI: %s", formAuthURI); //$NON-NLS-1$
        HttpPost postMethod = new HttpPost(formAuthURI); // appScan: https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/269082

        if (storedCredentials == null)
            storedCredentials = new UsernamePasswordCredentials(EMPTY_STRING, EMPTY_STRING);

        List<NameValuePair> parms = new ArrayList<NameValuePair>();
        parms.add(new BasicNameValuePair(FORM_AUTH_USER_FIELD, storedCredentials.getUserName() != null ? storedCredentials.getUserName() : EMPTY_STRING));
        parms.add(new BasicNameValuePair(FORM_AUTH_PASSWORD_FIELD, storedCredentials.getPassword() != null ? storedCredentials.getPassword() : EMPTY_STRING));

        CloseableHttpResponse response = null;

        // HTTP status code
        int rc;

        try {
            /*
             * When the username and/or password contain non-ascii characters, we must explicitly set the charset to UTF-8 in the Content-Type header.
             * Otherwise, the server assumes ISO-8859-1.
             */
            UrlEncodedFormEntity entity = new UrlEncodedFormEntity(parms, Consts.UTF_8);
            postMethod.setEntity(entity);

            /*
             * Disable redirection for this one POST request
             */
            RequestConfig oldConfig = context.getRequestConfig();
            RequestConfig newConfig = getRequestConfigBuilder().setRedirectsEnabled(false).build();
            context.setRequestConfig(newConfig);

            try {
                debug("CONTEXT PRIOR TO EXECUTION: ", context); //$NON-NLS-1$
                response = httpClient.execute(targetHost, postMethod, context);
                debug("CONTEXT AFTER EXECUTION: ", context); //$NON-NLS-1$

                rc = response.getStatusLine().getStatusCode();
                _logger.debug("statusCode: %d", rc); //$NON-NLS-1$

            } finally {
                context.setRequestConfig(oldConfig);
            }

            Header authHeader = response.getFirstHeader(FORM_AUTH_HEADER);
            if (authHeader != null) {
                if (authHeader.getValue().equals(FORM_AUTH_REQUIRED_MSG)) {
                    throw new AuthenticationException(MessageFormat.format(Messages.getClientString(NLS_SERVER_CONFIG_ERROR),
                            FORM_AUTH_URI));
                } else if (authHeader.getValue().equals(FORM_AUTH_FAILED_MSG)) {
                    AuthenticationException exp = new InvalidUserCredentialsException(Messages.getClientString(NLS_INVALID_CREDENTIALS));
                    exp.setRequestUri(getAppropriateRequestURL(context));
                    exp.addHeaders(getHttp3ResponseHeaders(context));
                    throw exp;
                } else {
                    throw new AuthenticationException(MessageFormat.format(Messages.getClientString(NLS_SERVLET_ERROR),
                            FORM_AUTH_REQUIRED_MSG, FORM_AUTH_FAILED_MSG, authHeader.getValue()));
                }
            } else if (rc == expectedRedirectResponse) {
                String redirectURI = response.getFirstHeader(LOCATION).getValue();
                if (redirectURI.matches("^.*" + FORM_AUTH_FAILED_URI + ".*$")) { //$NON-NLS-1$ //$NON-NLS-2$
                    // Login failed
                    InvalidUserCredentialsException exp = new InvalidUserCredentialsException(Messages.getClientString(NLS_INVALID_CREDENTIALS));
                    exp.setRequestUri(getAppropriateRequestURL(context));
                    exp.addHeaders(getHttp3ResponseHeaders(context));
                    throw exp;
                }
            } else if (rc != expectedOkResponse) {
                // The form auth request should always respond with a 200 ok or 302 redirect code
                // Unexpected response: fail the login
                String msg = MessageFormat.format(Messages.getClientString(NLS_UNEXPECTED_STATUS), formAuthURI,
                        expectedRedirectResponse, rc);
                throw new AuthenticationException(msg);
            }
        } catch (ClientProtocolException e) {
            String msg = MessageFormat.format(Messages.getClientString(NLS_UNEXPECTED_LOGIN_ERROR), e.getMessage());
            _logger.trace("formBasedAuthenticate - throwing AuthenticationException - %s", msg); //$NON-NLS-1$
            throw new AuthenticationException(msg, e);
        } catch (IOException e) {
            String msg = MessageFormat.format(Messages.getClientString(NLS_UNEXPECTED_LOGIN_ERROR), e.getMessage());
            _logger.trace("formBasedAuthenticate - throwing AuthenticationException - %s", msg); //$NON-NLS-1$
            throw new AuthenticationException(msg, e);
        } finally {
            if (response != null) {
                HttpClientUtils.closeQuietly(response);
            }
        }
    }

    /**
     * This is needed for Tomcat because it responds with SC_BAD_REQUEST when the j_security_check URL is visited before an authenticated URL has been visited.
     * This same URL must also be visited after authenticating with j_security_check otherwise tomcat will not consider the session authenticated
     */
    public static int visitAuthenticatedURL(String repoPath, HttpHost targetHost, CloseableHttpClient httpClient, HttpContext context)
            throws TransportAuthException
    {
        _logger.debug("Entering visitAuthenticatedUrl: repoPath=%s; targetHost=%s", repoPath, targetHost); //$NON-NLS-1$

        if (!repoPath.endsWith(SLASH))
            repoPath += SLASH;
        HttpGet authenticatedURL = new HttpGet(repoPath + "authenticated/identity"); //$NON-NLS-1$ // appScan: https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/269082
        _logger.debug("Path: %s", authenticatedURL.getURI().getPath()); //$NON-NLS-1$

        CloseableHttpResponse response = null;
        int statusCode = 0;
        try {
            response = httpClient.execute(targetHost, authenticatedURL, context);
            statusCode = response.getStatusLine().getStatusCode();
        } catch (Exception e) {
            String msg = MessageFormat.format(Messages.getClientString(NLS_UNEXPECTED_LOGIN_ERROR), e.getMessage());
            _logger.trace("visitAuthenticatedURL - throwning TransportAuthException: %s", msg); //$NON-NLS-1$
            throw new TransportAuthException(msg, e);
        } finally {
            if (response != null) {
                HttpClientUtils.closeQuietly(response);
            }
        }

        _logger.debug("HTTP status code after touching \"%s\": %d", //$NON-NLS-1$ 
                authenticatedURL.getURI().getPath(), statusCode);

        return statusCode;
    }

    /**
     * Returns whether or not the referenced credentials are defined (non-null and non-empty).
     * 
     * @param storedCredentials
     *            The credentials. May be <code>null</code>.
     * 
     * @return <code>true</code> if the user/password is non-null/non-empty, otherwise <code>false</code>.
     */
    static protected boolean credDefined(UsernamePasswordCredentials storedCredentials) {
        String storedUsername = null;
        String storedPassword = null;
        if (storedCredentials != null) {
            storedUsername = storedCredentials.getUserName();
            storedPassword = storedCredentials.getPassword();
        }

        boolean retval = !isBlank(storedUsername) && !isBlank(storedPassword);
        return retval;
    }

    static private boolean isBlank(String s) {
        return s == null || s.trim().isEmpty();
    }

    // OIDC headers of relevance
    private static final String AUTHORIZATION_REDIRECT_URL_HEADER = "X-JSA-AUTHORIZATION-REDIRECT"; //$NON-NLS-1$
    private static final String AUTHORIZATION_LOGIN_REQUIRED_HEADER = "X-JSA-LOGIN-REQUIRED"; //$NON-NLS-1$

    // Additional HttpContext information
    private static final String SERVICE_CONTEXT = RemoteTeamServer.class.getCanonicalName() + ".TeamService"; //$NON-NLS-1$

    // The Web UI proxy segment and uri parm
    public static final String PROXY_URI_PARM = "uri"; //$NON-NLS-1$
    public static final String PROXY_CONTEXT_SEGMENT = "proxy"; //$NON-NLS-1$

    // Set on the response whenever the response is from the WebUI proxy, and not the target
    private static final String FROM_WEBUI_PROXY_HEADER = "X-net-jazz-ajax-proxy"; //$NON-NLS-1$

    // name of custom header that authentication messages are stored in
    public static final String FORM_AUTH_HEADER = "X-com-ibm-team-repository-web-auth-msg"; //$NON-NLS-1$

    // auth header value when authentication is required
    public static final String FORM_AUTH_REQUIRED_MSG = "authrequired"; //$NON-NLS-1$

    // auth header value when authentication is required
    public static final String FORM_AUTH_FAILED_MSG = "authfailed"; //$NON-NLS-1$

    // URI the non-OIDC server redirects to when authentication fails
    public static final String FORM_AUTH_FAILED_URI = "/auth/authfailed"; //$NON-NLS-1$

    // URI the non-OIDC server redirects to when authentication is required
    public static final String FORM_AUTH_REQUESTED_URI = "/auth/authrequired"; //$NON-NLS-1$

    // Whether or not the response is via the Jazz proxy
    private static final String VIA_PROXY_HEADER = "X-Via-Jazz-Proxy"; //$NON-NLS-1$

    // Message indices
    private static final String NLS_HTTP_ERROR = "ClientHttpUtil.HttpError"; //$NON-NLS-1$
    private static final String NLS_HTTP_ERROR_SERVICE = "ClientHttpUtil.HttpError.Service"; //$NON-NLS-1$
    private static final String NLS_HTTP_ERROR_TIMEOUT = "ClientHttpUtil.HttpError.Timeout"; //$NON-NLS-1$
    private static final String NLS_HTTP_ERROR_NO_RESPONSE = "ClientHttpUtil.HttpError.NoResponse"; //$NON-NLS-1$
    private static final String NLS_HTTP_ERROR_SOCKET = "ClientHttpUtil.HttpError.Socket"; //$NON-NLS-1$
    private static final String NLS_HTTP_ERROR_INVALID = "ClientHttpUtil.HttpError.Invalid"; //$NON-NLS-1$
    private static final String NLS_HTTP_ERROR_UNKNOWN_HOST = "ClientHttpUtil.HttpError.UnknownHost"; //$NON-NLS-1$   
    private static final String NLS_GENERAL_ERROR = "ClientHttpUtil.GeneralError"; //$NON-NLS-1$
    private static final String NLS_REMOTE_HTTP_ERROR = "ClientHttpUtil.RemoteHttpClientError"; //$NON-NLS-1$
    private static final String NLS_UNAVAILABLE_ERROR = "ClientHttpUtil.UnavailableError"; //$NON-NLS-1$
    private static final String NLS_NOT_IMPLEMENTED_ERROR = "ClientHttpUtil.NotImplementedError"; //$NON-NLS-1$
    private static final String NLS_FORBIDDEN_ERROR = "ClientHttpUtil.ForbiddenError"; //$NON-NLS-1$
    private static final String NLS_UNAUTHORIZED_ERROR = "ClientHttpUtil.UnauthorizedError"; //$NON-NLS-1$
    private static final String NLS_KERBEROS_TICKET_INVALID = "ClientHttpUtil.KerberosTicketInvalid"; //$NON-NLS-1$
    private static final String NLS_NO_TARGET_KERBEROS = "ClientHttpUtil.NoTargetKerberos"; //$NON-NLS-1$   
    private static final String NLS_USERCREDENTIALS_NOT_POSSIBLE = "ClientHttpUtil.UserCredentialsNotPossible"; //$NON-NLS-1$
    private static final String NLS_CLIENTCERTIFICATE_NOT_ACCEPTED = "ClientHttpUtil.ClientCertificateNotAccepted"; //$NON-NLS-1$

    // No retry header
    static public final String X_NO_RETRY_HEADER = "X-com-ibm-team-repository.common.remoteaccess.noRetry"; //$NON-NLS-1$

    // submit credentials to this URI for authentication
    private static final String FORM_AUTH_URI = "j_security_check"; //$NON-NLS-1$

    // name of the field to use when submitting the username
    private static final String FORM_AUTH_USER_FIELD = "j_username"; //$NON-NLS-1$

    // name of the field to use when submitting the password
    private static final String FORM_AUTH_PASSWORD_FIELD = "j_password"; //$NON-NLS-1$

    // HTTP constants
    public static final String FORM_AUTH = "FORM"; //$NON-NLS-1$
    public static final String BASIC_AUTH = "BASIC"; //$NON-NLS-1$
    public static final String LOCATION = "Location"; //$NON-NLS-1$
    public static final String CONTENT_TYPE_HEADER = "Content-Type"; //$NON-NLS-1$
    public static final String CONTENT_TYPE_FORM_URLENCODED = "application/x-www-form-urlencoded; charset=utf-8"; //$NON-NLS-1$

    public static final String SLASH = "/"; //$NON-NLS-1$

    private static final String NLS_UNEXPECTED_LOGIN_ERROR = "TransportAuthUtil.UnexpectedLoginError"; //$NON-NLS-1$
    private static final String NLS_UNEXPECTED_STATUS = "TransportAuthUtil.UnexpectedStatus"; //$NON-NLS-1$
    private static final String NLS_INVALID_CREDENTIALS = "TransportAuthUtil.InvalidCredentials"; //$NON-NLS-1$
    private static final String NLS_SERVLET_ERROR = "TransportAuthUtil.ServletError"; //$NON-NLS-1$
    private static final String NLS_SERVER_CONFIG_ERROR = "TransportAuthUtil.ServerConfigError"; //$NON-NLS-1$

	public static boolean isWindowsAuthAvailable() {
		Boolean checkDomain = Boolean.valueOf( System.getProperty( "integratedWinAuth.CheckDomain", "true" )); //$NON-NLS-1$ //$NON-NLS-2$
		if( checkDomain && System.getenv("USERDNSDOMAIN") == null ) //$NON-NLS-1$
			return false;
		return WinHttpClients.isWinAuthAvailable();
	}

}
