/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *
 */
package com.ibm.team.repository.transport.client;

import java.net.MalformedURLException;
import java.net.URL;

import com.ibm.team.repository.common.transport.HttpUtil;

/**
 * This class describes some connection-oriented aspects of a server.
 * The class encapsulates the following values:
 * <ul>
 * 
 * <dl>url</dl>
 * <dt>
 * </dt>
 * 
 * <dl>userid</dl>
 * <dt>
 * </dt>
 * 
 * <dl>password</dl>
 * <dt>
 * </dt>
 * 
 * </ul>
 */
public class RemoteTeamServerConfiguration {
	private String      url;
	private String      userid;
	private String      password;
	private int         socketTimeout;
	
	// derived from url, but cached
	private String      urlHost;
	private String      urlPath;
	private String      urlProtocol;
	private int         urlPort;

	/**
	 * Create a new instance of this class.
	 */
	public RemoteTeamServerConfiguration() {
		super();
		
		socketTimeout    = 3 * 60 * 1000; // 3 minutes
	}

	/**
	 * Return the password.
	 */
	public String getPassword() {
		return password;
	}
	
	/**
	 * Set the password.
	 */
	public void setPassword(String password) {
		if (null == password) throw new IllegalArgumentException("password must not be null"); //$NON-NLS-1$
		this.password = password;
	}
	
	/**
	 * Return the URL.
	 */
	public String getURL() {
		return url;
	}

	/**
	 * Return the hostname portion of the URL.
	 * Returns null if the URL is not set or is invalid.
	 */
	public String getURLHost() {
		return urlHost;
	}
	
	/**
	 * Return the port portion of the URL.
	 * Returns -1 if the URL is not set or is invalid.
	 */
	public int getURLPort() {
		return urlPort;
	}
	
	/**
	 * Return the protocol portion of the URL.
	 * Returns null if the URL is not set or is invalid.
	 */
	public String getURLProtocol() {
		return urlProtocol;
	}
	
	/**
	 * Return the path portion of the URL.
	 * Returns null if the URL is not set or is invalid.
	 */
	public String getURLPath() {
		return urlPath;
	}
	
	/**
	 * Set the URL.
	 */
	public void setURL(String url) {
		if (null == url) throw new IllegalArgumentException("url must not be null"); //$NON-NLS-1$
		
		if (!url.endsWith(HttpUtil.SLASH)) url += HttpUtil.SLASH;
		
		this.url    = url;
		urlHost     = null;
		urlPath     = null;
		urlProtocol = null;
		urlPort     = 0;
		
		URL theURL;
		try {
			theURL = new URL(url);
		}
		catch (MalformedURLException e) {
			return;
		}
		
		urlHost     = theURL.getHost();
		urlPath     = theURL.getPath();
		urlProtocol = theURL.getProtocol();
		urlPort     = theURL.getPort();
	}
	
	/**
	 * Return the userid.
	 */
	public String getUserid() {
		return userid;
	}
	
	/**
	 * Set the userid.
	 */
	public void setUserid(String userid) {
		if (null == userid) throw new IllegalArgumentException("userid must not be null"); //$NON-NLS-1$
		this.userid = userid;
	}
	
	
	/**
	 * Return the socket timeout in milliseconds.
	 * A value of zero indicates an infinite timeout.
	 */
	public int getSocketTimeout() {
		return socketTimeout;
	}
	
	/**
	 * Set the socket timeout to the specified number of millseconds.
	 * A value of zero indicates an infinite timeout.
	 */
	public void setSocketTimeout(int socketTimeout) {
		if (socketTimeout < 0) throw new IllegalArgumentException("argument must be >= 0"); //$NON-NLS-1$
		this.socketTimeout = socketTimeout;
	}
}
