/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.transport.client;

import org.eclipse.core.runtime.jobs.ISchedulingRule;

/**
 * Provides a scheduling rule that protects access to a repository.
 */
public class RepositoryJobSchedulingRule implements ISchedulingRule {

	public static ISchedulingRule getReadAccessRule(String uri) {
		return new RepositoryJobSchedulingRule(uri, true);
	}
	
	public static ISchedulingRule getWriteAccessRule(String uri) {
		return new RepositoryJobSchedulingRule(uri, false);
	}
	
	
	private final String uri;
	private final boolean write;

	private RepositoryJobSchedulingRule(String uri, boolean write) {
		if (uri == null)
			throw new IllegalArgumentException("The uri must not be null"); //$NON-NLS-1$
		
		this.uri = uri;
		this.write = write;
	}
	

	public boolean contains(ISchedulingRule rule) {
		// This is a requirement of the ISchedulingRule interface
		if (rule == this)
			return true;
		
		return false;
	}

	public boolean isConflicting(ISchedulingRule rule) {
		// This is a requirement of the ISchedulingRule interface
		if (rule == this)
			return true;
		
		if (rule instanceof RepositoryJobSchedulingRule) {
			RepositoryJobSchedulingRule repoRule = (RepositoryJobSchedulingRule)rule;
			
			if (repoRule.uri.equals(uri)) {
				// If either rule is write access, then they conflict with
				// each other
				return write || repoRule.write;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

}
