/*******************************************************************************
 * Licensed Materials - Property of IBM © Copyright IBM Corporation 2014.
 * All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

package com.ibm.team.repository.transport.client;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Locale;

import org.apache.commons.httpclient.HttpMethod;
import org.apache.http.HttpRequest;

import com.ibm.team.repository.common.transport.HttpUtil;

/**
 * Common remote access transport utilities.
 */
public class TransportUtils {

    /**
     * Adds in a Jazz specific HTTP header to tell the server what user ID the client
     * is expecting.  This special header is used in some testing envirionments and doesn't 
     * apply to a production server.
     * 
     * @param userId May be <code>null</code>.  If <code>null</code> this routine
     * returns immediately (no id added).
     *
     * @param method The method to add the header to. Must not be <code>null</code>.
     */
    public static void addUseridHeader(String userId, HttpRequest method) {
        if (null == userId) {
            return;
        }
        try {
            userId = URLEncoder.encode(userId, HttpUtil.CharsetEncoding.UTF8.name());
        } catch (UnsupportedEncodingException e) {
            // Should not happen.  If it does, just try the unencoded id.
        }
        method.addHeader(HttpUtil.X_USERID, userId);
    }
    
    /**
     * Adds in a Jazz specific HTTP header to tell the server what user ID the client
     * is expecting.  This special header is used in some testing envirionments and doesn't 
     * apply to a production server.
     * 
     * @param userId May be <code>null</code>.  If <code>null</code> this routine
     * returns immediately (no id added).
     *
     * @param method The method to add the header to. Must not be <code>null</code>.
     */
    public static void addUseridHeader(String userId, HttpMethod method) {
        if (null == userId) {
            return;
        }
        try {
            userId = URLEncoder.encode(userId, HttpUtil.CharsetEncoding.UTF8.name());
        } catch (UnsupportedEncodingException e) {
            // Should not happen.  If it does, just try the unencoded id.
        }
        method.addRequestHeader(HttpUtil.X_USERID, userId);
    }

    /**
     * Sets the accept-language header on the specified <code>method</code>.
     * Any previously set accept-language header will be overwritten.
     * 
     * @param method The method to set the header on. Must not be <code>null</code>.
     */
    static public void setAcceptLanguageHeader(HttpRequest method) {
        if (method == null)
            throw new IllegalArgumentException("method must not be null"); //$NON-NLS-1$

        Locale locale = Locale.getDefault();
        StringBuffer buffer = new StringBuffer(locale.getLanguage());
        String country = locale.getCountry();
        if (country.length() > 0) {
            buffer.append('-').append(country);
        }
        String variant = locale.getVariant();
        if (variant.length() > 0) {
            buffer.append('-').append(variant);
        }
        
        String headerVal = buffer.toString();
        method.setHeader(HttpUtil.ACCEPT_LANGUAGE, headerVal);
   }
    
}
