/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.transport.client;

import java.io.ByteArrayInputStream;
import java.net.URI;

import org.eclipse.core.runtime.Status;

import com.ibm.team.repository.common.transport.HttpUtil;
import com.ibm.team.repository.transport.client.ITeamRawRestServiceClient.IRawRestClientConnection;

public class TrustingOauthHandler extends OAuthHandler {

	private ITeamRawRestServiceClient restClient = null;

	public TrustingOauthHandler(ITeamRawRestServiceClient restClient){
		this.restClient = restClient;
	}

	@Override
	public IResponse authorize(String authorizeURL) {
        com.ibm.team.repository.transport.client.ITeamRawRestServiceClient.IRawRestClientConnection.Response returnResponse = null;        
        IRawRestClientConnection connection = null;
        try {
            connection = restClient.getConnection(new URI(authorizeURL));
            connection.addRequestHeader("Accept", "text/json"); //$NON-NLS-1$ //$NON-NLS-2$
            byte body[] = String.format("authorize=%s", Boolean.TRUE.toString()).getBytes("UTF-8"); //$NON-NLS-1$ //$NON-NLS-2$
            
            returnResponse = connection.doPost(new ByteArrayInputStream(body), body.length, "application/x-www-form-urlencoded"); //$NON-NLS-1$
        }            
        catch (Exception e) {
            if (connection != null) {
                com.ibm.team.repository.transport.client.ITeamRawRestServiceClient.IRawRestClientConnection.Response response = 
                        connection.getResponse();
                /*
                 * If we get a 302, then we expect that to be finalization of the 
                 * authorization procedure.  Do a GET on it.
                 */
                if (response != null && response.getStatusCode() == 302) {
                    try {
                        connection = restClient.getConnection(new URI(response.getResponseHeader(HttpUtil.LOCATION)));
                        response = connection.doGet();
                        if (response.getStatusCode() == 200)
                            return new Response(authorizeURL, Status.OK_STATUS);
                        else
                            return new Response(authorizeURL, Status.CANCEL_STATUS);
                    }
                    catch (Exception ex) {
                        e.printStackTrace();
                        return new Response(authorizeURL, Status.CANCEL_STATUS);
                    }
                }
            }
            e.printStackTrace();
            return null;
        }
        if (returnResponse.getStatusCode() != 200) {
            return new Response(authorizeURL, Status.CANCEL_STATUS);
        }
        return new Response(authorizeURL, Status.OK_STATUS);
    }

}
