/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.repository.transport.internal.nls;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import com.ibm.team.repository.common.util.CommonMessages;

public class Messages {
    private static final String BUNDLE_NAME = "com.ibm.team.repository.transport.internal.nls.messages"; //$NON-NLS-1$
    private static final ResourceBundle RESOURCE_BUNDLE = ResourceBundle.getBundle(Messages.BUNDLE_NAME);

    public static String getClientString(String key) {
        return CommonMessages.getString(Messages.class, Messages.BUNDLE_NAME, key);
    }
    
    public static String getServerString(String key) {
        try {
            return Messages.RESOURCE_BUNDLE.getString(key);
        } catch (MissingResourceException e) {
            Messages.checkForWrongKey(key);        	
            return '!' + key + '!';
        }
    }
    
    private static void checkForWrongKey(String key) {
    	String prefix = "_NoId."; //$NON-NLS-1$
    	boolean prefixExists = key.startsWith(prefix);
    	String newKey = prefixExists == true ? key.substring(prefix.length()) : prefix + key;
    	try {
    		Messages.RESOURCE_BUNDLE.getString(newKey);
           	String error = String.format("The message key \"%s\" is wrong.  It should be \"%s\".", key, newKey); //$NON-NLS-1$
            System.err.println(error);           	
           	throw new RuntimeException(error);
    	} catch (MissingResourceException e) {
    		// No-op
    	}
    }
    
    private Messages() {
    	super();
    }
}
